#include "assert.h"

#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"

#include "collision.h"
#include "auxiliar.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activemog2.h"
#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int surface_flags;


ActiveROOM::ActiveROOM(ROOM *r,ActiveMAP *pmap,ActiveMOG2 *pgame)
{
	back_sfc=0;
	collision_data=0;
	room=r;
	map=pmap;

	viewx=0;
	viewy=0;

    game=pgame;
    last_active_cycle=game->get_current_cycle();

	room_variables.Copy(r->room_variables);

	{
        int i;
		List<PLACED_OBJECT> l;
		PLACED_OBJECT *o;

        for(i=0;i<MAX_OBJECT_PLANES;i++) {
            l.Instance(r->objects[i]);
            l.Rewind();
			while(l.Iterate(o)) {
				if (!o->object->multiroom_flag ||
					(o->object->multiroom_flag && game->find_object(o->ID)==0)) {
//					if (i!=0 || !o->object->passive()) {
						ActiveOBJECT *ao;
						ao=new ActiveOBJECT(*o,this,0);
						
						objects[i].Add(ao);
						if (!ao->object->passive() ||
							 ao->object->constitution==CONS_SOLID_SURFACE) nonpassive_objects[i].Add(ao);
//					} /* if */ 
				} /* if */ 
            } /* while */ 
        } /* for */ 
	}

	if (r->room_action!=0) {
		room_action=new ActionSCRIPT(*r->room_action);
		room_action->init();
	} else {
		room_action=0;
	} /* if */ 
	script_foreign_object=0;

	collision_grid_new=new List<ActiveOBJECT>[room->COLLISION_GRID_SX*room->COLLISION_GRID_SY];
	collision_grid_old_x=new List<ActiveOBJECT>[room->COLLISION_GRID_SX*room->COLLISION_GRID_SY];
	collision_grid_old_y=new List<ActiveOBJECT>[room->COLLISION_GRID_SX*room->COLLISION_GRID_SY];
	collision_grid_old_xy=new List<ActiveOBJECT>[room->COLLISION_GRID_SX*room->COLLISION_GRID_SY];
        
} /* ActiveROOM::ActiveROOM */ 


ActiveROOM::~ActiveROOM(void)
{
	int i,j;

    map=0;
    game=0;
	if (back_sfc!=0) SDL_FreeSurface(back_sfc);
	back_sfc=0;
	if (collision_data!=0) MOG2_destroy_cmap(collision_data);
	collision_data=0;

	for(i=0;i<MAX_OBJECT_PLANES;i++) while(!nonpassive_objects[i].EmptyP()) nonpassive_objects[i].ExtractIni();
	while(!objects_to_change_plane.EmptyP()) objects_to_change_plane.ExtractIni();

	if (room_action!=0) delete room_action;
	room_action=0;

	for(i=0;i<room->COLLISION_GRID_SX;i++) {
		for(j=0;j<room->COLLISION_GRID_SY;j++) {
			while(!collision_grid_new[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_new[i+j*room->COLLISION_GRID_SX].ExtractIni();
			while(!collision_grid_old_x[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_old_x[i+j*room->COLLISION_GRID_SX].ExtractIni();
			while(!collision_grid_old_y[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_old_y[i+j*room->COLLISION_GRID_SX].ExtractIni();
			while(!collision_grid_old_xy[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_old_xy[i+j*room->COLLISION_GRID_SX].ExtractIni();
		} /* for */ 
	} /* for */ 
	delete []collision_grid_new;
	delete []collision_grid_old_x;
	delete []collision_grid_old_y;
	delete []collision_grid_old_xy;
} /* ActiveROOM::~ActiveROOM */ 


void ActiveROOM::get_collision_grid_area(int x1,int y1,int x2,int y2,int *gx1,int *gy1,int *gx2,int *gy2)
{
	float fx,fy;

	fx=float((room->get_dx()*TILESIZE)/room->COLLISION_GRID_SX);
	fy=float((room->get_dy()*TILESIZE)/room->COLLISION_GRID_SY);

	x1-=room->COLLISION_GRID_TOLERANCE;
	y1-=room->COLLISION_GRID_TOLERANCE;
	x2+=room->COLLISION_GRID_TOLERANCE;
	y2+=room->COLLISION_GRID_TOLERANCE;

	x1=int(x1/fx);
	x2=int(x2/fx);
	y1=int(y1/fy);
	y2=int(y2/fy);

	if (x1<0) x1=0;
	if (y1<0) y1=0;
	if (x2<0) x2=0;
	if (y2<0) y2=0;
	if (x1>=room->COLLISION_GRID_SX) x1=room->COLLISION_GRID_SX-1;
	if (y1>=room->COLLISION_GRID_SY) y1=room->COLLISION_GRID_SY-1;
	if (x2>=room->COLLISION_GRID_SX) x2=room->COLLISION_GRID_SX-1;
	if (y2>=room->COLLISION_GRID_SY) y2=room->COLLISION_GRID_SY-1;

	*gx1=x1;
	*gy1=y1;
	*gx2=x2;
	*gy2=y2;	
} /* ActiveROOM::get_collision_grid_area */ 


bool ActiveROOM::cycle(KEYBOARDSTATE **LVK,int n_LVKs)
{
	List<ActiveOBJECT> l,todelete[MAX_OBJECT_PLANES];
	ActiveOBJECT *o;
    int i;


	if (room->COLLISION_GRID_SX>1 || room->COLLISION_GRID_SY>1) {
		int i,j,k;
		int ox1,oy1,ox2,oy2;
		int gx1,gy1,gx2,gy2;

		for(i=0;i<room->COLLISION_GRID_SX;i++) {
			for(j=0;j<room->COLLISION_GRID_SY;j++) {
				while(!collision_grid_new[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_new[i+j*room->COLLISION_GRID_SX].ExtractIni();
				while(!collision_grid_old_x[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_old_x[i+j*room->COLLISION_GRID_SX].ExtractIni();
				while(!collision_grid_old_y[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_old_y[i+j*room->COLLISION_GRID_SX].ExtractIni();
				while(!collision_grid_old_xy[i+j*room->COLLISION_GRID_SX].EmptyP()) collision_grid_old_xy[i+j*room->COLLISION_GRID_SX].ExtractIni();
			} /* for */ 
		} /* for */ 

		for(i=0;i<MAX_OBJECT_PLANES;i++) {
			if (room->plane_h_speed[i]==100 && room->plane_v_speed[i]==100) {
				l.Instance(objects[i]);
				l.Rewind();
				while(l.Iterate(o)) {
					o->get_family_bbox(&ox1,&oy1,&ox2,&oy2);
					get_collision_grid_area(ox1,oy1,ox2,oy2,&gx1,&gy1,&gx2,&gy2);
					for(j=gx1;j<=gx2;j++) {
						for(k=gy1;k<=gy2;k++) {
							if (j==gx1) {
								if (k==gy1) {
									collision_grid_new[j+k*room->COLLISION_GRID_SX].Add(o);
								} else {
									collision_grid_old_y[j+k*room->COLLISION_GRID_SX].Add(o);
								} /* if */ 
							} else {
								if (k==gy1) {
										collision_grid_old_x[j+k*room->COLLISION_GRID_SX].Add(o);
									} else {
										collision_grid_old_xy[j+k*room->COLLISION_GRID_SX].Add(o);
									} /* if */ 
								} /* if */ 
						} /* for */ 
					} /* for */ 
				} /* while */ 
			} /* if */ 
		} /* for */ 
	} /* if */ 

    last_active_cycle=game->get_current_cycle();

	create_collision_mask();
	
	draw_commands.Delete();

	/* ROOM SCRIPT: */ 
	if (room_action!=0 &&
		room_action->cycle(0,this,map,game,0,game->app,&script_foreign_object,LOCATION_ROOM,LVK,n_LVKs)) {
		delete room_action;
		room_action=0;
	} /* if */ 

	/* OBJECT PRECYCLES: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(nonpassive_objects[i]);
        l.Rewind();
        while(l.Iterate(o)) o->precycle();
    } /* for */ 

	/* OBJECT CYCLES: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(nonpassive_objects[i]);
        l.Rewind();
        while(l.Iterate(o)) {
			o->cycle(LVK,n_LVKs);
			if (o->terminate) todelete[i].Add(o);
        } /* while */ 
    } /* for */ 

	/* DELETE TERMINATED OBJECTS: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        while(!todelete[i].EmptyP()) {
            ActiveOBJECT *o=todelete[i].ExtractIni();
            objects[i].DeleteElement(o);
			nonpassive_objects[i].DeleteElement(o);		
			game->deleted_objects=true;
			if (objects_to_change_plane.MemberRefP(o)) objects_to_change_plane.DeleteElement(o);
			game->remove_focusing_references(o);

			if (room->COLLISION_GRID_SX>1 || room->COLLISION_GRID_SY>1) {
				int j,k;
				int ox1,oy1,ox2,oy2;
				int gx1,gy1,gx2,gy2;
				o->get_family_bbox(&ox1,&oy1,&ox2,&oy2);
				get_collision_grid_area(ox1,oy1,ox2,oy2,&gx1,&gy1,&gx2,&gy2);
				for(j=gx1;j<=gx2;j++) {
					for(k=gy1;k<=gy2;k++) {
						collision_grid_new[j+k*room->COLLISION_GRID_SX].DeleteElement(o);		
						collision_grid_old_x[j+k*room->COLLISION_GRID_SX].DeleteElement(o);		
						collision_grid_old_y[j+k*room->COLLISION_GRID_SX].DeleteElement(o);		
						collision_grid_old_xy[j+k*room->COLLISION_GRID_SX].DeleteElement(o);		
					} /* for */ 
				} /* for */ 
			} /* if */ 

            delete o;
        } /* while */ 
    } /* for */ 

    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(objects[i]);
        l.Rewind();
        while(l.Iterate(o)) o->delete_terminated_children();
    } /* for */ 

	/* ADD THE NEW OBJECTS: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        while(!objects_toadd[i].EmptyP()) {
            o=objects_toadd[i].ExtractIni();
            objects[i].Add(o);
			if (!o->passive()) {
				nonpassive_objects[i].Add(o);
			} else {
				if (i==0 && back_sfc!=0) {
					/* Delete the back image since the background surface has to be recomputed: */ 
					SDL_FreeSurface(back_sfc);
					back_sfc=0;
				} /* if */ 
			} /* if */ 
        } /* while */ 
    } /* for */ 

	/* CHANGE OBJECTS OF PLANE: */ 
	while(!objects_to_change_plane.EmptyP()) {
		o=objects_to_change_plane.ExtractIni();
		int new_p=o->object_plane;
		o->object_plane=-1;
		o->get_object_plane();
		objects[o->object_plane].DeleteElement(o);
		objects[new_p].Add(o);

		if (!o->passive()) {
			nonpassive_objects[o->object_plane].DeleteElement(o);
			nonpassive_objects[new_p].Add(o);
		} /* if */ 
		o->object_plane=new_p;
	} /* whiel */ 
	
	return true;
} /* ActiveROOM::cycle */ 



bool ao_order_topdown(ActiveOBJECT *o1,ActiveOBJECT *o2) { return o1->get_y()<=o2->get_y(); };
bool ao_order_bottomup(ActiveOBJECT *o1,ActiveOBJECT *o2) { return o1->get_y()>=o2->get_y(); };
bool ao_order_leftright(ActiveOBJECT *o1,ActiveOBJECT *o2) { return o1->get_x()<=o2->get_x(); };
bool ao_order_rightleft(ActiveOBJECT *o1,ActiveOBJECT *o2) { return o1->get_x()>=o2->get_x(); };

extern bool t_order_topdown(PLACED_TILE *t1,PLACED_TILE *t2);
extern bool t_order_bottomup(PLACED_TILE *t1,PLACED_TILE *t2);
extern bool t_order_leftright(PLACED_TILE *t1,PLACED_TILE *t2);
extern bool t_order_rightleft(PLACED_TILE *t1,PLACED_TILE *t2);

void ActiveROOM::draw(SDL_Surface *sfc,SDL_Rect vp,List<ActiveOBJECT> *focusing_objects,SDL_Rect *focusing_hs)
{
	int sizex=room->dx*TILESIZE;
	int sizey=room->dy*TILESIZE;

	if (back_sfc==0 ||
		back_sfc->w!=sizex ||
		back_sfc->h!=sizey) {

		/* If the room is TOO large, do NOT create a background surface: */ 
		if (sizex*sizey<800*600 && room->drawing_order==0) {

			/* The backgorund sfc doesn't have any APHA channel: */ 
			if (back_sfc!=0) SDL_FreeSurface(back_sfc);
			back_sfc=SDL_CreateRGBSurface(SDL_SRCALPHA|surface_flags,sizex,sizey,32,0,0,0,0);

			/* Draw the background: */ 
			SDL_FillRect(back_sfc,0,0);
			{
				List<ActiveOBJECT> l2;
				ActiveOBJECT *ao;
				List<PLACED_OBJECT> l;
				PLACED_OBJECT *o;
				List<PLACED_TILE> l1;
				PLACED_TILE *t;
				SDL_Rect r;

				l1.Instance(room->tiles[0]);
				l1.Rewind();
				while(l1.Iterate(t)) t->draw(0,0,back_sfc);

				/* Draw also Static objects with only 1 bitmap */ 
				l.Instance(room->objects[0]);
				l.Rewind();
				while(l.Iterate(o)) {
					if (o->object->passive()) o->draw(0,0,back_sfc);
				} /* while */ 

				/* The static objects that have been created after creating the room: */ 
				l2.Instance(objects[0]);
				l2.Rewind();
				while(l2.Iterate(ao)) {
					if (ao->object->passive()) {
						r.x=0;
						r.y=0;
						r.w=sizex;
						r.h=sizey;
						ao->draw(0,0,back_sfc,&r);
					} /* if */ 
				} /* while */ 
			}
		} /* if */ 
	} /* if */ 

	/* Compute the viewport: */ 
	if (!focusing_objects->EmptyP()) {
		int x=0,y=0;
		int no=0;
		int lx,ly;
		int tx,ty;
		List<ActiveOBJECT> fl;
		ActiveOBJECT *fo;
	
		fl.Instance(*focusing_objects);
		fl.Rewind();
		while(fl.Iterate(fo)) {
			if (fo->room==this) {
				fo->hotspot_coordinates(&lx,&ly);
				if (no>0 && room->horizontal_circularity) {
					int dif=abs((x/no)-lx);
					tx=lx;
					if (abs((x/no)-(lx+room->get_dx()*TILESIZE))<dif) {
						tx=lx+room->get_dx()*TILESIZE;
						dif=abs((x/no)-tx);
					} /* if */ 
					if (abs((x/no)-(lx-room->get_dx()*TILESIZE))<dif) {
						tx=lx-room->get_dx()*TILESIZE;
					} /* if */ 
					lx=tx;
				} /* if */ 
				if (no>0 && room->vertical_circularity) {
					int dif=abs((y/no)-ly);
					ty=ly;
					if (abs((y/no)-(ly+room->get_dy()*TILESIZE))<dif) {
						ty=ly+room->get_dy()*TILESIZE;
						dif=abs((y/no)-ty);
					} /* if */ 
					if (abs((y/no)-(ly-2*room->get_dy()*TILESIZE))<dif) {
						ty=ly-room->get_dy()*TILESIZE;
					} /* if */ 
					ly=ty;
				} /* if */ 
				x+=lx;
				y+=ly;
				no++;
			} /* if */ 
		} /* while */ 
		
		x/=no;
		y/=no;

		if (focusing_hs==0) {
			/* No FOCUSING_HS specified: */ 
			if (sizex>vp.w) {
				viewx=x-(vp.w/2);
				if (!room->horizontal_circularity) {
					if ((viewx+vp.w)>sizex) viewx=sizex-vp.w;
					if (viewx<0) viewx=0;
				} /* if */ 
			} else {
				viewx=(sizex/2)-(vp.w/2);
			} /* if */ 

			if (sizey>vp.h) {
				viewy=y-(vp.h/2);
				if (!room->vertical_circularity) {
					if ((viewy+vp.h)>sizey) viewy=sizey-vp.h;
					if (viewy<0) viewy=0;
				} /* if */ 
			} else {
				viewy=(sizey/2)-(vp.h/2);
			} /* if */ 
		} else {
			/* FOCUSING_HS specified: */ 
			SDL_Rect absolute_hs;

			absolute_hs.x=(focusing_hs->x*vp.w)/100;
			absolute_hs.y=(focusing_hs->y*vp.h)/100;
			absolute_hs.w=(focusing_hs->w*vp.w)/100;
			absolute_hs.h=(focusing_hs->h*vp.h)/100;
			if (sizex>vp.w) {
				if ((x-viewx)<absolute_hs.x) viewx=x-absolute_hs.x;
				if ((x-viewx)>(absolute_hs.x+absolute_hs.w)) viewx=x-(absolute_hs.x+absolute_hs.w);
				if (!room->horizontal_circularity) {
					if ((viewx+vp.w)>sizex) viewx=sizex-vp.w;
					if (viewx<0) viewx=0;
				} /* if */ 
			} else {
				viewx=-((vp.w/2)-(sizex/2));
			} /* if */ 

			if (sizey>vp.h) {
				if ((y-viewy)<absolute_hs.y) viewy=y-absolute_hs.y;
				if ((y-viewy)>(absolute_hs.y+absolute_hs.h)) viewy=y-(absolute_hs.y+absolute_hs.h);
				if (!room->vertical_circularity) {
					if ((viewy+vp.h)>sizey) viewy=sizey-vp.h;
					if (viewy<0) viewy=0;
				} /* if */ 
			} else {
				viewy=-((vp.h/2)-(sizey/2));
			} /* if */ 
		} /* if */ 
	} else {
		viewx=0;
		viewy=0;
	} /* if */ 

	/* Draw the room: */ 
	{
		SDL_Rect new_clip,room_rect;
		int i,j,itmp,jtmp;
		int x_min=viewx,x_max=viewx;
		int y_min=viewy,y_max=viewy;

		room_rect.x=vp.x-viewx;
		room_rect.y=vp.y-viewy;
		room_rect.w=room->get_dx()*TILESIZE;
		room_rect.h=room->get_dy()*TILESIZE;

		if (room->horizontal_circularity) {
			while(vp.x+x_min+room_rect.w>vp.x) {
				x_min-=room_rect.w;
			} /* while */ 
			while(vp.x+x_max<vp.x+vp.w) {
				x_max+=room_rect.w;
			} /* while */ 
		} /* if */ 

		if (room->vertical_circularity) {
			while(vp.y+y_min+room_rect.h>vp.y) {
				y_min-=room_rect.h;
			} /* while */ 
			while(vp.y+y_max<vp.y+vp.h) {
				y_max+=room_rect.h;
			} /* while */ 
		} /* if */ 

		for(i=x_min;i<=x_max;i+=room_rect.w) {
			for(j=y_min;j<=y_max;j+=room_rect.h) {
				room_rect.x=vp.x-i;
				room_rect.y=vp.y-j;
				new_clip.x=vp.x;
				new_clip.y=vp.y;
				new_clip.w=vp.w;
				new_clip.h=vp.h;
				itmp=i;
				jtmp=j;
				if (!room->horizontal_circularity &&
					new_clip.x<room_rect.x) {
					new_clip.w-=room_rect.x-new_clip.x;
					itmp+=room_rect.x-new_clip.x;
					new_clip.x=room_rect.x;
				} /* if */ 
				if (!room->vertical_circularity &&
					new_clip.y<room_rect.y) {
					new_clip.h-=room_rect.y-new_clip.y;
					jtmp+=room_rect.y-new_clip.y;
					new_clip.y=room_rect.y;
				} /* if */ 
				if (!room->horizontal_circularity &&
					new_clip.x+new_clip.w>room_rect.x+room_rect.w) new_clip.w=(room_rect.x+room_rect.w)-new_clip.x;
				if (!room->vertical_circularity &&
					new_clip.y+new_clip.h>room_rect.y+room_rect.h) new_clip.h=(room_rect.y+room_rect.h)-new_clip.y;

				draw_internal(sfc,new_clip,itmp,jtmp);
			} /* for */ 
		} /* for */ 
	}
} /* ActiveROOM::draw */ 


void ActiveROOM::draw_internal(SDL_Surface *sfc,SDL_Rect vp,int viewx,int viewy)
{
	SDL_Rect old_clip;

	SDL_GetClipRect(sfc,&old_clip);
	SDL_SetClipRect(sfc,&vp);

	/* Draw the room: */ 
	/* Background: */ 
	{
		List<ActiveOBJECT> l;
		ActiveOBJECT *o;
		int x,y;

		/* Compute plane offset: */ 
		{
			if (room->plane_h_align[0]==0) {
				/* LEFT: */ 
				x=(viewx*room->plane_h_speed[0])/100;
			} /* if */ 
			if (room->plane_h_align[0]==1) {
				/* CENTER: */ 
				x=(((room->get_dx()*TILESIZE)/2)-(vp.w/2))+
				   (/**/((viewx-( ((room->get_dx()*TILESIZE)/2)-(vp.w/2) ))*room->plane_h_speed[0])/**//100);
			} /* if */ 
			if (room->plane_h_align[0]==2) {
				/* RIGHT: */ 
				x=(room->get_dx()*TILESIZE-vp.w) + (((viewx - (room->get_dx()*TILESIZE-vp.w) )*room->plane_h_speed[0])/100);
			} /* if */ 
			if (room->plane_v_align[0]==0) {
				/* TOP: */ 
				y=(viewy*room->plane_v_speed[0])/100;
			} /* if */ 
			if (room->plane_v_align[0]==1) {
				/* CENTER: */ 
				y=(((room->get_dy()*TILESIZE)/2)-(vp.h/2))+
				   (/**/((viewy-( ((room->get_dy()*TILESIZE)/2)-(vp.h/2) ))*room->plane_v_speed[0])/**//100);
			} /* if */ 
			if (room->plane_v_align[0]==2) {
				/* BOTTOM: */ 
				y=(room->get_dy()*TILESIZE-vp.h) + (((viewy - (room->get_dy()*TILESIZE-vp.h) )*room->plane_v_speed[0])/100);
			} /* if */ 
		}

		x-=vp.x;
		y-=vp.y;

		if (back_sfc!=0) {
			SDL_Rect d;
			d.x=-x;
			d.y=-y;
			d.w=vp.w;
			d.h=vp.h;

			SDL_BlitSurface(back_sfc,0,sfc,&d);
			l.Instance(nonpassive_objects[0]);
			l.Rewind();
			while(l.Iterate(o)) {
				o->draw(-x,-y,sfc,&vp);
			} /* while */ 

		} else {
			List<PLACED_TILE> l1;
			PLACED_TILE *t;
			List<ActiveOBJECT> l2;
			ActiveOBJECT *o;

            l1.Instance(room->tiles[0]);
            l1.Rewind();
            l2.Instance(objects[0]);
            l2.Rewind();

			switch(room->drawing_order) {
			case 0:
				/* NONE: */ 
	            while(l1.Iterate(t)) t->draw(-x,-y,sfc,vp);
		        while(l2.Iterate(o)) o->draw(-x,-y,sfc,&vp);
				break;

			case 1:
				/* TOP-DOWN: */ 
				l1.Sort(&t_order_topdown);
				l2.Sort(&ao_order_topdown);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_y()<=l2.GetObj()->get_y())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 2:
				/* BOTOOM-UP: */ 
				l1.Sort(&t_order_bottomup);
				l2.Sort(&ao_order_bottomup);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_y()>=l2.GetObj()->get_y())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 3:
				/* LEFT-RIGHT: */ 
				l1.Sort(&t_order_leftright);
				l2.Sort(&ao_order_leftright);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_x()<=l2.GetObj()->get_x())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 4:
				/* RIGHT-LEFT: */ 
				l1.Sort(&t_order_rightleft);
				l2.Sort(&ao_order_rightleft);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_x()>=l2.GetObj()->get_x())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			} /* switch */ 

		} /* if */ 
	}

	/* Foreground: */ 
	{
        int i;
		List<PLACED_TILE> l1;
		PLACED_TILE *t;
		List<ActiveOBJECT> l2;
		ActiveOBJECT *o;

        for(i=1;i<MAX_OBJECT_PLANES;i++) {
			int x,y;

			/* Compute plane offset: */ 
			{
				if (room->plane_h_align[i]==0) {
					/* LEFT: */ 
					x=(viewx*room->plane_h_speed[i])/100;
				} /* if */ 
				if (room->plane_h_align[i]==1) {
					/* CENTER: */ 
					x=(((room->get_dx()*TILESIZE)/2)-(vp.w/2))+
					   (/**/((viewx-( ((room->get_dx()*TILESIZE)/2)-(vp.w/2) ))*room->plane_h_speed[i])/**//100);
				} /* if */ 
				if (room->plane_h_align[i]==2) {
					/* RIGHT: */ 
					x=(room->get_dx()*TILESIZE-vp.w) + (((viewx - (room->get_dx()*TILESIZE-vp.w) )*room->plane_h_speed[i])/100);
				} /* if */ 
				if (room->plane_v_align[i]==0) {
					/* TOP: */ 
					y=(viewy*room->plane_v_speed[i])/100;
				} /* if */ 
				if (room->plane_v_align[i]==1) {
					/* CENTER: */ 
					y=(((room->get_dy()*TILESIZE)/2)-(vp.h/2))+
					   (/**/((viewy-( ((room->get_dy()*TILESIZE)/2)-(vp.h/2) ))*room->plane_v_speed[i])/**//100);
				} /* if */ 
				if (room->plane_v_align[i]==2) {
					/* BOTTOM: */ 
					y=(room->get_dy()*TILESIZE-vp.h) + (((viewy - (room->get_dy()*TILESIZE-vp.h) )*room->plane_v_speed[i])/100);
				} /* if */ 
			}

			x-=vp.x;
			y-=vp.y;

            l1.Instance(room->tiles[i]);
            l1.Rewind();
            l2.Instance(objects[i]);
            l2.Rewind();

			switch(room->drawing_order) {
			case 0:
				/* NONE: */ 
	            while(l1.Iterate(t)) t->draw(-x,-y,sfc,vp);
		        while(l2.Iterate(o)) o->draw(-x,-y,sfc,&vp);
				break;

			case 1:
				/* TOP-DOWN: */ 
				l1.Sort(&t_order_topdown);
				l2.Sort(&ao_order_topdown);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_y()<=l2.GetObj()->get_y())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 2:
				/* BOTOOM-UP: */ 
				l1.Sort(&t_order_bottomup);
				l2.Sort(&ao_order_bottomup);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_y()>=l2.GetObj()->get_y())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 3:
				/* LEFT-RIGHT: */ 
				l1.Sort(&t_order_leftright);
				l2.Sort(&ao_order_leftright);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_x()<=l2.GetObj()->get_x())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 4:
				/* RIGHT-LEFT: */ 
				l1.Sort(&t_order_rightleft);
				l2.Sort(&ao_order_rightleft);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_x()>=l2.GetObj()->get_x())) {
						l1.GetObj()->draw(-x,-y,sfc,vp);
						l1.Next();
					} else {
						l2.GetObj()->draw(-x,-y,sfc,&vp);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			} /* switch */ 
        } /* for */ 
	}

	/* Extra drawing elements: */ 
	{
		List<DRAWCOMMAND> l;
		DRAWCOMMAND *dc;

		SDL_LockSurface(sfc);
		l.Instance(draw_commands);
		l.Rewind();
		while(l.Iterate(dc)) {
			switch(dc->command) {
			case 0:
				putpixel(sfc,dc->x1-viewx,dc->y1-viewy,SDL_MapRGBA(sfc->format,dc->r,dc->g,dc->b,dc->a));
				break;
			case 1:
				draw_line(sfc,dc->x1-viewx,dc->y1-viewy,dc->x2-viewx,dc->y2-viewy,SDL_MapRGBA(sfc->format,dc->r,dc->g,dc->b,dc->a));
				break;
			case 2:
				{
					SDL_Rect r;

					r.x=dc->x1-viewx;
					r.y=dc->y1-viewy;
					r.w=dc->x2-dc->x1;
					r.h=dc->y2-dc->y1;
					if (dc->x2>dc->x1 && dc->y2>dc->y1) {
						SDL_FillRect(sfc,&r,SDL_MapRGBA(sfc->format,dc->r,dc->g,dc->b,dc->a));
					} /* if */ 
				}
				break;
			case 3:
				{
					TILE *t;

					t=game->active_game->tiles[(dc->x2)%TILECATEGORIES][dc->y2];

					SDL_UnlockSurface(sfc);
					if (t!=0) t->draw(dc->x1-viewx,dc->y1-viewy,sfc);
					SDL_LockSurface(sfc);
				}
				break;
			} /* switch */ 
		} /* while */ 
		SDL_UnlockSurface(sfc);
	}

	SDL_SetClipRect(sfc,&old_clip);
} /* ActiveROOM::draw_internal */ 


int ActiveROOM::room_object_collision(ActiveOBJECT *o,bool hierarchical)
{
	return room_object_collision(o,o->x,o->y,hierarchical);
} /* ActiveROOM::room_object_collision */ 


int ActiveROOM::room_object_collision(ActiveOBJECT *o,int x,int y,bool hierarchical)
{
	List<ActiveOBJECT> l1;
	ActiveOBJECT *o2;
	int i,j;
	bool found;
	int collision_flags=o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE;
	int tmp_res=0;


	assert(o->object->ntiles>0);

	for(found=false,j=0;j<o->object->nparts;j++) {
		if (o->tile[j]>=0) found=true;
	} /* for */ 
	if (!found) return 0;

	/* Collision with the background: */ 
	if (collision_data==0) create_collision_mask();

	if (o->low_level_collision(x,y,0,0,collision_data,collision_flags,hierarchical)) return CONS_SOLID;

	/* Collision with other objects: */ 
	if (room->COLLISION_GRID_SX>1 || room->COLLISION_GRID_SY>1) {
		int i,j;
		int ox1,oy1,ox2,oy2;
		int gx1,gy1,gx2,gy2;

		o->get_family_bbox(&ox1,&oy1,&ox2,&oy2);
		get_collision_grid_area(ox1,oy1,ox2,oy2,&gx1,&gy1,&gx2,&gy2);

		for(i=gx1;i<=gx2;i++) {
			for(j=gy1;j<=gy2;j++) {
				if (i==gx1 && j==gy1) {
					/* old_xy */ 
					l1.Instance(collision_grid_old_xy[i+j*room->COLLISION_GRID_SX]);
					l1.Rewind();
					while(l1.Iterate(o2)) {
						if ((!o2->object->passive() ||
							 o2->object->constitution==CONS_SOLID_SURFACE) &&
							o!=o2 && !o->other_instances.MemberRefP(o2)) {
							if (o->collision(x,y,o2,collision_flags,CONS_SOLID,hierarchical)) tmp_res|=CONS_SOLID;
							if (o->collision(x,y,o2,collision_flags,CONS_SOLID_SURFACE,hierarchical)) tmp_res|=CONS_SOLID_SURFACE;
							if ((tmp_res&CONS_SOLID)!=0) return tmp_res;
						} /* if */ 
					} /* while */ 
				} /* if */ 

				if (i==gx1) {
					/* old_x */ 
					l1.Instance(collision_grid_old_x[i+j*room->COLLISION_GRID_SX]);
					l1.Rewind();
					while(l1.Iterate(o2)) {
						if ((!o2->object->passive() ||
							 o2->object->constitution==CONS_SOLID_SURFACE) &&
							o!=o2 && !o->other_instances.MemberRefP(o2)) {
							if (o->collision(x,y,o2,collision_flags,CONS_SOLID,hierarchical)) tmp_res|=CONS_SOLID;
							if (o->collision(x,y,o2,collision_flags,CONS_SOLID_SURFACE,hierarchical)) tmp_res|=CONS_SOLID_SURFACE;
							if ((tmp_res&CONS_SOLID)!=0) return tmp_res;
						} /* if */ 
					} /* while */ 
				} /* if */ 

				if (j==gy1) {
					/* old_y */ 
					l1.Instance(collision_grid_old_y[i+j*room->COLLISION_GRID_SX]);
					l1.Rewind();
					while(l1.Iterate(o2)) {
						if ((!o2->object->passive() ||
							 o2->object->constitution==CONS_SOLID_SURFACE) &&
							o!=o2 && !o->other_instances.MemberRefP(o2)) {
							if (o->collision(x,y,o2,collision_flags,CONS_SOLID,hierarchical)) tmp_res|=CONS_SOLID;
							if (o->collision(x,y,o2,collision_flags,CONS_SOLID_SURFACE,hierarchical)) tmp_res|=CONS_SOLID_SURFACE;
							if ((tmp_res&CONS_SOLID)!=0) return tmp_res;
						} /* if */ 
					} /* while */ 
				} /* if */ 

				l1.Instance(collision_grid_new[i+j*room->COLLISION_GRID_SX]);
				l1.Rewind();
				while(l1.Iterate(o2)) {
					if ((!o2->object->passive() ||
						 o2->object->constitution==CONS_SOLID_SURFACE) &&
						o!=o2 && !o->other_instances.MemberRefP(o2)) {
						if (o->collision(x,y,o2,collision_flags,CONS_SOLID,hierarchical)) tmp_res|=CONS_SOLID;
						if (o->collision(x,y,o2,collision_flags,CONS_SOLID_SURFACE,hierarchical)) tmp_res|=CONS_SOLID_SURFACE;
						if ((tmp_res&CONS_SOLID)!=0) return tmp_res;
					} /* if */ 
				} /* while */ 
			} /* for */ 
		} /* for */ 

	} else {
		for(i=0;i<MAX_OBJECT_PLANES;i++) {
			if (room->plane_h_speed[i]==100 && room->plane_v_speed[i]==100) {
				l1.Instance(nonpassive_objects[i]);
				l1.Rewind();
				while(l1.Iterate(o2)) {
					if (o!=o2 && !o->other_instances.MemberRefP(o2)) {
						if (o->collision(x,y,o2,collision_flags,CONS_SOLID,hierarchical)) tmp_res|=CONS_SOLID;
						if (o->collision(x,y,o2,collision_flags,CONS_SOLID_SURFACE,hierarchical)) tmp_res|=CONS_SOLID_SURFACE;
						if ((tmp_res&CONS_SOLID)!=0) return tmp_res;
					} /* if */ 
				} /* while */ 
			} /* if */ 
		} /* for */ 
	} /* if */ 

	return tmp_res;
} /* ActiveROOM::room_object_collision */ 


bool ActiveROOM::room_object_to_character_collision(ActiveOBJECT *o)
{
	return room_object_to_character_collision(o,o->x,o->y);
} /* ActiveROOM::room_object_to_character_collision */ 


bool ActiveROOM::room_object_to_character_collision(ActiveOBJECT *o,int x,int y)
{
        int i;
	List<ActiveOBJECT> l1;
	ActiveOBJECT *o2;
	int collision_flags=o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE;

	/* Collision with character objects: */ 
    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l1.Instance(objects[i]);
        l1.Rewind();
        while(l1.Iterate(o2)) {
                if (o!=o2 &&
                        o2->object->constitution==CONS_CHARACTER &&
                        o->collision(x,y,o2,collision_flags,CONS_ALL,true)) return true;
        } /* while */ 
    } /* for */ 

	return false;
} /* ActiveROOM::room_object_to_character_collision */ 


ActiveOBJECT *ActiveROOM::room_object_collision2(ActiveOBJECT *o,int x,int y,int constitution,ConditionSCRIPT *s,ActiveOBJECT **foreign_object,bool hierarchical)
{
    int i;
	List<ActiveOBJECT> l1;
	ActiveOBJECT *o2,*tmp;
	int collision_flags=o->object->constitution|CONS_SOLID|CONS_SOLID_SURFACE;


	/* Collision with other objects: */ 
	if (room->COLLISION_GRID_SX>1 || room->COLLISION_GRID_SY>1) {
		int i,j;
		int ox1,oy1,ox2,oy2;
		int gx1,gy1,gx2,gy2;
		o->get_family_bbox(&ox1,&oy1,&ox2,&oy2);
		get_collision_grid_area(ox1,oy1,ox2,oy2,&gx1,&gy1,&gx2,&gy2);

		for(i=gx1;i<=gx2;i++) {
			for(j=gy1;j<=gy2;j++) {

				if (i==gx1 && j==gy1) {
					/* old_xy */ 
					l1.Instance(collision_grid_old_xy[i+j*room->COLLISION_GRID_SX]);
					l1.Rewind();
					while(l1.Iterate(o2)) {
						if (o!=o2) {
							tmp=o->collision2(x,y,o2,collision_flags,constitution,s,foreign_object,hierarchical);
							if (tmp!=0) return tmp;
						} /* if */ 
					} /* while */ 
				} /* if */ 

				if (i==gx1) {
					/* old_x */ 
					l1.Instance(collision_grid_old_x[i+j*room->COLLISION_GRID_SX]);
					l1.Rewind();
					while(l1.Iterate(o2)) {
						if (o!=o2) {
							tmp=o->collision2(x,y,o2,collision_flags,constitution,s,foreign_object,hierarchical);
							if (tmp!=0) return tmp;
						} /* if */ 
					} /* while */ 
				} /* if */ 

				if (j==gy1) {
					/* old_y */ 
					l1.Instance(collision_grid_old_y[i+j*room->COLLISION_GRID_SX]);
					l1.Rewind();
					while(l1.Iterate(o2)) {
						if (o!=o2) {
							tmp=o->collision2(x,y,o2,collision_flags,constitution,s,foreign_object,hierarchical);
							if (tmp!=0) return tmp;
						} /* if */ 
					} /* while */ 
				} /* if */ 

				l1.Instance(collision_grid_new[i+j*room->COLLISION_GRID_SX]);
				l1.Rewind();
				while(l1.Iterate(o2)) {
					if (o!=o2) {
						tmp=o->collision2(x,y,o2,collision_flags,constitution,s,foreign_object,hierarchical);
						if (tmp!=0) return tmp;
					} /* if */ 
				} /* while */ 
			} /* for */ 
		} /* for */ 
	} else {
        for(i=0;i<MAX_OBJECT_PLANES;i++) {
			if (room->plane_h_speed[i]==100 && room->plane_v_speed[i]==100) {
				l1.Instance(objects[i]);
				l1.Rewind();
				while(l1.Iterate(o2)) {
					if (o!=o2) {
						tmp=o->collision2(x,y,o2,collision_flags,constitution,s,foreign_object,hierarchical);
						if (tmp!=0) return tmp;
					} /* if */ 
				} /* while */ 
			} /* if */ 
        } /* for */ 
	} /* if */ 

	return 0;
} /* ActiveROOM::room_object_collision2 */ 


void ActiveROOM::place_character_in_safe_place(ActiveOBJECT *o)
{
	int center_x,center_y;
	int new_x=0,new_y=0;
	int distance=-1;
	int found=false;
	int i,j,k;

	center_x=((room->dx/2)*TILESIZE);
	center_y=((room->dy/2)*TILESIZE);

	/* Look for another place if the character is over a wall: */ 
	create_collision_mask();
	for(i=0;i<room->dx;i++) {
		for(j=0;j<room->dy;j++) {
			if (!room_object_collision(o,i*TILESIZE,j*TILESIZE,true) &&
				!room_object_to_character_collision(o,i*TILESIZE,j*TILESIZE) &&
				room_object_collision(o,i*TILESIZE,(j+1)*TILESIZE,true)) {
				int d,d1,d2;
				int y=j*TILESIZE;
				for(k=0;k<TILESIZE;k++) {
					if (!room_object_collision(o,i*TILESIZE,j*TILESIZE+k,true)) {
						y=j*TILESIZE+k;
					} /* if */ 
				} /* for */ 

				found=true;
				d1=((i*TILESIZE)-center_x);
				if (d1<0) d1=-d1;
				d2=(y-center_y);
				if (d2<0) d2=-d2;
				d=d1+d2;
				if (distance==-1 ||
					d<distance) {
					new_x=i*TILESIZE;
					new_y=y;
					distance=d;
				} /* if */ 
			} /* if */ 
		} /* for */ 
	} /* for */ 

	if (!found) {
		/* Place character in the middle of the screen: */ 
		new_x=center_x;
		new_y=center_y;
	} /* if */ 

	
	o->x=new_x;
	o->y=new_y;

} /* ActiveROOM::place_character_in_safe_place */ 


void ActiveROOM::create_collision_mask(void)
{
	if (collision_data==0) {
        int i;
		SDL_Surface *tmp_sfc;
		List<PLACED_OBJECT> l;
		PLACED_OBJECT *o;
		List<PLACED_TILE> l1;
		PLACED_TILE *t;
		int dx=room->dx*TILESIZE;
		int dy=room->dy*TILESIZE;
		
		assert(dx>0);
		assert(dy>0);

		tmp_sfc=SDL_CreateRGBSurface(SDL_SRCALPHA|surface_flags,dx,dy,32,0,0,0,0);

		SDL_FillRect(tmp_sfc,0,0);

		for(i=0;i<MAX_OBJECT_PLANES;i++) {
			if (room->plane_h_speed[i]==100 && room->plane_v_speed[i]==100) {	
				l1.Instance(room->tiles[i]);
				l1.Rewind();
				while(l1.Iterate(t)) t->draw_collision_mask(0,0,tmp_sfc);

				/* Draw also passive objects: */ 
				l.Instance(room->objects[i]);
				l.Rewind();
				while(l.Iterate(o)) {
						if (o->object->passive() && 
							(o->object->constitution==CONS_SOLID)) o->draw_collision_mask(0,0,tmp_sfc);
				} /* while */ 
			} /* if */ 
		} /* for */ 
            
		collision_data=MOG2_make_cmap(tmp_sfc);

		SDL_FreeSurface(tmp_sfc);
	} /* if */ 
} /* ActiveROOM::create_collsion_mask */ 



bool ActiveROOM::find_object(ActiveOBJECT *object)
{
        int i;
	List<ActiveOBJECT> l;
	ActiveOBJECT *o;

    for(i=0;i<MAX_OBJECT_PLANES;i++) {
        l.Instance(objects[i]);
        l.Rewind();
        while(l.Iterate(o)) {
                if (o->offspringp(object)) return true;
        } /* while */ 
    } /* for */ 

	return false;
} /* ActiveROOM::find_object */ 



int ActiveROOM::get_variable_value(char *vname,int *ival,char **sval,ActiveOBJECT **oval)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(room_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) return v->get(ival,sval,oval);
	} /* while */ 
	return VALUE_NONE;
} /* ActiveROOM::get_variable_value */ 


void ActiveROOM::set_variable_value(char *vname,int value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(room_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveROOM::set_variable_value */ 


void ActiveROOM::set_variable_value(char *vname,char *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(room_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveROOM::set_variable_value */ 


void ActiveROOM::set_variable_value(char *vname,ActiveOBJECT *value)
{
	List<VARIABLE> l;
	VARIABLE *v;

	l.Instance(room_variables);
	l.Rewind();
	while(l.Iterate(v)) {
		if (v->cmp(vname)) {
			v->set(value);
			return;
		} /* if */ 
	} /* while */ 
} /* ActiveROOM::set_variable_value */ 


void ActiveROOM::remove_object_references(ActiveOBJECT *o)
{
	int i;

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		objects[i].DeleteElement(o);
		nonpassive_objects[i].DeleteElement(o);
	} /* for */ 
} /* ActiveROOM::remove_object_references */ 







/*

					// CIRCULARITIES: 
					if (room->horizontal_circularity) {
						int offs;

						offs=x;
						while(sizex-offs>0) offs+=sizex;
						while(offs>-vp.w) {
							// Do not draw if offs==x since it has been already drawn! 
							if (offs!=x) {
								l1.Rewind();
								l2.Rewind();
								while(l1.Iterate(t)) t->draw(vp.x-offs,vp.y-y,sfc,vp);
								while(l2.Iterate(o)) o->draw(vp.x-offs,vp.y-y,sfc,&vp);
							} // if  
							offs-=sizex;
						} // while 
					} // if  
					if (room->vertical_circularity) {
						int offs;

						offs=y;
						while(sizey-offs>0) offs+=sizey;
						while(offs>-vp.h) {
							// Do not draw if offs==x since it has been already drawn! 
							if (offs!=y) {
								l1.Rewind();
								l2.Rewind();
								while(l1.Iterate(t)) t->draw(vp.x-x,vp.y-offs,sfc,vp);
								while(l2.Iterate(o)) o->draw(vp.x-x,vp.y-offs,sfc,&vp);
							} // if  
							offs-=sizey;
						} // while 
					} // if 


*/


