#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#include <sys/time.h>
#include <time.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include "ctype.h"
#include "math.h"
#include "SDL.h"
#include "SDL_ttf.h"

#include "auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

#define SINCOS_TABLE_SIZE	4096

/* Tablas precalculadas: */ 
extern bool sincos_table;
extern float sin_table[SINCOS_TABLE_SIZE];
extern float cos_table[SINCOS_TABLE_SIZE];


void refraction_draw(SDL_Surface *orig,SDL_Rect r,SDL_Surface *refraction,float distance,float kh,float kv,float oh,float ov,float wave_height)
{
	int i,j;
	int ah,av;
	Uint32 *dest_ptr;
	float h1,h2,h3;
	float nx,ny,nz;
	float ox,oy;
	int final_x,final_y;
	int inc=SINCOS_TABLE_SIZE/64;
	float wave_normal_y=-5.0F;
	float inv_wave_normal_y=1.0F/wave_normal_y;

	if (wave_height==0) wave_height=100.0F;
	if (kh!=0) kh=SINCOS_TABLE_SIZE/kh;
	if (kv!=0) kv=SINCOS_TABLE_SIZE/kv;
	wave_normal_y=(-50.0F)/float(wave_height);
	inv_wave_normal_y=1.0F/wave_normal_y;

	if (!sincos_table) {
		for(i=0;i<SINCOS_TABLE_SIZE;i++) {
			sin_table[i]=float(sin((float(i)*(3.141592F*2.0F))/float(SINCOS_TABLE_SIZE)));
			cos_table[i]=float(cos((float(i)*(3.141592F*2.0F))/float(SINCOS_TABLE_SIZE)));
		} /* for */ 
		sincos_table=true;
	} /* if */ 


	for(j=0;j<refraction->h;j++) {
		dest_ptr=((Uint32* )((Uint8 *)refraction->pixels + j * refraction->pitch));

		for(i=0;i<refraction->w;i++) {
			// Clculo de la normal:
			// funcin: sin(kh*px+oh)*sin(kv*pz+ov)
			ah=int(kh*(i+oh))&0x0fff;
			av=int(kv*(j+ov))&0x0fff;

//			*dest_ptr=SDL_MapRGBA(refraction->format,int((sin_table[ah]*sin_table[av])*127)+127,0,0,255);
//			dest_ptr++;

			// compute normals:
			h1=sin_table[ah]*sin_table[av];
			h2=sin_table[(ah+inc)&0x0fff]*sin_table[av];
			h3=sin_table[ah]*sin_table[(av+inc)&0x0fff];
			nx=h2-h1;
			ny=wave_normal_y;
			nz=h3-h1;

//			n=1.0F/float(sqrt(nx*nx+ny*ny+nz*nz));
//			nx*=n; ny*=n; nz*=n;
			nx*=inv_wave_normal_y;
			ny*=inv_wave_normal_y;
			nz*=inv_wave_normal_y;

			// compute the offset of the pixel:
			ox=nx*distance;
			oy=nz*distance;

			final_x=int(r.x+i+ox);
			if (final_x<0) final_x=0;
			if (final_x>=orig->w) final_x=orig->w-1;
			final_y=int(r.y+j+oy);
			if (final_y<0) final_y=0;
			if (final_y>=orig->h) final_y=orig->h-1;

			*dest_ptr=*((Uint32* )((Uint8 *)orig->pixels + final_y * orig->pitch + (final_x << 2)));
			dest_ptr++;
		} /* for */ 
	} /* for */ 

} /* refraction_draw */ 