#include "assert.h"
#include "stdlib.h"
#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "actionscript.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


ROOM::ROOM(MAP *m)
{
	int i;

	map=m;
	x=NOPOSITION;
	y=NOPOSITION;
	dx=0;
	dy=0;
	used_object_planes=2;
	drawing_order=0;
	horizontal_circularity=false;
	vertical_circularity=false;
	room_action=new ActionSCRIPT();

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		plane_h_speed[i]=100;
		plane_v_speed[i]=100;
		plane_h_align[i]=0;
		plane_v_align[i]=0;
	} /* for */ 

	COLLISION_GRID_SX=16;
	COLLISION_GRID_SY=16;
	COLLISION_GRID_TOLERANCE=8;
} /* ROOM::ROOM */ 


ROOM::ROOM(MAP *m,int ndx,int ndy)
{
	int i;

	map=m;
	x=NOPOSITION;
	y=NOPOSITION;
	dx=ndx;
	dy=ndy;
	used_object_planes=2;
	drawing_order=0;
	horizontal_circularity=false;
	vertical_circularity=false;
	room_action=new ActionSCRIPT();

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		plane_h_speed[i]=100;
		plane_v_speed[i]=100;
		plane_h_align[i]=0;
		plane_v_align[i]=0;
	} /* for */ 

	COLLISION_GRID_SX=16;
	COLLISION_GRID_SY=16;
	COLLISION_GRID_TOLERANCE=8;
} /* ROOM::ROOM */ 


ROOM::ROOM(ROOM &r)
{
	int i;
	room_variables.Copy(r.room_variables);
	map=r.map;
	x=r.x;
	y=r.y;
	dx=r.dx;
	dy=r.dy;
	used_object_planes=r.used_object_planes;
	drawing_order=r.drawing_order;
	horizontal_circularity=r.horizontal_circularity;
	vertical_circularity=r.vertical_circularity;

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
	    tiles[i].Copy(r.tiles[i]);
	    objects[i].Copy(r.objects[i]);
	} /* for */ 

	if (r.room_action!=0) {
		room_action=new ActionSCRIPT(*r.room_action);
	} else {
		room_action=0;
	} /* if */ 

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		plane_h_speed[i]=r.plane_h_speed[i];
		plane_v_speed[i]=r.plane_v_speed[i];
		plane_h_align[i]=r.plane_h_align[i];
		plane_v_align[i]=r.plane_v_align[i];
	} /* for */ 

	COLLISION_GRID_SX=r.COLLISION_GRID_SX;
	COLLISION_GRID_SY=r.COLLISION_GRID_SY;
	COLLISION_GRID_TOLERANCE=r.COLLISION_GRID_TOLERANCE;
} /* ROOM::ROOM */ 


ROOM::~ROOM(void)
{
	if (room_action!=0) delete room_action;
	room_action=0;
} /* ROOM::~ROOM */ 


bool ROOM::save(FILE *fp,MOG2 *game)
{	
	int i;
	List<VARIABLE> l;
	VARIABLE *v;
	List<PLACED_TILE> l1;
	PLACED_TILE *t;
	List<PLACED_OBJECT> l2;
	PLACED_OBJECT *o;

	fprintf(fp,"VARIABLES %i\n",room_variables.Length());
	l.Instance(room_variables);
	l.Rewind();
	while(l.Iterate(v)) v->save(fp);

	fprintf(fp,"POS %i %i\n",x,y);
	fprintf(fp,"SIZE %i %i\n",dx,dy);

	fprintf(fp,"%i\n",used_object_planes);
	for(i=0;i<used_object_planes;i++) {
	    fprintf(fp,"%i\n",tiles[i].Length());
	    l1.Instance(tiles[i]);
	    l1.Rewind();
	    while(l1.Iterate(t)) t->save(fp,game);

	    fprintf(fp,"%i\n",objects[i].Length());
	    l2.Instance(objects[i]);
	    l2.Rewind();
	    while(l2.Iterate(o)) o->save(fp,game);
	} /* for */ 

	fprintf(fp,"%i\n",drawing_order);
	if (horizontal_circularity) fprintf(fp,"H_CIRCULARITY\n");
						   else fprintf(fp,"H_BOUNDED\n");
	if (vertical_circularity) fprintf(fp,"V_CIRCULARITY\n");
						 else fprintf(fp,"V_BOUNDED\n");

	fprintf(fp,"%i %i %i\n",COLLISION_GRID_SX,COLLISION_GRID_SY,COLLISION_GRID_TOLERANCE);

	room_action->save(fp);

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		fprintf(fp,"%i %i %i %i\n",plane_h_speed[i],plane_v_speed[i],plane_h_align[i],plane_v_align[i]);
	} /* for */ 
	
	return true;
} /* ROOM::save */ 


bool ROOM::load(FILE *fp,MOG2 *game)
{
	char tmp[80];
	int i,j,n;

	if (2!=fscanf(fp,"%s %i",tmp,&n)) return false;

	room_variables.Delete();
	for(i=0;i<n;i++) {
		VARIABLE *v;

		v=new VARIABLE();
		v->load(fp);
		room_variables.Add(v);
	} /* for */ 

	if (3!=fscanf(fp,"%s %i %i",tmp,&x,&y)) return false;
	if (3!=fscanf(fp,"%s %i %i",tmp,&dx,&dy)) return false;

	if (1!=fscanf(fp,"%i",&used_object_planes)) return false;

	if (used_object_planes>MAX_OBJECT_PLANES) return false;

	for(j=0;j<used_object_planes;j++) {
	    if (1!=fscanf(fp,"%i",&n)) return false;
	    tiles[j].Delete();
	    for(i=0;i<n;i++) {
			PLACED_TILE *t;

			t=new PLACED_TILE();
			t->load(fp,game);
			tiles[j].Add(t);
	    } /* for */ 

	    if (1!=fscanf(fp,"%i",&n)) return false;
	    objects[j].Delete();
	    for(i=0;i<n;i++) {
		PLACED_OBJECT *o;

		o=new PLACED_OBJECT();
		o->load(fp,game,this,0);
		objects[j].Add(o);
	    } /* for */ 
	} /* for */ 	

	if (1!=fscanf(fp,"%i",&drawing_order)) return false;


	if (1!=fscanf(fp,"%s",tmp)) return false;
	if (strcmp(tmp,"H_CIRCULARITY")==0) horizontal_circularity=true;
								   else horizontal_circularity=false;
	if (1!=fscanf(fp,"%s",tmp)) return false;
	if (strcmp(tmp,"V_CIRCULARITY")==0) vertical_circularity=true;
								   else vertical_circularity=false;

	if (3!=fscanf(fp,"%i %i %i",&COLLISION_GRID_SX,&COLLISION_GRID_SY,&COLLISION_GRID_TOLERANCE)) return false;

	if (room_action!=0) delete room_action;
	room_action=new ActionSCRIPT();
	if (!room_action->load(fp,game)) return false;

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		if (4!=fscanf(fp,"%i %i %i %i",&plane_h_speed[i],&plane_v_speed[i],&plane_h_align[i],&plane_v_align[i])) return false;
	} /* for */ 

	return true;
} /* ROOM::load */ 


int ROOM::get_x(void)
{
	return x;
} /* ROOM::get_x */ 


int ROOM::get_y(void)
{
	return y;
} /* ROOM::get_y */ 


int ROOM::get_dx(void)
{
	return dx;
} /* ROOM::get_dx */ 


int ROOM::get_dy(void)
{
	return dy;
} /* ROOM::get_dy */ 



bool o_order_topdown(PLACED_OBJECT *o1,PLACED_OBJECT *o2) { return o1->get_y()<=o2->get_y(); };
bool o_order_bottomup(PLACED_OBJECT *o1,PLACED_OBJECT *o2) { return o1->get_y()>=o2->get_y(); };
bool o_order_leftright(PLACED_OBJECT *o1,PLACED_OBJECT *o2) { return o1->get_x()<=o2->get_x(); };
bool o_order_rightleft(PLACED_OBJECT *o1,PLACED_OBJECT *o2) { return o1->get_x()>=o2->get_x(); };

bool t_order_topdown(PLACED_TILE *t1,PLACED_TILE *t2) { return t1->get_y()<=t2->get_y(); };
bool t_order_bottomup(PLACED_TILE *t1,PLACED_TILE *t2) { return t1->get_y()>=t2->get_y(); };
bool t_order_leftright(PLACED_TILE *t1,PLACED_TILE *t2) { return t1->get_x()<=t2->get_x(); };
bool t_order_rightleft(PLACED_TILE *t1,PLACED_TILE *t2) { return t1->get_x()>=t2->get_x(); };


void ROOM::draw(SDL_Surface *sfc,int sx,int sy,int sdx,int sdy,int stx,int sty,float scale,bool *dplane)
{
	int i;

	for(i=0;i<used_object_planes;i++) {
		if (dplane==0 || dplane[i]) {
			List<PLACED_TILE> l1;
			List<PLACED_OBJECT> l2;
			PLACED_TILE *t;
			PLACED_OBJECT *o;

			l1.Instance(tiles[i]);
			l1.Rewind();
			l2.Instance(objects[i]);
			l2.Rewind();

			switch(drawing_order) {
			case 0:
				/* NONE: */ 
				if (scale==1.0) while(l1.Iterate(t)) t->draw(sx,sy,sfc);
						   else while(l1.Iterate(t)) t->draw_scaled(sx,sy,sfc,scale);

				if (scale==1.0) while(l2.Iterate(o)) o->draw(sx,sy,sfc);
						   else while(l2.Iterate(o)) o->draw_scaled(sx,sy,sfc,scale);
				break;

			case 1:
				/* TOP-DOWN: */ 
				l1.Sort(&t_order_topdown);
				l2.Sort(&o_order_topdown);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_y()<=l2.GetObj()->get_y())) {
						if (scale==1.0) l1.GetObj()->draw(sx,sy,sfc);
								   else l1.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l1.Next();
					} else {
						if (scale==1.0) l2.GetObj()->draw(sx,sy,sfc);
								   else l2.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 2:
				/* BOTOOM-UP: */ 
				l1.Sort(&t_order_bottomup);
				l2.Sort(&o_order_bottomup);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_y()>=l2.GetObj()->get_y())) {
						if (scale==1.0) l1.GetObj()->draw(sx,sy,sfc);
								   else l1.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l1.Next();
					} else {
						if (scale==1.0) l2.GetObj()->draw(sx,sy,sfc);
								   else l2.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 3:
				/* LEFT-RIGHT: */ 
				l1.Sort(&t_order_leftright);
				l2.Sort(&o_order_leftright);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_x()<=l2.GetObj()->get_x())) {
						if (scale==1.0) l1.GetObj()->draw(sx,sy,sfc);
								   else l1.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l1.Next();
					} else {
						if (scale==1.0) l2.GetObj()->draw(sx,sy,sfc);
								   else l2.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			case 4:
				/* RIGHT-LEFT: */ 
				l1.Sort(&t_order_rightleft);
				l2.Sort(&o_order_rightleft);

				while(!l1.EndP() || !l2.EndP()) {
					if (!l1.EndP() &&
						(l2.EndP() ||
						 l1.GetObj()->get_x()>=l2.GetObj()->get_x())) {
						if (scale==1.0) l1.GetObj()->draw(sx,sy,sfc);
								   else l1.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l1.Next();
					} else {
						if (scale==1.0) l2.GetObj()->draw(sx,sy,sfc);
								   else l2.GetObj()->draw_scaled(sx,sy,sfc,scale);
						l2.Next();
					} /* if */ 
				} /* while */ 
				break;
			} /* switch */ 
		} /* if */ 
	} /* for */ 
} /* if */ 


void ROOM::draw_frame(SDL_Surface *sfc,int sx,int sy,int sdx,int sdy,int stx,int sty,float scale)
{
	SDL_Rect r;

	r.x=sx-stx;
	r.y=sy-sty;
	r.w=int(dx*TILESIZE*scale);
	r.h=int(dy*TILESIZE*scale);
	SDL_FillRect(sfc,&r,0x003f3fff);

	r.x=1+sx-stx;
	r.y=1+sy-sty;
	r.w=int(dx*TILESIZE*scale)-2;
	r.h=int(dy*TILESIZE*scale)-2;

	SDL_FillRect(sfc,&r,0x003f3f7f);
} /* if */ 



PLACED_OBJECT *ROOM::search_ladder(int x,int y,int dx,int dy)
{
	List<PLACED_OBJECT> l;
	PLACED_OBJECT *o;
	int i;
	int TOLERANCE=8;

	for(i=0;i<MAX_OBJECT_PLANES;i++) {
		l.Instance(objects[i]);
		l.Rewind();
		while(l.Iterate(o)) {
			if (o->object->constitution==CONS_LADDER) {
				if (o->x-TOLERANCE<=x &&
					o->x+o->get_dx()+TOLERANCE>=x+dx &&
					o->y<y+dy &&
					o->y+o->get_dy()>y) {
					return o;
				} /* if */ 
			} /* if */ 
		} /* while */ 
	} /* for */ 

	return 0;
} /* search_ladder */ 
