#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"

#include "List.h"
#include "auxiliar.h"

#include "samples.h"
#include "animscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "AnimScriptWnd.h"
#include "AnimEditorWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BAnimEditorWnd::BAnimEditorWnd(int x,int y,int dx,int dy,AnimSCRIPT **os,OBJECT *o,class EDITOR *e,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;
	int but_dx;

	xmed=((dx-30)/2)+60;

	but_dx=get_text_width("ACCEPT",font)+10;
	b=new BButton(20,dy-20,but_dx,12,"ACCEPT",1);
	buttons.Add(b);

	but_dx=get_text_width("CANCEL",font)+10;
	b=new BButton(120,dy-20,but_dx,12,"CANCEL",2);
	buttons.Add(b);

	but_dx=get_text_width("PREVIEW",font)+10;
	b=new BButton(xmed,dy-40,but_dx,12,"PREVIEW",3);
	buttons.Add(b);

	but_dx=get_text_width("STOP",font)+10;
	b=new BButton(xmed+100,dy-40,but_dx,12,"STOP",4);
	buttons.Add(b);

	but_dx=get_text_width("FASTER",font)+10;
	b=new BButton(xmed,dy-20,but_dx,12,"FASTER",5);
	buttons.Add(b);

	but_dx=get_text_width("SLOWER",font)+10;
	b=new BButton(xmed+100,dy-20,but_dx,12,"SLOWER",6);
	buttons.Add(b);


	script=new AnimSCRIPT(**os);
	inaction_script=0;
	output_script=os;

	object=o;

	animating=false;
	animation_timmer=0;
	animation_timmer2=0;
	animation_speed=1;

	vsw=new BViewAnimScriptWnd(10,10,xmed,dy-60,script,object,e,font);

} /* BAnimEditorWnd::BAnimEditorWnd */ 


BAnimEditorWnd::~BAnimEditorWnd()
{
	delete vsw;
	delete script;
	if (inaction_script!=0) delete inaction_script;

} /* BAnimEditorWnd::BAnimEditorWnd */ 


void BAnimEditorWnd::contens_size(int *dx,int *dy)
{
	*dx=old_dx;
	*dy=old_dy;
} /* BAnimEditorWnd::contens_size */ 



bool BAnimEditorWnd::cycle(int stx,int sty)
{
	if (!BActiveWindow::cycle(stx,sty)) return false;

	vsw->cycle(stx+10,sty+10);
	if (vsw->is_dirty()) set_dirty();

	if (inaction_script!=0 && animating) {
		animation_timmer2++;
		if (animation_timmer2>=animation_speed) {
			animation_timmer++;
			animation_timmer2=0;
			inaction_script->cycle();
		} /* if */ 
		set_dirty();
	} /* if */ 

	return true;
} /* BAnimEditorWnd::cycle */ 



void BAnimEditorWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	vsw->draw(sfc,10-stx,10-sty,xmed-20,dy-60);

	if (inaction_script!=0) {
		char tmp[80];
		int tile=0;

		r.x=xmed-stx;
		r.y=10-sty;
		r.w=(dx-xmed-10);
		r.h=dy-60;
		SDL_FillRect(sfc,&r,0x00000000);

		tile=inaction_script->get_tile();

		if (tile>=0 && tile<object->ntiles) {				
			object->tiles[tile]->draw((r.x+(r.w/2)-object->link_x[tile][0])-stx,
									  (r.y+(r.h/2)-object->link_y[tile][0])-sty,
									  sfc);
		} /* if */ 

		sprintf(tmp,"%i",animation_timmer);
		print_left(tmp,font,255,255,255,sfc,r.x+2-stx,r.y+2-sty);
	} 

	

} /* BAnimEditorWnd::draw_contens */ 


bool BAnimEditorWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		delete *output_script;
		*output_script=script;
		script=0;
		return false;
		break;

	case 2:
		return false;
		break;

	case 3:
		animating=true;
		animation_timmer=0;
		animation_timmer2=0;
		if (inaction_script!=0) delete inaction_script;
		inaction_script=new AnimSCRIPT(*script);
		inaction_script->init();
		return true;
		break;

	case 4:
		animating=false;
		return true;
		break;

	case 5:
		animation_speed--;
		if (animation_speed<1) animation_speed=1;
		return true;
		break;
	case 6:
		animation_speed++;
		if (animation_speed>16) animation_speed=16;
		return true;
		break;
	} /* switch */ 

	return false;
} /* BAnimEditorWnd::button_pressed */ 

