#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#else
#include <stddef.h>
#include <sys/types.h>
#include <dirent.h>
#include "ctype.h"
#include "unistd.h"
#endif

#include "assert.h"
#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BFileDlg.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


BFileDlg::BFileDlg(int x,int y,int dx,int dy,char *q,char *b1,char *f,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;
	int but_dx;

	but_dx=get_text_width(b1,font)+10;
	b=new BButton(20,dy-20,but_dx,12,b1,1);
	buttons.Add(b);
	
	but_dx=get_text_width("CANCEL",font)+10;
	b=new BButton(120,dy-20,but_dx,12,"CANCEL",2);
	buttons.Add(b);

	query=q;
	complete_filename=f;
	strcpy(filename,f);
	getcwd(path,256);
#ifdef _WIN32
        strcat(path,"\\");
#else
	strcat(path,"/");
#endif
	sprintf(complete_filename,"%s%s",path,filename);
	files=new BFileListWnd(5,30,dx-10,dy-80,complete_filename,selected,font);
} /* BFileDlg::BFileDlg */ 


BFileDlg::~BFileDlg()
{
	delete files;
} /* BFileDlg::BFileDlg */ 


void BFileDlg::contens_size(int *dx,int *dy)
{
	*dx=old_dx;
	*dy=old_dy;
} /* BFileDlg::contens_size */ 



bool BFileDlg::cycle(int stx,int sty)
{
	if (!BActiveWindow::cycle(stx,sty)) return false;

	sprintf(complete_filename,"%s%s",path,filename);
	selected[0]=0;

	files->cycle(stx+5,sty+30);
	if (files->is_dirty()) dirty=true;

	if (selected[0]!=0) {
		if (strcmp(selected,"..")==0) {
			int i=0,last=-1,prelast=-1;

			while(path[i]!=0) {
				if (path[i]=='\\' || path[i]=='/') {
					prelast=last;
					last=i;
				} /* if */ 
				i++;
			} /* while */ 
			path[prelast+1]=0;
		} else {
                
#ifdef _WIN32
			char tmp[256];
			WIN32_FIND_DATA finfo;
			HANDLE h;

			sprintf(tmp,"%s%s",path,selected);
			h=FindFirstFile(tmp,&finfo);
			if (h!=INVALID_HANDLE_VALUE) {
				if ((finfo.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY)!=0) {
					if (strcmp(selected,".")!=0) {
						strcat(path,selected);
						strcat(path,"\\");
					} else {
					} /* if */ 
				} else {
					if (strcmp(selected,filename)==0) {
						int l;
						char tmp[256];

						// Convert the ABSOLUTE path in a relative path:
						getcwd(tmp,256);
						l=strlen(tmp);
						if (complete_filename[l]=='\\' || complete_filename[l]=='/') l++;
						strcpy(complete_filename,complete_filename+l);
						return false;
					} /* if */ 
					strcpy(filename,selected);
				} /* if */ 
			} /* if */ 
			FindClose(h);
#else
                        char tmp[256];
			DIR *dp;
			struct dirent *ep;
			  
            sprintf(tmp,"%s%s",path,selected);
			dp = opendir (path);
			if (dp != NULL)
			 {
			    while (ep = readdir (dp))
			     {
                                if (strcmp(ep->d_name,selected)==0) {
                                    if (ep->d_type==DT_DIR) {
                                            if (strcmp(selected,".")!=0) {
                                                    strcat(path,selected);
                                                    strcat(path,"/");
                                            } else {
                                            } /* if */ 
                                    } else {
                                        if (strcmp(selected,filename)==0) {
                                                int l;
                                                char tmp[256];

                                                // Convert the ABSOLUTE path in a relative path:
                                                getcwd(tmp,256);
                                                l=strlen(tmp);
                                                if (complete_filename[l]=='\\' || complete_filename[l]=='/') l++;
						assert(int(strlen(complete_filename))>l);
                                                strcpy(complete_filename,complete_filename+l);
						return false;
					} /* if */ 
					strcpy(filename,selected);
                                    } /* if */ 
                                } /* if */ 
                             }
			    (void) closedir (dp);
			 }


#endif

		} /* if */ 
	} /* if */ 
	return true;
} /* BFileDlg::cycle */ 



void BFileDlg::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);
	files->draw(sfc,5-stx,30-sty,dx-10,dy-80);

	r.x=5-stx;
	r.y=(dy-40)-sty;
	r.w=dx-10;
	r.h=11;
	SDL_FillRect(sfc,&r,0);
	print_left(filename,font,255,255,255,sfc,r.x+2-stx,r.y+2-sty);
	print_left("_",font,255,255,255,sfc,r.x+2+get_text_width(filename,font)-stx,r.y+2-sty);

	print_centered(query,font,255,255,255,sfc,dx/2-stx,8-sty);
} /* BFileDlg::draw_contens */ 


bool BFileDlg::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
			int l;
			char tmp[256];

			// Convert the ABSOLUTE path in a relative path:
			getcwd(tmp,256);
			l=strlen(tmp);
			if (complete_filename[l]=='\\' || complete_filename[l]=='/') l++;
			strcpy(complete_filename,complete_filename+l);
			for(l=0;complete_filename[l]!=0;l++) if (complete_filename[l]=='\\') complete_filename[l]='/';
		}
		return false;
		break;

	case 2:
		complete_filename[0]=0;
		return false;
		break;
	} /* switch */ 

	return false;
} /* BFileDlg::button_pressed */ 



bool BFileDlg::event(SDL_Event *event)
{
	if (event->type==SDL_KEYDOWN) {
		/* Keyboard event */ 
		if (event->key.keysym.sym==SDLK_BACKSPACE) {
			if (filename[0]!=0) filename[strlen(filename)-1]=0;
		} else {
			if (event->key.keysym.sym==SDLK_RETURN) return false;

			if (event->key.keysym.unicode < 0x80 && event->key.keysym.unicode > 0) {
				filename[strlen(filename)+1]=0;
				filename[strlen(filename)]=(char)event->key.keysym.unicode;
			} /* if */ 
		} /* if */ 

		filename[32]=0;
		dirty=true;
    } /* if */ 

	return true;
} /* BFileDlg::Event */ 

