#ifdef _WIN32
#include "windows.h"
#include "direct.h"
#endif

#include "string.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_net.h"
#include "SDL_mixer.h"
#include "SDL_sound.h"
#include "List.h"
#include "collision.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"
#include "music.h"

#include "conditionscript.h"
#include "valuescript.h"
#include "actionscript.h"
#include "animscript.h"
#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activemap.h"
#include "activemog2.h"
#include "activeinterface.h"
#include "activemog2application.h"
#include "mog2multiplayer.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "GamePreviewDlg.h"
#include "GameDebugWnd.h"

#include "ScriptDebugWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X;
extern int SCREEN_Y;
extern int COLOUR_DEPTH;
extern bool SOUND_ENABLED;
extern int extra_flags;
extern bool fullscreen;
extern char *application_name;

/* Redrawing constant: */ 
int GAMEDEBUG_REDRAWING_PERIOD=40;
extern SDL_Surface *screen_sfc;

extern int buffer_fps,act_fps,fps_timer;
extern int buffer_fps_d,fps_d;

const int VIEWPORT_Y=320;
const int DEBUG_START_Y=330;
const int SCRIPT_START_X=300;



GameDebugWindow::GameDebugWindow(int dx,int dy,TTF_Font *font,MOG2 *g,int gm,bool show_fpsp,char *file,List<NWMOG2PlayerInfo> *players) : BActiveWindow(0,0,dx,dy,font)
{
	int i;
	SDL_Rect vp;
	ActiveOBJECT *o;
	List<NWMOG2PlayerInfo> l;
	NWMOG2PlayerInfo *pc;

	n_LVKs=0;
	game_mode=gm;
	replay_fp=0;
	event_time=-1;
	event_key=0;
	event_val=0;

	show_fps=show_fpsp;

	app=new ActiveMOG2Application(g);
	game=app->active_game;

	if (game_mode==0 || game_mode==2) {

		l.Instance(*players);
		l.Rewind();
		while(l.Iterate(pc)) {
			vp=pc->vp_area;
			if (pc->start_interface==0) {
				OBJECT *gwo=0;
				INTERFACE *i;
				i=new INTERFACE(vp.w,vp.h,&gwo);
				interfaces.Add(i);
				objects.Add(gwo);
				o=game->add_player(i,pc->start_room,pc->avatar->name,pc->name,&vp,pc->vp_zoom,0);
				game->viewports[game->viewports.Length()-1]->current_interface->objects[0][0]->gw_focusing.Add(o);
			} else {
				o=game->add_player(pc->start_interface,pc->start_room,pc->avatar->name,pc->name,&vp,pc->vp_zoom,0);
			} /* if */ 
			o->left_key=pc->left_key;
			o->right_key=pc->right_key;
			o->up_key=pc->up_key;
			o->down_key=pc->down_key;
			o->fire1_key=pc->fire1_key;
			o->fire2_key=pc->fire2_key;
			o->fire3_key=pc->fire3_key;
			o->fire4_key=pc->fire4_key;
		} /* while */ 
	} /* if */ 


	switch(game_mode) {
	case 0:
		keyboardstate=new KEYBOARDSTATE *[1];
		keyboardstate[0]=new KEYBOARDSTATE();
		n_LVKs=1;
		break;
	case 1:
		{
			int np,j;
			unsigned int ID_high,ID_low;
			char *name,*name2,*name3,*name4;
			char tmp[80];
			int rx,ry;
			SDL_Rect vp;
			float zoom;
			MAP *start_map;
			ROOM *start_room;
			INTERFACE *start_interface;

			replay_fp=fopen(file,"r");

			if (2!=fscanf(replay_fp,"%u %u",&ID_high,&ID_low) ||
				ID_high!=g->ID_high ||
				ID_low!=g->ID_low) return;

			/* read player configuration: */ 
			if (2!=fscanf(replay_fp,"%i %i",&np,&n_LVKs)) return;

			keyboardstate=new KEYBOARDSTATE *[n_LVKs];
			for(j=0;j<n_LVKs;j++) {
				keyboardstate[j]=new KEYBOARDSTATE();
			} /* for */ 

			for(i=0;i<np;i++) {
				int x,y,w,h;
				if (!load_string(replay_fp,&name)) return;
				if (!load_string(replay_fp,&name2)) {
					delete []name;
					return;
				} /* if */ 
				if (2!=fscanf(replay_fp,"%i %i",&rx,&ry)) {
					delete []name;
					delete []name2;
					return;
				} /* if */ 
				if (!load_string(replay_fp,&name3)) {
					delete []name;
					delete []name2;
					return;
				} /* if */ 
				if (5!=fscanf(replay_fp,"%i %i %i %i %f",&x,&y,&w,&h,&zoom)) {
					delete []name;
					delete []name2;
					delete []name3;
					return;
				} /* ifr */ 
				vp.x=x;
				vp.y=y;
				vp.w=w;
				vp.h=h;

				if (1!=fscanf(replay_fp,"%s",tmp)) {
					delete []name;
					delete []name2;
					delete []name3;
					return;
				} /* if */ 
				if (strcmp(tmp,"INTERFACE")==0) {
					if (!load_string(replay_fp,&name4)) {
						delete []name;
						delete []name2;
						delete []name3;
						return;
					} /* if */ 
				} else {
					name4=0;
				} /* if */ 

				/* Search map: */ 
				{
					MAP *m;
					List<MAP> l;
					ROOM *r;
					List<ROOM> l2;
					INTERFACE *interf;
					List<INTERFACE> l3;
					bool found=false;

					start_map=0;

					l.Instance(g->maps);
					l.Rewind();
					while(!found && l.Iterate(m)) {
						if (m->cmp(name3)) {
							start_map=m;
							found=true;
						} /* if */ 
					} /* while */ 

					if (!found) {
						delete []name;
						delete []name2;
						delete []name3;
						delete []name4;
						return;
					} /* if */ 

					/* Search room: */ 
					found=false;

					start_room=0;
					
					l2.Instance(start_map->rooms);
					l2.Rewind();
					while(!found && l2.Iterate(r)) {
						if (r->get_x()==rx &&
							r->get_y()==ry) {
							start_room=r;
							found=true;
						} /* if */ 
					} /* while */ 

					/* Search interface: */ 
					start_interface=0;
					if (name4!=0) {
						found=false;
						l3.Instance(g->interfaces);
						l3.Rewind();
						while(!found && l3.Iterate(interf)) {
							if (interf->cmp(name4)) {
								start_interface=interf;
								found=true;
							} /* if */ 
						} /* while */ 
					} /* if */ 

					if (!found)  {
						delete []name;
						delete []name2;
						delete []name3;
						delete []name4;
						return;
					} /* if */ 
					if (pc->start_interface==0) {
						OBJECT *gwo=0;
						INTERFACE *i;
						i=new INTERFACE(vp.w,vp.h,&gwo);
						interfaces.Add(i);
						objects.Add(gwo);
						o=game->add_player(i,start_room,name,name2,&vp,zoom,0);
						game->viewports[game->viewports.Length()-1]->current_interface->objects[0][0]->gw_focusing.Add(o);
					} else {
						o=game->add_player(start_interface,start_room,name,name2,&vp,zoom,0);
					} /* if */ 	
				}

				if (8!=fscanf(replay_fp,"%i %i %i %i %i %i %i %i",
							  &o->left_key,&o->right_key,&o->up_key,&o->down_key,
							  &o->fire1_key,&o->fire2_key,&o->fire3_key,&o->fire4_key)) {
					delete []name;
					delete []name2;
					delete []name3;
					delete []name4;
					return;
				} /* if */ 

				delete []name;
				delete []name2;
				delete []name3;
				delete []name4;
			} /* for */ 

		}
		break;
	case 2:
		{
			int i;
			NWMOG2PlayerInfo *pi;
			char *tmp;

			keyboardstate=new KEYBOARDSTATE *[1];
			keyboardstate[0]=new KEYBOARDSTATE();
			n_LVKs=1;

			replay_fp=fopen(file,"w");
			fprintf(replay_fp,"%u %u\n",g->ID_high,g->ID_low);
			fprintf(replay_fp,"%i %i\n",players->Length(),1);

			for(i=0;i<players->Length();i++) {	
				pi=(*players)[i];
				encode_string(pi->avatar->name,&tmp);
				fprintf(replay_fp,"%s\n",tmp);
				delete tmp;

				encode_string(pi->name,&tmp);
				fprintf(replay_fp,"%s\n",tmp);
				delete tmp;

				fprintf(replay_fp,"%i %i\n",pi->start_room->get_x(),pi->start_room->get_y());

				encode_string(pi->start_map->name,&tmp);
				fprintf(replay_fp,"%s\n",tmp);
				delete tmp;

				fprintf(replay_fp,"%i %i %i %i %f\n",pi->vp_area.x,pi->vp_area.y,
												 	 pi->vp_area.w,pi->vp_area.h,
													 pi->vp_zoom);

				if (pi->start_interface!=0) {
					encode_string(pi->start_interface->name,&tmp);
					fprintf(replay_fp,"INTERFACE %s\n",tmp);
					delete tmp;
				} else {
					fprintf(replay_fp,"NOINTERFACE\n");
				} /* if */ 

				fprintf(replay_fp,"%i %i %i %i %i %i %i %i\n",pi->left_key,pi->right_key,pi->up_key,pi->down_key,
															  pi->fire1_key,pi->fire2_key,pi->fire3_key,pi->fire4_key);		
			} /* for */		
		}
		break;
	}/* switch */ 

	running_mode=0;
	selected_object=0;
	selected_script=0;
	bstop=new BButton(0,40,40,20,"STOP",1);
	bstep=new BButton(0,70,40,20,"STEP",2);
	brun=new BButton(0,100,40,20,"RUN",3);

	sdw=new BScriptDebugWnd(SCRIPT_START_X,DEBUG_START_Y+20,
						    dx-SCRIPT_START_X,dy-DEBUG_START_Y-20,0,font);

} /* GameDebugWindow::GameDebugWindow */ 


GameDebugWindow::~GameDebugWindow()
{
	int i;

	for(i=0;i<n_LVKs;i++) {
		delete keyboardstate[i];
		keyboardstate[i]=0;
	} /* if */ 
	delete []keyboardstate;
	if (game_mode==2) {
		/* Saving replay: */ 
		fprintf(replay_fp,"END\n");
	} /* if */ 

	if (game_mode!=0) fclose(replay_fp);

	delete app;

	delete bstop;
	delete bstep;
	delete brun;
} /* GameDebugWindow:: */ 


void GameDebugWindow::Execute(void)
{
	int time,act_time;
	SDL_Event event;
	bool quit=false;

	active_window=this;

	SDL_QuitSubSystem(SDL_INIT_VIDEO);
	SDL_InitSubSystem(SDL_INIT_VIDEO);
	screen_sfc = SDL_SetVideoMode(old_dx, old_dy, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

//	if (SOUND_ENABLED) {
//		init_music(game);
//		Mix_HookMusic(myMusicPlayer, (void *)0);
//	} /* if */ 

	time=SDL_GetTicks();
	while (!quit) {
		while( SDL_PollEvent( &event ) ) {
			bool captured=false;

            switch( event.type ) {
                /* Keyboard event */ 
                case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_F12 ||
						event.key.keysym.sym==SDLK_ESCAPE) {
						captured=true;
						quit = true;
					} /* if */ 

					if (event.key.keysym.sym==SDLK_RETURN) {
						SDLMod modifiers=SDL_GetModState();

						if ((modifiers&KMOD_ALT)!=0) {
							captured=true;

							/* Toogle FULLSCREEN mode: */ 
							if (fullscreen) fullscreen=false;
									   else fullscreen=true;
							SDL_QuitSubSystem(SDL_INIT_VIDEO);
							SDL_InitSubSystem(SDL_INIT_VIDEO);
							if (SDL_WasInit(SDL_INIT_VIDEO)) {
								screen_sfc = SDL_SetVideoMode(old_dx, old_dy, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
								SDL_WM_SetCaption(application_name, 0);
								SDL_ShowCursor(SDL_DISABLE);
								SDL_EnableUNICODE(1);
							} else {
								quit = true;
							} /* if */ 
						} /* if */ 
					} /* if */ 
                    break;

                /* SDL_QUIT event (window close) */ 
                case SDL_QUIT:
					captured=true;

                    quit = true;
                    break;
            } /* switch */ 

			if (!captured) {
				if (!this->event(&event)) quit=true;
			} /* if */ 
        } /* while */ 

		act_time=SDL_GetTicks();
		if (act_time-time>=GAMEDEBUG_REDRAWING_PERIOD)
		{
			act_fps++;
			if ((act_time-fps_timer)>1000) {
				fps_timer=act_time;
				buffer_fps=act_fps;
				buffer_fps_d=fps_d;
				act_fps=0;
				fps_d=0;
			} /* if */ 

			time+=GAMEDEBUG_REDRAWING_PERIOD;
			if ((act_time-time)>8*GAMEDEBUG_REDRAWING_PERIOD) time=act_time;

			check_mouse();

			if (!cycle(old_x,old_y)) {
				quit=true;
			} /* if */ 

			/* Available scripts: */ 
			{
				List<ActiveMAP> ml;
				ActiveMAP *am;
				List<ActiveROOM> rl;
				ActiveROOM *ar;
				char *tmp,tmp2[128];
				int mc=0,rc=0;

				while(!available_scripts.EmptyP()) available_scripts.ExtractIni();
				available_scripts_label.Delete();

				available_scripts.Add(game->game_action);
				sprintf(tmp2,"G");
				tmp=new char[strlen(tmp2)+1];
				strcpy(tmp,tmp2);
				available_scripts_label.Add(tmp);

				ml.Instance(game->active_maps);
				ml.Rewind();
				while(ml.Iterate(am)) {
					available_scripts.Add(am->map_action);
					sprintf(tmp2,"M%i",mc);
					mc++;
					tmp=new char[strlen(tmp2)+1];
					strcpy(tmp,tmp2);
					available_scripts_label.Add(tmp);

					rl.Instance(am->active_rooms);
					rl.Rewind();
					rc=0;
					while(rl.Iterate(ar)) {
						available_scripts.Add(ar->room_action);
						sprintf(tmp2,"R%i-%i",mc,rc);
						rc++;
						tmp=new char[strlen(tmp2)+1];
						strcpy(tmp,tmp2);
						available_scripts_label.Add(tmp);
					} /* while */ 
				} /* while */ 

				if (selected_object!=0 && game->find_object(selected_object)) {
					int i;
					/* standard scripts: */ 
					switch(selected_object->object->control) {
					case CTRL_STANDARD:
						if (selected_object->state>=0 && 
							selected_object->state<selected_object->object->n_states) {
							available_scripts.Add(selected_object->g_state_action[selected_object->state]);
							sprintf(tmp2,"O%i",selected_object->state);
							tmp=new char[strlen(tmp2)+1];
							strcpy(tmp,tmp2);
							available_scripts_label.Add(tmp);
						} /* if */ 
						break;
					case CTRL_CHARACTER:
						break;
					case CTRL_TEXT:
						break;
					} /* if */ 

					/* conditional scripts: */ 
					for(i=0;i<selected_object->object->n_s_states;i++) {
						if (selected_object->g_s_state_executing[i]) {
							available_scripts.Add(selected_object->g_s_state_action[i]);
							sprintf(tmp2,"OC%i",i);
							tmp=new char[strlen(tmp2)+1];
							strcpy(tmp,tmp2);
							available_scripts_label.Add(tmp);
						} /* if */ 
					} /* for */ 
					
				} else {
					selected_object=0;
				} /* if */ 
			}

			if (((mouse_button&SDL_BUTTON(1) &&
				!(old_mouse_button&SDL_BUTTON(1))) ||
				(mouse_button&SDL_BUTTON(3) &&
				!(old_mouse_button&SDL_BUTTON(3))))) {
				int x;
				char *label;
				
				x=SCRIPT_START_X;
				available_scripts.Rewind();
				available_scripts_label.Rewind();
				while(available_scripts_label.Iterate(label)) {
					if (local_mouse_x>=x && local_mouse_x<=x+get_text_width(label,font) &&
						local_mouse_y>=DEBUG_START_Y && local_mouse_y<=DEBUG_START_Y+TTF_FontHeight(font)) {
						selected_script=available_scripts.GetObj();
					} /* if */ 
					x+=get_text_width(label,font)+10;
					available_scripts.Next();
				} /* while */ 
			} /* if */ 

			/* Selected script: */ 
			if (selected_script!=0) {
				if (available_scripts.MemberRefP(selected_script)) {
					sdw->set_script(selected_script);
					sdw->cycle(SCRIPT_START_X,DEBUG_START_Y+20);
					sdw->set_dirty();
				} else {
					selected_script=0;
				} /* if */ 
			} /* if */ 

		}

		if (!quit) {
			fps_d++;
			SDL_FillRect(screen_sfc,0,0);
			if (game!=0) {
				game->draw(screen_sfc);
//				game->sound(0,-1,-1);
			} /* if */ 

			/* show DEBUG INFO: */ 
			{
				char tmp[256];
				int y=DEBUG_START_Y;
				
				if (show_fps) {
					sprintf(tmp,"FPS: %i/%i",buffer_fps,buffer_fps_d);
					print_left(tmp,font,255,255,255,screen_sfc,0,0);
				} /* if */ 
				sprintf(tmp,"CYCLE: %i",game->get_current_cycle());
				print_left(tmp,font,255,255,255,screen_sfc,0,20);

				local_draw_button(screen_sfc,bstop,0,0,(running_mode==0 ? true : false));
				local_draw_button(screen_sfc,bstep,0,0,(running_mode==1 ? true : false));
				local_draw_button(screen_sfc,brun,0,0,(running_mode==2 ? true : false));

				/* selected objects: */ 
				if (!game->find_object(selected_object)) selected_object=0;
				if (selected_object==0) {
					print_left("SELECTED OBJECT: -",font,255,255,255,screen_sfc,0,y);
					y+=10;
				} else {
					sprintf(tmp,"SELECTED OBJECT: %s(%p)",selected_object->object->name,selected_object);
					print_left(tmp,font,255,255,255,screen_sfc,0,y);
					y+=10;
					if (selected_object->parent==0) {
						sprintf(tmp,"PARENT: -");
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;
					} else {
						sprintf(tmp,"PARENT: %p -> %s [GLUED TO PART %i(%i) + (%i,%i)] PRECEDENCE: %i",selected_object->parent,selected_object->parent->object->name,
								selected_object->glued_part,selected_object->glued_link,selected_object->glued_xoffs,selected_object->glued_yoffs,selected_object->relative_precedence);
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;
					} /* if */ 

					{
						/* Glued children: */ 
						List<ActiveOBJECT> l;
						ActiveOBJECT *ao;
						char *tmp2=0,*tmp3=0;

						tmp2=new char[strlen("GLUED CHILDREN:")+1];
						sprintf(tmp2,"GLUED CHILDREN:");

						l.Instance(selected_object->glued_children);
						l.Rewind();
						while(l.Iterate(ao)) {
							sprintf(tmp,"%s(%p)",ao->object->name,ao);
							tmp3=new char[strlen(tmp2)+strlen(tmp)+4];
							sprintf(tmp3,"%s , %s",tmp2,tmp);
							delete []tmp2;
							tmp2=tmp3;
							tmp3=0;
						} /* while */ 

						print_left(tmp2,font,255,255,255,screen_sfc,0,y);
						y+=10;
						delete []tmp2;

						/* Independent children: */ 					
						tmp2=new char[strlen("INDEPENDENT CHILDREN:")+1];
						sprintf(tmp2,"INDEPENDENT CHILDREN:");

						l.Instance(selected_object->independent_children);
						l.Rewind();
						while(l.Iterate(ao)) {
							sprintf(tmp,"%s(%p)",ao->object->name,ao);
							tmp3=new char[strlen(tmp2)+strlen(tmp)+4];
							sprintf(tmp3,"%s , %s",tmp2,tmp);
							delete []tmp2;
							tmp2=tmp3;
							tmp3=0;
						} /* while */ 

						print_left(tmp2,font,255,255,255,screen_sfc,0,y);
						y+=10;
						delete []tmp2;
					}

					{
						sprintf(tmp,"STATE: %i/%i TIMMER: %i",selected_object->state,selected_object->old_state,selected_object->state_timmer);
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;

						sprintf(tmp,"STATE BEFORE SPECIAL: %i",selected_object->state_before_special);
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;

						{
							int i;
							char *tmp2=0,*tmp3=0;

							tmp2=new char[strlen("PART TILES:")+1];
							sprintf(tmp2,"PART TILES:");

							for(i=0;i<selected_object->object->nparts;i++) {
								sprintf(tmp,"%i",selected_object->tile[i]);
								tmp3=new char[strlen(tmp2)+strlen(tmp)+4];
								sprintf(tmp3,"%s , %s",tmp2,tmp);
								delete []tmp2;
								tmp2=tmp3;
								tmp3=0;
							} /* while */ 

							print_left(tmp2,font,255,255,255,screen_sfc,0,y);
							y+=10;
							delete []tmp2;
						}

						sprintf(tmp,"POSITION: (%i,%i) + (%i,%i)/100",int(selected_object->x),int(selected_object->y),selected_object->precision_x,selected_object->precision_y);
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;

						sprintf(tmp,"ENERGY: %i",selected_object->energy);
						print_left(tmp,font,255,255,255,screen_sfc,0,y);
						y+=10;

						/* Variables: */ 
						switch(selected_object->object->control) {
						case CTRL_STANDARD:
							break;
						case CTRL_CHARACTER:
							sprintf(tmp,"JUMP: INERTIA: %i  STOPPED: %s",selected_object->jumping_inertia,(selected_object->stopped_jumping ? "true":"false"));
							print_left(tmp,font,255,255,255,screen_sfc,0,y);
							y+=10;

							sprintf(tmp,"LOOKING_RIGHT_IN_LADDER: %s",(selected_object->looking_right_in_ladder ? "true":"false"));
							print_left(tmp,font,255,255,255,screen_sfc,0,y);
							y+=10;
							break;
						case CTRL_TEXT:
							break;
						} /* switch */ 

						switch(selected_object->object->constitution) {
						case CONS_NONE:
						case CONS_SOLID:
						case CONS_SOLID_SURFACE:
						case CONS_CHARACTER:
							break;
						case CONS_WEAPON:
							if (selected_object->object->damage_type!=0) {
								sprintf(tmp,"DAMAGE: %i, TYPE: %s",selected_object->damage,selected_object->object->damage_type);
							} else {
								sprintf(tmp,"DAMAGE: %i, TYPE: -",selected_object->damage);
							} /* if */ 
							print_left(tmp,font,255,255,255,screen_sfc,0,y);
							y+=10;
							break;
						case CONS_LADDER:
							break;
						case CONS_ITEM:
							break;

						} /* switch */ 


						/* General Variables: */ 
						{
							List<VARIABLE> l;
							VARIABLE *v;

							l.Instance(selected_object->object_variables);
							l.Rewind();
							while(l.Iterate(v)) {
								int type;
								int ival;
								char *sval;

								type=v->get(&ival,&sval,0);
								if (type==VALUE_INTEGER) {
									sprintf(tmp,"VARIABLE %s: %i",v->name,ival);
								} else if (type==VALUE_STRING) {
									sprintf(tmp,"VARIABLE %s: %s",v->name,sval);
								} else {
									sprintf(tmp,"VARIABLE %s: #OBJ#",v->name);
								} /* if */ 
								print_left(tmp,font,255,255,255,screen_sfc,0,y);
								y+=10;
							} /* while */ 
						}
					}
				} /* if */ 
					
				/* Action scripts: */ 
				{
					/* Available Scripts: */ 
					{
						int x;
						char *label;
						
						x=SCRIPT_START_X;
						available_scripts.Rewind();
						available_scripts_label.Rewind();
						while(available_scripts_label.Iterate(label)) {
							if (selected_script==available_scripts.GetObj()) {
								print_left(label,font,255,0,0,screen_sfc,x,DEBUG_START_Y);
							} else {
								print_left(label,font,255,255,255,screen_sfc,x,DEBUG_START_Y);
							} /* if */ 
							x+=get_text_width(label,font)+10;
							available_scripts.Next();
						} /* while */ 
					}

					/* Display selected script: */ 
					if (selected_script!=0) {
						sdw->draw(screen_sfc,SCRIPT_START_X,DEBUG_START_Y+20,
											 old_dx-SCRIPT_START_X,old_dy-DEBUG_START_Y-20);
					} /* if */ 
				}

/*
	AnimSCRIPT *state_anim[MAX_PARTS][CPOS_NSTATES];
	AnimSCRIPT **g_state_anim[MAX_PARTS];
	ActionSCRIPT **g_state_action;
	ConditionSCRIPT **g_state_condition;			
*/
			}

			draw_mouse_pointer(screen_sfc);
			SDL_Flip(screen_sfc);
		} /* if */ 

	}

	while(!available_scripts.EmptyP()) available_scripts.ExtractIni();
	available_scripts_label.Delete();

//	if (SOUND_ENABLED) {
//		Mix_HookMusic(0, 0);
//		init_music(game);
//	} /* if */ 

	SDL_QuitSubSystem(SDL_INIT_VIDEO);
	SDL_InitSubSystem(SDL_INIT_VIDEO);
	screen_sfc = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0));
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

} /* GameDebugWindow::Execute */ 


bool GameDebugWindow::cycle(int stx,int sty)
{
	int i;
	bool retval=true;

	dirty=true;

	/* running_mode = 0 -> stop	*/ 
	/* running_mode = 1 -> step	*/ 
	/* running_mode = 2 -> run	*/ 
	if (running_mode!=0) {
		if (running_mode==1) running_mode=0;

		if (game_mode==0 || game_mode==2) {
			keyboardstate[0]->cycle();

			if (game!=0) {
				retval=game->cycle(keyboardstate,1);
				app->current_cycle++;
			} /* if */ 

			if (game_mode==2) {
				/* Saving replay: */ 
				fprintf(replay_fp,"NEWCYCLE\n");
				for(i=0;i<n_LVKs;i++) keyboardstate[i]->save(replay_fp);
			} /* if */ 

		} /* if */ 

		if (game_mode==1) {
			char tmp[80];

			/* Load replay: */ 
			if (1!=fscanf(replay_fp,"%s",tmp) ||
				strcmp(tmp,"NEWCYCLE")!=0) return false;

			for(i=0;i<n_LVKs;i++) keyboardstate[i]->load(replay_fp);

			if (game!=0) {
				retval=game->cycle(keyboardstate,1);
				app->current_cycle++;
			} /* if */ 
		} /* if */ 

		for(i=0;i<n_LVKs;i++) keyboardstate[i]->keyevents.Delete();

	} /* if */ 

	local_mouse_x=mouse_x;
	local_mouse_y=mouse_y;

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3))))) {
		/* STOP, STEP and RUN buttons: */ 
		if (local_mouse_x>=bstop->m_x && local_mouse_x<bstop->m_x+bstop->m_dx &&
			local_mouse_y>=bstop->m_y && local_mouse_y<bstop->m_y+bstop->m_dy) {
			running_mode=0;
		} /* if */ 
		if (local_mouse_x>=bstep->m_x && local_mouse_x<bstep->m_x+bstep->m_dx &&
			local_mouse_y>=bstep->m_y && local_mouse_y<bstep->m_y+bstep->m_dy) {
			running_mode=1;
		} /* if */ 
		if (local_mouse_x>=brun->m_x && local_mouse_x<brun->m_x+brun->m_dx &&
			local_mouse_y>=brun->m_y && local_mouse_y<brun->m_y+brun->m_dy) {
			running_mode=2;
		} /* if */ 

		/* Select one object: */ 
		if (local_mouse_x>=100 &&
			local_mouse_y>=0 && local_mouse_y<VIEWPORT_Y) {
			/* Find the right viewport: */ 
			List<VIEWPORT> l;
			VIEWPORT *v;
			int i;
			List<ActiveOBJECT> l2;
			ActiveOBJECT *gwindow;

			selected_object=0;

			l.Instance(game->viewports);
			l.Rewind();
			while(l.Iterate(v)) {
				if (local_mouse_x>=v->vp.x &&
					local_mouse_x<v->vp.x+v->vp.w &&
					local_mouse_y>=v->vp.y &&
					local_mouse_y<v->vp.y+v->vp.h) {
					/* Click inside this viewport: */ 

					if (v->current_interface==0) {
						/* There is no interface in this viewport!! */ 

					} else {
						/* There is an active interface!: */ 

						for(i=0;i<MAX_OBJECT_PLANES;i++) {
							l2.Instance(v->current_interface->objects[i]);
							l2.Rewind();
							while(l2.Iterate(gwindow)) {
								if (gwindow->object->control==CTRL_GAMEWINDOW &&
									!gwindow->gw_focusing.EmptyP()) {

									int i;
									int mx,my;
									List<ActiveOBJECT> l;
									ActiveOBJECT *ao;
									SDL_Surface *tmp_sfc=SDL_CreateRGBSurface(0,1,1,32,0,0,0,0);
									SDL_FillRect(tmp_sfc,0,0xffffffff);
									MOG2CollisionData *tmp_collision=MOG2_make_cmap(tmp_sfc);
									
									mx=int((local_mouse_x-v->last_x)/v->zoom)-gwindow->x+gwindow->gw_last_viewx+gwindow->gw_focusing[0]->room->room->x*TILESIZE;
									my=int((local_mouse_y-v->last_y)/v->zoom)-gwindow->y+gwindow->gw_last_viewy+gwindow->gw_focusing[0]->room->room->y*TILESIZE;

									for(i=0;i<MAX_OBJECT_PLANES;i++) {
										l.Instance(gwindow->gw_focusing[0]->room->objects[i]);
										l.Rewind();
										while(l.Iterate(ao)) {
											if (ao->low_level_collision(ao->x+gwindow->gw_focusing[0]->room->room->x*TILESIZE,
																		ao->y+gwindow->gw_focusing[0]->room->room->y*TILESIZE,
																		mx,my,tmp_collision,CONS_ALL,true)) {
												bool new_collision=false;
												// Check for collision with a child object: 
												selected_object=ao;
												do {
													List<ActiveOBJECT> l2;
													ActiveOBJECT *child;

													new_collision=false;

													l2.Instance(ao->independent_children);
													l2.Rewind();
													while(l2.Iterate(child)) {
														if (child->low_level_collision(child->x+gwindow->gw_focusing[0]->room->room->x*TILESIZE,
																					   child->y+gwindow->gw_focusing[0]->room->room->y*TILESIZE,
																					   mx,my,tmp_collision,CONS_ALL,true)) {
															selected_object=child;
															new_collision=true;
														} // if 
													} // while 

													l2.Instance(ao->glued_children);
													l2.Rewind();
													while(l2.Iterate(child)) {
														if (child->low_level_collision(child->x+gwindow->gw_focusing[0]->room->room->x*TILESIZE,
																					   child->y+gwindow->gw_focusing[0]->room->room->y*TILESIZE,
																					   mx,my,tmp_collision,CONS_ALL,true)) {
															selected_object=child;
															new_collision=true;
														} // if 
													} // while 
													ao=selected_object;
												} while(new_collision);
											} // if
										} // while
									} // for

									MOG2_destroy_cmap(tmp_collision);
									SDL_FreeSurface(tmp_sfc);

								} /* if */ 
							} /* while */ 
						} /* for */ 
					} /* if */ 

				} /* if */ 
			} /* while */ 

		} /* if */  
	} /* if */ 

	return retval;
} /* GameDebugWindow::cycle */ 


void GameDebugWindow::contens_size(int *dx,int *dy)
{
} /* GameDebugWindow::contens_size */ 


void GameDebugWindow::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
} /* GameDebugWindow::draw_contens */ 


bool GameDebugWindow::event(SDL_Event *event)
{
	if (active_window!=this) return true;

	if (event->type==SDL_KEYDOWN && 
		(game_mode==0 || game_mode==2)) {
		/* Keyboard event */ 
		SDL_keysym *ks;
		ks=new SDL_keysym();
		*ks=event->key.keysym;
		keyboardstate[0]->keyevents.Add(ks);
    } /* if */ 

	return true;
} /* GameDebugWindow::Event */ 



char *GameDebugWindow::action_script_state(ActionSCRIPT *script)
{
	char *tmp=0;

	switch(script->command) {
	case ACTION_CMD_NONE:
		tmp=new char[strlen("none")+1];
		strcpy(tmp,"none");
		break;
	case ACTION_CMD_SEQ:
		{
			char *tmp2,*tmp3,*tmp4;
			List<ActionSCRIPT> l;
			ActionSCRIPT *s;

			l.Instance(script->aparlist);
			if (script->aparlist.EndP()) {
				tmp=new char[strlen("SEQ{ ... }")+1];
				strcpy(tmp,"SEQ{ ... }");
			} else {
				s=script->aparlist.GetObj();

				l.Rewind();
				if (l.GetObj()==s) {
					/* The current command is the first one: */ 
					tmp2=new char[strlen("SEQ{\n")+1];
					strcpy(tmp2,"SEQ{\n");
				} else {
					/* The current command is not the first one: */ 
					tmp2=new char[strlen("SEQ{\n...\n")+1];
					strcpy(tmp2,"SEQ{\n...\n");
				} /* if */ 

				l.Forward();
				if (l.GetObj()==s) {
					/* The current command is the last one: */ 
					tmp4=new char[strlen("\n}")+1];
					strcpy(tmp4,"\n}");
				} else {
					/* The current command is not the last one: */ 
					tmp4=new char[strlen("\n...\n}")+1];
					strcpy(tmp4,"\n...\n}");
				} /* if */ 

				tmp3=action_script_state(s);

				tmp=new char[strlen(tmp2)+strlen(tmp3)+strlen(tmp4)+1];
				sprintf(tmp,"%s%s%s",tmp2,tmp3,tmp4);
				delete []tmp2;
				delete []tmp3;
				delete []tmp4;
			} /* if */ 
		}
		break;
            
	case ACTION_CMD_PARALLEL:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=action_script_state(as);
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length();
			} /* if */ 
			len+=strlen("PARALLEL{\n\n}")+1;

			tmp=new char[len];

			sprintf(tmp,"PARALLEL{\n");
			pos=strlen("PARALLEL{\n");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(tmp+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(tmp+pos,"\n");
					pos+=1;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(tmp+pos,"\n}");
		}		
		break;
                
	case ACTION_CMD_IF:
		{
			char *s,*s2,*s3;
			int len=0;

			s=condition_script_state(script->cond);
			len+=strlen(s);
		
			s2=action_script_state(script->aparlist[0]);
			s3=action_script_state(script->aparlist[1]);
			
			len+=strlen(s2);
			len+=strlen(s3);
			len+=strlen("IF (  )\n\nELSE ")+1;

			tmp=new char[len];

			sprintf(tmp,"IF ( %s )\n%s\nELSE %s",s,s2,s3);

			delete []s;
			delete []s2;
			delete []s3;
		}
		break;
                
	case ACTION_CMD_WHILE:
		{
			char *tmp2,*tmp3;

			tmp2=condition_script_state(script->cond);
			tmp3=action_script_state(script->aparlist[0]);

			tmp=new char[strlen(tmp2)+strlen(tmp3)+16];
			sprintf(tmp,"WHILE( %s ) {\n%s\n}",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case ACTION_CMD_WHILE_ELSE:
		{
			char *tmp2,*tmp3,*tmp4;

			tmp2=condition_script_state(script->cond);
			tmp3=action_script_state(script->aparlist[0]);
			tmp4=action_script_state(script->aparlist[1]);

			tmp=new char[strlen(tmp2)+strlen(tmp3)+16];
			sprintf(tmp,"WHILE( %s ) {\n%s\n} ELSE %s",tmp2,tmp3,tmp4);
			delete []tmp2;
			delete []tmp3;
			delete []tmp4;
		}
		break;

	case ACTION_CMD_DO:
		{
			char *tmp2,*tmp3;

			tmp2=condition_script_state(script->cond);
			tmp3=action_script_state(script->aparlist[0]);

			tmp=new char[strlen(tmp2)+strlen(tmp3)+19];
			sprintf(tmp,"DO {\n%s\n} WHILE( %s )",tmp3,tmp2);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case ACTION_CMD_PAUSE:
		{
			char *tmp2;
			char tmp3[8];

			tmp2=value_script_state(script->vpar[0]);
			sprintf(tmp3,"%i",script->timmer);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+11];
			sprintf(tmp,"PAUSE( %s/%s )",tmp3,tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_ATOMIC:
		{
			char *tmp2;

			tmp2=action_script_state(script->aparlist[0]);
			tmp=new char[strlen(tmp2)+11];
			sprintf(tmp,"ATOMIC( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_TERMINATE:
		tmp=new char[strlen("terminate")+1];
		strcpy(tmp,"terminate");
		break;

	case ACTION_CMD_TERMINATECHILD:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+19];
			sprintf(tmp,"TERMINATECHILD( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_STATE:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+10];
			sprintf(tmp,"STATE( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_SET:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+8];
			sprintf(tmp,"SET %s = %s",tmp2,tmp3);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_MOVE:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+12];
			sprintf(tmp,"MOVE( %s , %s )",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case ACTION_CMD_CREATEOBJECT:
		{
			/* CREATEOBJECT (xoffs,yoffs,TYPE,precedence);	*/ 
			char *s,*s2,*s3;

			s=value_script_state(script->vpar[0]);
			s2=value_script_state(script->vpar[1]);
			s3=value_script_state(script->vpar[2]);

			tmp=new char[strlen("CREATEOBJECT (  ,  ,  )")+strlen(s)+strlen(s2)+strlen(s3)+1];
			sprintf(tmp,"CREATEOBJECT ( %s , %s , %s )",s,s2,s3);
			delete []s;
			delete []s2;
			delete []s3;
		}
		break;


	case ACTION_CMD_CREATECHILD:
		{
			/* CREATECHILD (xoffs,yoffs,TYPE,precedence);	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6;

			s=value_script_state(script->vpar[0]);
			s2=value_script_state(script->vpar[1]);
			s3=value_script_state(script->vpar[2]);
			s4=value_script_state(script->vpar[3]);
			s5=value_script_state(script->vpar[4]);
			s6=value_script_state(script->vpar[5]);

			tmp=new char[strlen("CREATECHILD (  ,  ,  ,  ,  ,  )")+strlen(s)+strlen(s2)+strlen(s3)+strlen(s4)+strlen(s5)+strlen(s6)+1];
			sprintf(tmp,"CREATECHILD ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;
	case ACTION_CMD_CREATEGLUEDCHILD:
		{
			/* CREATEGLUEDCHILD (xoffs,yoffs,TYPE,precedence);	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6;

			s=value_script_state(script->vpar[0]);
			s2=value_script_state(script->vpar[1]);
			s3=value_script_state(script->vpar[2]);
			s4=value_script_state(script->vpar[3]);
			s5=value_script_state(script->vpar[4]);
			s6=value_script_state(script->vpar[5]);

			tmp=new char[strlen("CREATEGLUEDCHILD (  ,  ,  ,  ,  ,  )")+strlen(s)+strlen(s2)+strlen(s3)+strlen(s4)+strlen(s5)+strlen(s6)+1];
			sprintf(tmp,"CREATEGLUEDCHILD ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	default:
		tmp=new char[strlen("#action#")+1];
		strcpy(tmp,"#action#");
		break;
	} /* switch */ 

	return tmp;
} /* GameDebugWindow::action_script_state */ 


char *GameDebugWindow::condition_script_state(ConditionSCRIPT *script)
{
	char *tmp=0;

	switch(script->command) {
	case CONDITION_CMD_TRUE:
		tmp=new char[strlen("true")+1];
		strcpy(tmp,"true");
		break;
	case CONDITION_CMD_FALSE:
		tmp=new char[strlen("false")+1];
		strcpy(tmp,"false");
		break;
	case CONDITION_CMD_NOT:
		{
			char *tmp2;

			tmp2=condition_script_state(script->aparlist[0]);
			tmp=new char[strlen(tmp2)+8];
			sprintf(tmp,"NOT( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_AND:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=condition_script_state(as);
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("AND {\n\n}")+1;

			tmp=new char[len];

			sprintf(tmp,"AND {\n");
			pos=strlen("AND {\n");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(tmp+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(tmp+pos,"\n");
					pos++;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(tmp+pos,"}");
		}
		break;

	case CONDITION_CMD_OR:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=condition_script_state(as);
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("OR {\n\n}")+1;

			tmp=new char[len];

			sprintf(tmp,"OR {\n");
			pos=strlen("OR {\n");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(tmp+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(tmp+pos,"\n");
					pos++;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(tmp+pos,"}");
		}
		break;

	case CONDITION_CMD_TIMMER:
		{
			char *tmp2;
			char tmp3[8];

			tmp2=value_script_state(script->vpar[0]);
			sprintf(tmp3,"%i",script->timmer);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+12];
			sprintf(tmp,"TIMMER( %s/%s )",tmp3,tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_COLLISION:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+17];
			sprintf(tmp,"COLLISION( %s , %s )",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case CONDITION_CMD_INSIDE_ROOM:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+19];
			sprintf(tmp,"INSIDE_ROOM( %s , %s )",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case CONDITION_CMD_KEYPRESS:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+13];
			sprintf(tmp,"KEYPRESS( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_KEYPRESSED:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+15];
			sprintf(tmp,"KEYPRESSED( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_VARIABLE_EQ:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+5];
			sprintf(tmp,"%s == %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_GR:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+4];
			sprintf(tmp,"%s > %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_GREQ:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+5];
			sprintf(tmp,"%s >= %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_LW:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+4];
			sprintf(tmp,"%s < %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_LWEQ:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+5];
			sprintf(tmp,"%s <= %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	default:
		tmp=new char[strlen("#condition#")+1];
		strcpy(tmp,"#condition#");
		break;
	} /* switch */ 

	return tmp;
} /* GameDebugWindow::condition_script_state */ 


char *GameDebugWindow::value_script_state(ValueSCRIPT *script)
{
	char *tmp=0;

	switch(script->command) {
	case VALUE_CMD_INTEGER:
		{
			char tmp2[16];

			sprintf(tmp2,"%i",script->ipar);
			tmp=new char[strlen(tmp2)+1];
			strcpy(tmp,tmp2);
		}
		break;
	case VALUE_CMD_STRING:
		tmp=new char[strlen(script->spar)+1];
		strcpy(tmp,script->spar);
		break;
	case VALUE_CMD_KEY:
		{
			char tmp2[16];

			sprintf(tmp2,input_key_name(script->ipar));
			tmp=new char[strlen(tmp2)+1];
			strcpy(tmp,tmp2);
		}
		break;
	case VALUE_CMD_VARIABLE:
		tmp=new char[strlen(script->vname)+1];
		sprintf(tmp,"%s",script->vname);
		break;
/*
#define VALUE_CMD_RANDOM		5
#define VALUE_CMD_ADD			6
#define VALUE_CMD_SUB			7
#define VALUE_CMD_MUL			8
#define VALUE_CMD_DIV			9
*/
	default:
		tmp=new char[strlen("#value#")+1];
		strcpy(tmp,"#value#");
		break;
	} /* switch */ 

	return tmp;
} /* GameDebugWindow::value_script_state */ 

