#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"

#include "GVariableWnd.h"
#include "MMapWnd.h"

#include "OTopWnd.h"
#include "interfaces.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

MMapWnd::MMapWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;

	editor=e;
	placing_room=false;
	room_x=0;
	room_y=0;
	show_grid=true;
	grid_size=4;

	b=new BButton(160,0,100,12,"GRID",1);
	buttons.Add(b);

	b=new BButton(270,0,120,12,"FRAMES/MINIATURES",2);
	buttons.Add(b);

} /* MMapWnd::MMapWnd */ 


MMapWnd::~MMapWnd()
{
} /* MMapWnd::~MMapWnd */ 


bool MMapWnd::cycle(int stx,int sty)
{
	int x1,y1,x2,y2,rdx,rdy;

	BWindow::cycle(stx,sty);

	if (editor->map!=0) {
		editor->map->CMC(&x1,&y1,&x2,&y2);
		editor->map->BiggestUnplacedRoom(&rdx,&rdy);
		if (local_mouse_x>old_stx &&
			local_mouse_y>old_sty &&
			local_mouse_x<old_stx+old_dx &&
			local_mouse_y<old_sty+old_dy &&
			local_mouse_x>1 &&
			local_mouse_y>8+TTF_FontHeight(font) &&
			local_mouse_x<=2+(x2-x1)*2+rdx*4 &&
			local_mouse_y<=9+TTF_FontHeight(font)+(y2-y1)*2+rdy*4) {

			if (editor->mv_selected!=0) {
				set_dirty();

				/* Placing a room in the map: */ 
				room_x=((local_mouse_x-(2+rdx*2))/2)+x1;
				room_y=((local_mouse_y-(9+TTF_FontHeight(font)+rdy*2))/2)+y1;
				placing_room=true;

				if (show_grid) {
					int tmp;

					if (room_x<0) {
						tmp=room_x%grid_size;
						if (-tmp>grid_size/2) room_x-=grid_size+tmp;
										 else room_x-=tmp;
					} else {
						tmp=room_x%grid_size;
						if (tmp>grid_size/2) room_x+=grid_size-tmp;
										else room_x-=tmp;
					} /* if */ 
					if (room_y<0) {
						tmp=room_y%grid_size;
						if (-tmp>grid_size/2) room_y-=grid_size+tmp;
										 else room_y-=tmp;
					} else {
						tmp=room_y%grid_size;
						if (tmp>grid_size/2) room_y+=grid_size-tmp;
										else room_y-=tmp;
					} /* if */ 
				} /* if */ 


				if (editor->map->fit_roomp(room_x,room_y,editor->mv_selected->dx,editor->mv_selected->dy)) {
					legal_position=true;
				} else {
					legal_position=false;
				} /* if */ 

				/* Place room on map: */ 
				if (mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1)) &&
					legal_position) {
					if (x1==x2 && y1==y2) {
						editor->mv_selected->x=0;
						editor->mv_selected->y=0;
					} else {
						editor->mv_selected->x=room_x;
						editor->mv_selected->y=room_y;
					} /* if */ 
					editor->mv_selected=0;
					placing_room=false;
				} /* if */ 

				if (mouse_button&SDL_BUTTON(3) &&
					!(old_mouse_button&SDL_BUTTON(3))) {
					/* Cancel placing a room: */ 
					placing_room=false;
					editor->mv_selected=0;
				} /* if */ 
			} else {
				if (placing_room) set_dirty();
				placing_room=false;

				/* Not placing a room: */ 
				if (mouse_button&SDL_BUTTON(3) &&
					!(old_mouse_button&SDL_BUTTON(3))) {
					/* Test if the user has clicked over a room: */ 
					int map_x,map_y;
					ROOM *r;

					map_x=((local_mouse_x-(2+rdx*2))/2)+x1;
					map_y=((local_mouse_y-(9+TTF_FontHeight(font)+rdy*2))/2)+y1;

					r=editor->map->get_room(map_x,map_y);
					if (r!=0) {
						BVChoiceDlg *dlg;
						int res=0;

						dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Remove from map\nEdit room\nCopy\nDelete\n",&res,font);
						dlg->Execute();
						delete dlg;

						if (res==1) {
							r->x=NOPOSITION;
							r->y=NOPOSITION;
							set_dirty();
						} /* if */ 

						if (res==2) {
							editor->room=r;
							editor->view=ROOM_VIEW;
							editor->rv_selected_obj=0;
							editor->rv_selected_tile=0;

							editor->set_all_dirty();
						} /* if */ 

						if (res==3) {
							ROOM *nr;

							nr=new ROOM(*r);
							nr->x=NOPOSITION;
							nr->y=NOPOSITION;
							editor->map->rooms.Add(nr);

							editor->set_all_dirty();
						} /* if */ 

						if (res==4) {
							editor->map->rooms.DeleteElement(r);
							delete r;
							set_dirty();
						} /* if */ 
					} /* if */ 								
				} /* if */ 
			} /* if */ 
		} else {
			if (placing_room) set_dirty();
			placing_room=false;
		} /* if */ 
	} /* if */ 

	return true;
} /* MMapWnd::cycle */ 


void MMapWnd::contens_size(int *dx,int *dy)
{
	char tmp[80];
	int x1,y1,x2,y2,rdx,rdy;
	
	BWindow::contens_size(dx,dy);

	editor->map->CMC(&x1,&y1,&x2,&y2);
	editor->map->BiggestUnplacedRoom(&rdx,&rdy);

	if ((rdx*3+x2-x1)*2+8>*dx) *dx=(rdx*3+x2-x1)*2+8;
	if ((rdy*3+y2-y1)*2+7+TTF_FontHeight(font)+8>*dy) *dy=(rdy*3+y2-y1)*2+7+TTF_FontHeight(font)+8;

	sprintf(tmp,"MAP NAME: %s",editor->map->name);
	if (get_text_width(tmp,font)+6>*dx) *dx=get_text_width(tmp,font)+6;
//	if (int(160+font->get_dx()*strlen("RX: -0000  RY: -0000")+6)>*dx) *dx=int(160+font->get_dx()*strlen("RX: -0000  RY: -0000")+6);
} /* MMapWnd::contens_size */ 


void MMapWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	char tmp[80];
	int x1,y1,x2,y2,rdx,rdy;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	sprintf(tmp,"MAP NAME: %s",editor->map->name);
	print_left(tmp,font,255,255,255,sfc,3-stx,3-sty);

	editor->map->CMC(&x1,&y1,&x2,&y2);
	editor->map->BiggestUnplacedRoom(&rdx,&rdy);
	draw_rectangle(sfc,0-stx,7+TTF_FontHeight(font)-sty,(rdx*3+x2-x1)*2+4,(rdy*3+y2-y1)*2+4,0x00ffffff);

	if (show_grid) {
		int i;
		int startx,starty,endx,endy;

		startx=(x1-rdx)/grid_size;
		starty=(y1-rdy)/grid_size;
		startx*=grid_size;
		starty*=grid_size;
		endx=(x2+rdx*2)/grid_size;
		endy=(y2+rdy*2)/grid_size;
		endx*=grid_size;
		endy*=grid_size;
		for(i=startx;i<=endx;i+=grid_size) {
			draw_line(sfc,2+rdx*2+(i-x1)*2-stx,9+TTF_FontHeight(font)-sty,
						  2+rdx*2+(i-x1)*2-stx,9+TTF_FontHeight(font)+(rdy*3+y2-y1)*2-sty,0x003f3f3f);
		} /* for */ 
		for(i=starty;i<=endy;i+=grid_size) {
			draw_line(sfc,2-stx,9+TTF_FontHeight(font)+rdy*2+(i-y1)*2-sty,
						  2+(rdx*3+x2-x1)*2-stx,9+TTF_FontHeight(font)+rdy*2+(i-y1)*2-sty,0x003f3f3f);
		} /* for */ 
	} /* if */ 

	if (editor->show_miniatures) {
		editor->map->draw(sfc,2+rdx*2-stx,
							  9+TTF_FontHeight(font)+rdy*2-sty,
							  (rdx*2+x2-x1)*2,
							  (rdy*2+y2-y1)*2,
							  0,0,0.125f);
	} else {
		editor->map->draw_frame(sfc,2+rdx*2-stx,
									9+TTF_FontHeight(font)+rdy*2-sty,
									(rdx*2+x2-x1)*2,
									(rdy*2+y2-y1)*2,
									0,0,0.125f);
	} /* if */ 

	if (placing_room) {
		/* Place room on map: */ 
		if (legal_position) {
			draw_rectangle(sfc,2+rdx*2+(room_x-x1)*2-stx,
							   9+TTF_FontHeight(font)+rdy*2+(room_y-y1)*2-sty,
							   editor->mv_selected->dx*2,editor->mv_selected->dy*2,0x0000ff00);
		} else {
			draw_rectangle(sfc,2+rdx*2+(room_x-x1)*2-stx,
							   9+TTF_FontHeight(font)+rdy*2+(room_y-y1)*2-sty,
							   editor->mv_selected->dx*2,editor->mv_selected->dy*2,0x00ff0000);
		} /* if */ 

	} /* if */ 

} /* MMapWnd::draw_contens */ 


bool MMapWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
			/* GRID on/off: */ 
			if (show_grid) {
				if (grid_size==8) {
					grid_size=4;
				} else {
					show_grid=false;
				} /* if */ 
			} else {
				show_grid=true;
				grid_size=8;
			} /* if */ 

			set_dirty();
		}
		break;
	case 2:
		{
			/* FRAMES/MINIATURES: */ 
			if (editor->show_miniatures) editor->show_miniatures=false;
									else editor->show_miniatures=true;

			set_dirty();
		}
	default:
		break;
	} /* switch */ 

	return true;
} /* MMapWnd::button_pressed */ 



bool MMapWnd::event(SDL_Event *event)
{
	if (active_window!=this) return true;

	if (event->type==SDL_KEYDOWN) {
		/* Keyboard event */ 
		if (event->key.keysym.sym==SDLK_UP) {
			old_sty-=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_DOWN) {
			old_sty+=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_LEFT) {
			old_stx-=TILESIZE;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_RIGHT) {
			old_stx+=TILESIZE;
		} /* if */ 

		if (event->key.keysym.sym==SDLK_PAGEUP) {
			old_sty-=TILESIZE*16;
		} /* if */ 
		if (event->key.keysym.sym==SDLK_PAGEDOWN) {
			old_sty+=TILESIZE*16;
		} /* if */ 

		set_dirty();
    } /* if */ 

	return true;
} /* MMapWnd::Event */ 