#include "string.h"
#include "stdlib.h"
#include "math.h"

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "mog2multiplayer.h"
#include "activeobject.h"
#include "activeroom.h"
#include "activeMAP.h"
#include "activemog2.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BListWnd.h"
#include "BMultiListWnd.h"
#include "GamePreviewDlg.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"
#include "POVariableWnd.h"

#include "GameWnd.h"
#include "GameDebugWnd.h"
#include "NetworkConfigDlg.h"
#include "networktest.h"
#include "NetworkGameWnd.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

const int MAX_CONNECTIONS=256;
const int CONNECTION_TIMEOUT=1000;
const int LISTENING_TIME=0;
extern int extra_flags;

BNetworkConfigDlg::BNetworkConfigDlg(int x,int y,int dx,int dy,int gm,char *rf,NWMOG2GameInfo *gi,List<OBJECT> *ch,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;

	b=new BButton(20,dy-20,70,12,"ACCEPT",1);
	buttons.Add(b);
	b=new BButton(110,dy-20,70,12,"CANCEL",2);
	buttons.Add(b);
	b=new BButton(200,dy-20,70,12,"TEST",3);
	buttons.Add(b);
	b=new BButton(290,dy-20,70,12,"PLAY",4);
	buttons.Add(b);

	bdisable=new BButton(10,10,10,10,"",0);
	bserver=new BButton(10,30,10,10,"",0);
	bclient=new BButton(10,50,10,10,"",0);

	game_mode=gm;
	replay_file=rf;

	network_mode=0;
	network_active=false;
	last_message_type=0;

	characters=ch;
	gameinfo=gi;

	strcpy(connect_to,"0.0.0.0");

	lw=new BListWnd(10,70,140,dy-100,&names_list,font);

	current_client=0;
	client_showing=0;
	current_player=0;

} /* BNetworkConfigDlg::BNetworkConfigDlg */ 


BNetworkConfigDlg::~BNetworkConfigDlg()
{
	if (network_active) {
		disconnect();

		draw_message("Network connection finalized");
	} /* if */ 

	characters=0;
	delete bdisable;
	delete bserver;
	delete bclient;
	delete lw;
} /* BNetworkConfigDlg::~BNetworkConfigDlg */ 


bool BNetworkConfigDlg::cycle(int stx,int sty)
{
	bool retval;

	retval=BWindow::cycle(stx,sty);

	{
		List<NWMOG2ClientInfo> l;
		NWMOG2ClientInfo *ci;
		char *tmp;
		int current=0;
		int selected=lw->get_selected();

		names_list.Delete();

		if (selected==current) client_showing=gameinfo->local;
		if (gameinfo->local==gameinfo->server) {
			tmp=new char[80];
			sprintf(tmp,"%s (LOCAL SERVER)",gameinfo->local->computer_name);
			names_list.Add(tmp);
			current++;
		} else {
			tmp=new char[80];
			sprintf(tmp,"%s (LOCAL)",gameinfo->local->computer_name);
			names_list.Add(tmp);
			current++;
		} /* if */ 

		if (gameinfo->server!=0 && gameinfo->server!=gameinfo->local) {
			if (selected==current) client_showing=gameinfo->server;

			tmp=new char[80];
			sprintf(tmp,"%s (SERVER)",gameinfo->server->computer_name);
			names_list.Add(tmp);
			current++;
		} /* if */ 

		l.Instance(gameinfo->clients);
		l.Rewind();
		while(l.Iterate(ci)) {
			if (ci!=gameinfo->local) {
				if (selected==current) client_showing=ci;

				tmp=new char[80];
				strcpy(tmp,ci->computer_name);
				names_list.Add(tmp);
				current++;
			} /* if */ 
		} /* while */ 

		if (selected>=current) {
			client_showing=0;
			lw->set_selected(-1);
		} else {
			lw->set_selected(lw->get_selected());
		} /* if */ 

		lw->set_dirty();
	}


	lw->cycle(stx+10,sty+70);
	if (lw->is_dirty()) set_dirty();

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		if (local_mouse_x>=160 && local_mouse_x<160+get_text_width("999999",font) &&
			local_mouse_y>=10 && local_mouse_y<10+TTF_FontHeight(font)) {
			int x,y;
			BIntegerEdDlg *dlg;

			if (network_active) disconnect();
			network_mode=0;

			new_window_coordinates(220,60,&x,&y);
			dlg=new BIntegerEdDlg(x,y,220,60,"Enter port:",&gameinfo->port,6,font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 

		if (local_mouse_x>=100 && local_mouse_x<150+get_text_width(gameinfo->local->computer_name,font) &&
			local_mouse_y>=30 && local_mouse_y<30+TTF_FontHeight(font)) {
			int x,y;
			BStringEdDlg *dlg;

			if (network_active) disconnect();
			network_mode=0;

			new_window_coordinates(320,60,&x,&y);
			dlg=new BStringEdDlg(x,y,320,60,"Enter computer name:",gameinfo->local->computer_name,40,"",font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 

		if (local_mouse_x>=100 && local_mouse_x<200+get_text_width(connect_to,font) &&
			local_mouse_y>=50 && local_mouse_y<50+TTF_FontHeight(font)) {
			int x,y;
			BStringEdDlg *dlg;

			if (network_active) disconnect();
			network_mode=0;

			new_window_coordinates(320,60,&x,&y);
			dlg=new BStringEdDlg(x,y,320,60,"Enter server address:",connect_to,40," ",font);
			dlg->Execute();
			delete dlg;

			set_dirty();
		} /* if */ 

		if (local_mouse_x>=bdisable->m_x && local_mouse_x<bdisable->m_x+bdisable->m_dx &&
			local_mouse_y>=bdisable->m_y && local_mouse_y<bdisable->m_y+bdisable->m_dy) {
			network_mode=0;
			set_dirty();

			if (network_active) {
				disconnect();

				draw_message("Network connection finalized");
			} /* if */ 
		} /* if */ 

		if (local_mouse_x>=bserver->m_x && local_mouse_x<bserver->m_x+bserver->m_dx &&
			local_mouse_y>=bserver->m_y && local_mouse_y<bserver->m_y+bserver->m_dy) {

			if (network_active) disconnect();

			if (!init_server()) {
				disconnect();
				draw_message("Cannot initialize network");
			} else {
			   draw_message("Network initialized");
			} /* if */ 

			set_dirty();
		} /* if */ 

		if (local_mouse_x>=bclient->m_x && local_mouse_x<bclient->m_x+bclient->m_dx &&
			local_mouse_y>=bclient->m_y && local_mouse_y<bclient->m_y+bclient->m_dy) {

			if (network_active) disconnect();	
			
			if (!init_client()) {
				disconnect();
				draw_message("Cannot initialize network");
			} else {
				draw_message("Network initialized");
			} /* if */ 

			set_dirty();
		} /* if */ 

	} /* if */ 


	/* Network code: */ 
	if (network_active) { 
		int result;
		char data[16];

		set_dirty();

		switch(network_mode) {
		case 1:
			/* SERVER CODE: */ 
			{
				switch(connection_state) {
				case 0:
					{
						int numready;
						/* AWAITING FOR CLIENTS: */ 
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0) {
							List<NWMOG2ClientInfo> l,todisconnect;
							NWMOG2ClientInfo *ci,*ci2;

							l.Instance(gameinfo->clients);
							l.Rewind();
							while(l.Iterate(ci)) {
								if (SDLNet_SocketReady(ci->socket)) {
									/* ALREADY CONNECTED CLIENT REQUESTING TO DISCONNECT: */ 
									todisconnect.Add(ci);
								} /* if */ 
							} /* while */ 

							while(!todisconnect.EmptyP()) {
								ci=todisconnect.ExtractIni();
								gameinfo->clients.DeleteElement(ci);

								SDLNet_Write16(1,data);
								SDLNet_Write16(ci->LVK_ID,data+2);

								l.Instance(gameinfo->clients);
								l.Rewind();
								while(l.Iterate(ci2)) SDLNet_TCP_Send(ci2->socket,data,4);

								SDLNet_TCP_DelSocket(gameinfo->socket_set,ci->socket);
								delete ci;
							} /* while */ 

							if (SDLNet_SocketReady(gameinfo->server_socket)) {
								/* NEW CLIENT REQUESTING FOR CONNECTION: */ 
								TCPsocket s;

								s=SDLNet_TCP_Accept(gameinfo->server_socket);
								result=SDLNet_TCP_Recv(s,data,4);
								if (result<4) {
									disconnect();
									draw_message("Network connection finalized");
								} else {
									last_message_type=result=SDLNet_Read16(data);
									if (result==0) {
										result=SDLNet_TCP_Recv(s,data,8);
										/* incomming connection of a client: */ 
										if (gameinfo->clients.Length()<MAX_CONNECTIONS &&
											SDLNet_Read32(data)==gameinfo->game->ID_low &&
											SDLNet_Read32(data+4)==gameinfo->game->ID_high) {
											int itmp;
											/* Accept client: */ 
											itmp=gameinfo->new_LVK_identifier();
											SDLNet_Write16(2,data);
											SDLNet_Write16(itmp,data+2);
											result=SDLNet_TCP_Send(s,data,4);
											if (result!=4) {
												disconnect();
												draw_message("Network connection finalized");
											} else {
												NWMOG2ClientInfo *ci=new NWMOG2ClientInfo();
												ci->socket=s;
												ci->LVK_ID=itmp;
												if (SDLNet_TCP_AddSocket(gameinfo->socket_set,s)==-1) return false;
												gameinfo->clients.Add(ci);
												current_client=gameinfo->clients.Length()-1;
												connection_state=1;
											} /* if */ 
										} else {
											/* Reject client: */ 
											SDLNet_Write16(3,data);
											SDLNet_Write16(0,data+2);
											result=SDLNet_TCP_Send(s,data,4);
											if (result!=4) {
												disconnect();
												draw_message("Network connection finalized");
											} else {
												SDLNet_TCP_Close(s);
											} /* if */ 
										} /* if */ 
									} /* if */ 
								} /* if */ 
							} /* if */ 
						} /* if */ 
					} 
					break;
				case 1: 
					/* SENDING SERVER INFORMATION TO THE CLIENT: */ 
					{
						List<NWMOG2PlayerInfo> l;
						NWMOG2PlayerInfo *pi;
						char name[80],tmpdata[16];

						SDLNet_Write16(15,data);
						SDLNet_Write16(gameinfo->local->LVK_ID,data+2);
						if (SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,data,4)!=4 ||
							SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,gameinfo->local->computer_name,80)!=80) {
							disconnect();
							draw_message("Network connection finalized");
						} else {
							l.Instance(gameinfo->local->players);
							l.Rewind();
							while(l.Iterate(pi)) {
								SDLNet_Write16(13,data);
								SDLNet_Write16(gameinfo->local->LVK_ID,data+2);
								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,data,4);

								SDLNet_Write16(pi->player_ID,data);
								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,data,2);

								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,pi->name,80);

								strcpy(name,pi->avatar->name);
								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,name,80);

								SDLNet_Write16(pi->left_key,tmpdata);
								SDLNet_Write16(pi->right_key,tmpdata+2);
								SDLNet_Write16(pi->up_key,tmpdata+4);
								SDLNet_Write16(pi->down_key,tmpdata+6);
								SDLNet_Write16(pi->fire1_key,tmpdata+8);
								SDLNet_Write16(pi->fire2_key,tmpdata+10);
								SDLNet_Write16(pi->fire3_key,tmpdata+12);
								SDLNet_Write16(pi->fire4_key,tmpdata+14);
								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,tmpdata,16);

								strcpy(name,pi->start_map->name);
								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,name,80);

								SDLNet_Write16(pi->start_room->get_x(),tmpdata);
								SDLNet_Write16(pi->start_room->get_y(),tmpdata+2);
								SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,tmpdata,4);

							} /* while */ 
							SDLNet_Write16(14,data);
							SDLNet_Write16(gameinfo->local->LVK_ID,data+2);
							if (SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,data,4)!=4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								connection_state=2;
								network_timeout_start=SDL_GetTicks();
								network_timeout=CONNECTION_TIMEOUT;
							} /* if */ 
						} /* if */ 

					}
					break;

				case 2:
					/* OBTAINING INFORMATION OF NEW CLIENT: */ 
					{
						int numready;
						
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0 && SDLNet_SocketReady(gameinfo->clients[current_client]->socket)) {
							/* Answer received: */ 
							if (SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,data,4)!=4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								last_message_type=result=SDLNet_Read16(data);
								if (result==10) {
									NWMOG2ClientInfo *ci;

									ci=gameinfo->find_by_LVK_identifier(SDLNet_Read16(data+2));
									if (ci==0 ||
										SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,ci->computer_name,80)!=80) {
										disconnect();
										draw_message("Network connection finalized");
									} else {
										network_timeout_start=SDL_GetTicks();
										network_timeout=CONNECTION_TIMEOUT;
									} /* if */ 
								} /* if */ 
								if (result==11) {
									SDLNet_Write16(12,data);
									SDLNet_Write16(gameinfo->new_player_identifier(),data+2);
									if (SDLNet_TCP_Send(gameinfo->clients[current_client]->socket,data,4)!=4) {
										disconnect();
										draw_message("Network connection finalized");
									} else {
										network_timeout_start=SDL_GetTicks();
										network_timeout=CONNECTION_TIMEOUT;
									} /* if */ 
								} /* if */ 
								if (result==13) {
									/* RECEIVING PLAYER INFORMATION: */ 

									NWMOG2ClientInfo *ci;

									ci=gameinfo->find_by_LVK_identifier(SDLNet_Read16(data+2));
									if (ci==0) {
										disconnect();
										draw_message("Network connection finalized");
									} else {
										NWMOG2PlayerInfo *pi;
										char name[80],tmpdata[16],name2[80],tmpdata2[4];

										pi=gameinfo->find_by_player_identifier(SDLNet_Read16(data+2));
										if (pi!=0) {
											disconnect();
											draw_message("Network connection finalized");
										} else {
											pi=new NWMOG2PlayerInfo();
											pi->player_ID=SDLNet_Read16(data+2);
											if (SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,data,2)!=2 ||
												SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,pi->name,80)!=80 ||
												SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,name,80)!=80 ||
												SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,tmpdata,16)!=16 ||
												SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,name2,80)!=80 ||
												SDLNet_TCP_Recv(gameinfo->clients[current_client]->socket,tmpdata2,4)!=4) {
												disconnect();
												draw_message("Network connection finalized");
											} else {
												List<OBJECT> l;
												OBJECT *o;

												pi->avatar=0;
												l.Instance(*characters);
												l.Rewind();
												while(l.Iterate(o)) if (o->cmp(name)) pi->avatar=o;
												pi->player_ID=SDLNet_Read16(data);
												pi->left_key=SDLNet_Read16(tmpdata);
												pi->right_key=SDLNet_Read16(tmpdata+2);
												pi->up_key=SDLNet_Read16(tmpdata+4);
												pi->down_key=SDLNet_Read16(tmpdata+6);
												pi->fire1_key=SDLNet_Read16(tmpdata+8);
												pi->fire2_key=SDLNet_Read16(tmpdata+10);
												pi->fire3_key=SDLNet_Read16(tmpdata+12);
												pi->fire4_key=SDLNet_Read16(tmpdata+14);
												pi->vp_area.x=0;
												pi->vp_area.y=0;
												pi->vp_area.w=0;
												pi->vp_area.h=0;
												pi->vp_zoom=1.0F;

												/* Search map: */ 
												{
													MAP *m;
													List<MAP> l;
													bool found=false;

													pi->start_map=0;

													l.Instance(gameinfo->game->maps);
													l.Rewind();
													while(!found && l.Iterate(m)) {
														if (m->cmp(name2)) {
															pi->start_map=m;
															found=true;
														} /* if */ 
													} /* while */ 

													if (found) {
														/* Search room: */ 
														ROOM *r;
														List<ROOM> l;
														found=false;

														pi->start_room=0;
														
														l.Instance(pi->start_map->rooms);
														l.Rewind();
														while(!found && l.Iterate(r)) {
															if (r->get_x()==SDLNet_Read16(tmpdata2) &&
																r->get_y()==SDLNet_Read16(tmpdata2+2)) {
																pi->start_room=r;
																found=true;
															} /* if */ 
														} /* while */ 

														if (found) {
															ci->players.Add(pi);

															network_timeout_start=SDL_GetTicks();
															network_timeout=CONNECTION_TIMEOUT;
														} else {
															disconnect();
															draw_message("Network connection finalized: Map error");
														} /* if */ 
													} else {
														disconnect();
														draw_message("Network connection finalized: Map error");
													} /* if */ 
												}
											} /* if */ 
										} /* if */ 
									} /* if */ 
								} /* if */ 
								if (result==14) {
									connection_state=3;
								} /* if */ 
							} /* if */ 
						} else {
							if (SDL_GetTicks()-network_timeout_start>network_timeout) {
								/* TIMEOUT: */ 
								gameinfo->clients.DeleteElement(gameinfo->clients[current_client]);
								connection_state=0;
							} /* if */ 
						} /* if */ 
					}
					break;

				case 3:
					/* INFORMING TO ALL THE CLIENTS OF THE LAST ADDED CLIENT GAMEINFO: */ 
					{
						List<NWMOG2ClientInfo> cl;
						NWMOG2ClientInfo *ci;
						List<NWMOG2PlayerInfo> l;
						NWMOG2PlayerInfo *pi;
						char name[80],tmpdata[16];
						bool failure;

						cl.Instance(gameinfo->clients);
						
						SDLNet_Write16(10,data);
						SDLNet_Write16(gameinfo->clients[current_client]->LVK_ID,data+2);

						failure=false;
						cl.Rewind();
						while(cl.Iterate(ci)) {
							if (ci!=gameinfo->clients[current_client]) {
								if (SDLNet_TCP_Send(ci->socket,data,4)!=4 ||
									SDLNet_TCP_Send(ci->socket,gameinfo->clients[current_client]->computer_name,80)!=80) {
									failure=true;
								} /* if */ 
							} /* if */ 
						} /* while */ 
						if (failure) {
							disconnect();
							draw_message("Network connection finalized");
						} else {
							l.Instance(gameinfo->clients[current_client]->players);
							l.Rewind();
							while(l.Iterate(pi)) {

								cl.Rewind();
								while(cl.Iterate(ci)) {
									if (ci!=gameinfo->clients[current_client]) {
										SDLNet_Write16(13,data);
										SDLNet_Write16(gameinfo->clients[current_client]->LVK_ID,data+2);
										SDLNet_TCP_Send(ci->socket,data,4);

										SDLNet_Write16(pi->player_ID,data);
										SDLNet_TCP_Send(ci->socket,data,2);

										SDLNet_TCP_Send(ci->socket,pi->name,80);

										strcpy(name,pi->avatar->name);
										SDLNet_TCP_Send(ci->socket,name,80);

										SDLNet_Write16(pi->left_key,tmpdata);
										SDLNet_Write16(pi->right_key,tmpdata+2);
										SDLNet_Write16(pi->up_key,tmpdata+4);
										SDLNet_Write16(pi->down_key,tmpdata+6);
										SDLNet_Write16(pi->fire1_key,tmpdata+8);
										SDLNet_Write16(pi->fire2_key,tmpdata+10);
										SDLNet_Write16(pi->fire3_key,tmpdata+12);
										SDLNet_Write16(pi->fire4_key,tmpdata+14);
										SDLNet_TCP_Send(ci->socket,tmpdata,16);

										strcpy(name,pi->start_map->name);
										SDLNet_TCP_Send(ci->socket,name,80);

										SDLNet_Write16(pi->start_room->get_x(),tmpdata);
										SDLNet_Write16(pi->start_room->get_y(),tmpdata+2);
										SDLNet_TCP_Send(ci->socket,tmpdata,4);
									} /* if */ 
								} /* while */ 
							} /* while */ 

							SDLNet_Write16(14,data);
							SDLNet_Write16(gameinfo->clients[current_client]->LVK_ID,data+2);

							failure=false;
							cl.Rewind();
							while(cl.Iterate(ci)) {
								if (ci!=gameinfo->clients[current_client]) {
									if (SDLNet_TCP_Send(ci->socket,data,4)!=4) failure=true;
								} /* if */ 
							} /* while */ 
							if (failure) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								connection_state=0;
							} /* if */ 
						} /* if */ 

					}
					break;

				case 4:
					/* DISCONNECTING: */ 
					break;
				} /* switch */ 
			}
			break;

		case 2:
			/* CLIENT CODE: */ 
			{
				switch(connection_state) {
				case 0:
					/* CONNECTING: */ 
					SDLNet_Write16(0,data);
					SDLNet_Write16(0,data+2);
					SDLNet_Write32(gameinfo->game->ID_low,data+4);
					SDLNet_Write32(gameinfo->game->ID_high,data+8);

					result=SDLNet_TCP_Send(gameinfo->server_socket,data,12);
					if (result!=12) {
						disconnect();
						draw_message("Network connection finalized");
					} else {
						connection_state=1;
						network_timeout_start=SDL_GetTicks();
						network_timeout=CONNECTION_TIMEOUT;
					} /* if */ 
					break;

				case 1:
					/* AWAITING ANSWER: */ 
					{		
						int numready;
						
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0 && SDLNet_SocketReady(gameinfo->server_socket)) {
							/* Answer received: */ 
							result=SDLNet_TCP_Recv(gameinfo->server_socket,data,4);
							if (result<4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								last_message_type=result=SDLNet_Read16(data);
								if (result==2) {
									/* Connection accepted: */ 
									connection_state=2;
									result=SDLNet_Read16(data+2);
									gameinfo->local->LVK_ID=result;
									gameinfo->clients.Add(gameinfo->local);
									network_timeout_start=SDL_GetTicks();
									network_timeout=CONNECTION_TIMEOUT;
								} /* if */ 
								if (result==3) {
									/* Connection refused: */ 
									disconnect();
									draw_message("Network connection finalized");
								} /* if */ 
							} /* if */ 
						} else {
							if (SDL_GetTicks()-network_timeout_start>network_timeout) {
								/* TIMEOUT: */ 
								connection_state=0;
							} /* if */ 
						} /* if */ 
					}

					break;

				case 2:
					/* RECEIVING SERVER INFORMATION: */ 
					{		
						int numready;
						
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0 && SDLNet_SocketReady(gameinfo->server_socket)) {
							/* Answer received: */ 
							result=SDLNet_TCP_Recv(gameinfo->server_socket,data,4);
							if (result<4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								last_message_type=result=SDLNet_Read16(data);
								if (result==15) {
									/* SERVER NAME: */ 
									if (gameinfo->server==0) {
										gameinfo->server=new NWMOG2ClientInfo();
										gameinfo->server->LVK_ID=SDLNet_Read16(data+2);

										if (SDLNet_TCP_Recv(gameinfo->server_socket,gameinfo->server->computer_name,80)!=80) {
											disconnect();
											draw_message("Network connection finalized");
										} else {
											network_timeout_start=SDL_GetTicks();
											network_timeout=CONNECTION_TIMEOUT;
										} /* if */ 
									} /* if */ 
								} /* if */ 

								if (result==13) {
									NWMOG2ClientInfo *ci;
									/* SERVER PLAYER: */ 

									ci=gameinfo->find_by_LVK_identifier(SDLNet_Read16(data+2));
									if (ci==0) {
										disconnect();
										draw_message("Network connection finalized");
									} else {
										NWMOG2PlayerInfo *pi;
										char avatar_name[80],tmpdata[16],name2[80],tmpdata2[16];

										pi=gameinfo->find_by_player_identifier(SDLNet_Read16(data+2));
										if (pi!=0) {
											disconnect();
											draw_message("Network connection finalized");
										} else {
											pi=new NWMOG2PlayerInfo();
											pi->player_ID=SDLNet_Read16(data+2);
											if (SDLNet_TCP_Recv(gameinfo->server_socket,data,2)!=2 ||
												SDLNet_TCP_Recv(gameinfo->server_socket,pi->name,80)!=80 ||
												SDLNet_TCP_Recv(gameinfo->server_socket,avatar_name,80)!=80 ||
												SDLNet_TCP_Recv(gameinfo->server_socket,tmpdata,16)!=16 ||
												SDLNet_TCP_Recv(gameinfo->server_socket,name2,80)!=80 ||
												SDLNet_TCP_Recv(gameinfo->server_socket,tmpdata2,4)!=4) {
												disconnect();
												draw_message("Network connection finalized");
											} else {
												List<OBJECT> l;
												OBJECT *o;

												pi->avatar=0;
												l.Instance(*characters);
												l.Rewind();
												while(l.Iterate(o)) if (o->cmp(avatar_name)) pi->avatar=o;
												pi->player_ID=SDLNet_Read16(data);
												pi->left_key=SDLNet_Read16(tmpdata);
												pi->right_key=SDLNet_Read16(tmpdata+2);
												pi->up_key=SDLNet_Read16(tmpdata+4);
												pi->down_key=SDLNet_Read16(tmpdata+6);
												pi->fire1_key=SDLNet_Read16(tmpdata+8);
												pi->fire2_key=SDLNet_Read16(tmpdata+10);
												pi->fire3_key=SDLNet_Read16(tmpdata+12);
												pi->fire4_key=SDLNet_Read16(tmpdata+14);
												pi->vp_area.x=0;
												pi->vp_area.y=0;
												pi->vp_area.w=0;
												pi->vp_area.h=0;
												pi->vp_zoom=1.0F;

												/* Search map: */ 
												{
													MAP *m;
													List<MAP> l;
													bool found=false;

													pi->start_map=0;

													l.Instance(gameinfo->game->maps);
													l.Rewind();
													while(!found && l.Iterate(m)) {
														if (m->cmp(name2)) {
															pi->start_map=m;
															found=true;
														} /* if */ 
													} /* while */ 

													if (found) {
														/* Search room: */ 
														ROOM *r;
														List<ROOM> l;
														found=false;

														pi->start_room=0;
														
														l.Instance(pi->start_map->rooms);
														l.Rewind();
														while(!found && l.Iterate(r)) {
															if (r->get_x()==SDLNet_Read16(tmpdata2) &&
																r->get_y()==SDLNet_Read16(tmpdata2+2)) {
																pi->start_room=r;
																found=true;
															} /* if */ 
														} /* while */ 

														if (found) {
															ci->players.Add(pi);

															network_timeout_start=SDL_GetTicks();
															network_timeout=CONNECTION_TIMEOUT;
														} else {
															disconnect();
															draw_message("Network connection finalized: Map error");
														} /* if */ 
													} else {
														disconnect();
														draw_message("Network connection finalized: Map error");
													} /* if */ 
												}
											} /* if */ 
										} /* if */ 
									} /* if */ 
								} /* if */ 
								if (result==14) {
									/* FINISHED SERVER INFO: */ 
									connection_state=3;
								} /* if */ 
							} /* if */ 
						} else {
							if (SDL_GetTicks()-network_timeout_start>network_timeout) {
								/* TIMEOUT: */ 
								connection_state=0;
							} /* if */ 
						} /* if */ 
					}
					break;

				case 3:
					/* SENDING LOCAL INFORMATION: */ 
					{
						SDLNet_Write16(10,data);
						SDLNet_Write16(gameinfo->local->LVK_ID,data+2);

						result=SDLNet_TCP_Send(gameinfo->server_socket,data,4);
						if (result!=4) {
							disconnect();
							draw_message("Network connection finalized");
						} /* if */ 

						result=SDLNet_TCP_Send(gameinfo->server_socket,gameinfo->local->computer_name,80);
						if (result!=80) {
							disconnect();
							draw_message("Network connection finalized");
						} else {
							connection_state=4;
							network_timeout_start=SDL_GetTicks();
							network_timeout=CONNECTION_TIMEOUT;
							current_player=0;
						} /* if */ 
					}
					break;

				case 4:
					/* SENDING LOCAL PLAYER INFORMATION */ 
					{
						if (gameinfo->local->players.Length()>current_player) {
							SDLNet_Write16(11,data);
							SDLNet_Write16(0,data+2);

							result=SDLNet_TCP_Send(gameinfo->server_socket,data,4);
							if (result!=4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								network_timeout_start=SDL_GetTicks();
								network_timeout=CONNECTION_TIMEOUT;
								connection_state=5;
							} /* if */ 
						} else {
							SDLNet_Write16(14,data);
							SDLNet_Write16(gameinfo->local->LVK_ID,data+2);

							result=SDLNet_TCP_Send(gameinfo->server_socket,data,4);
							if (result!=4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								connection_state=6;
							} /* if */ 
						} /* if */ 
					}
					break;

				case 5:
					/* WAITING FOR SERVER PLAYER_ID: */ 
					{		
						int numready;
						
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0 && SDLNet_SocketReady(gameinfo->server_socket)) {
							/* Answer received: */ 
							if (SDLNet_TCP_Recv(gameinfo->server_socket,data,4)!=4 ||
								SDLNet_Read16(data)!=12) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								char name[80],tmpdata[16];
								NWMOG2PlayerInfo *pi=gameinfo->local->players[current_player];

								pi->player_ID=SDLNet_Read16(data+2);

								SDLNet_Write16(13,data);
								SDLNet_Write16(gameinfo->local->LVK_ID,data+2);
								SDLNet_TCP_Send(gameinfo->server_socket,data,4);

								SDLNet_Write16(pi->player_ID,data);
								SDLNet_TCP_Send(gameinfo->server_socket,data,2);

								SDLNet_TCP_Send(gameinfo->server_socket,pi->name,80);

								strcpy(name,pi->avatar->name);
								SDLNet_TCP_Send(gameinfo->server_socket,name,80);

								SDLNet_Write16(pi->left_key,tmpdata);
								SDLNet_Write16(pi->right_key,tmpdata+2);
								SDLNet_Write16(pi->up_key,tmpdata+4);
								SDLNet_Write16(pi->down_key,tmpdata+6);
								SDLNet_Write16(pi->fire1_key,tmpdata+8);
								SDLNet_Write16(pi->fire2_key,tmpdata+10);
								SDLNet_Write16(pi->fire3_key,tmpdata+12);
								SDLNet_Write16(pi->fire4_key,tmpdata+14);
								SDLNet_TCP_Send(gameinfo->server_socket,tmpdata,16);

								strcpy(name,pi->start_map->name);
								SDLNet_TCP_Send(gameinfo->server_socket,name,80);

								SDLNet_Write16(pi->start_room->get_x(),tmpdata);
								SDLNet_Write16(pi->start_room->get_y(),tmpdata+2);
								SDLNet_TCP_Send(gameinfo->server_socket,tmpdata,4);

								current_player++;
								connection_state=4;
							} /* if */ 
						} else {
							if (SDL_GetTicks()-network_timeout_start>network_timeout) {
								network_timeout_start=SDL_GetTicks();
								network_timeout=CONNECTION_TIMEOUT;
								connection_state=4;
							} /* if */ 
						} /* if */ 


					}
					break;

				case 6:
					/* WAITING FOR SERVER UPDATES IN THE GAMEINFO AND THE START SIGNAL: */ 
					{		
						int numready;
						NWMOG2ClientInfo *ci;
						NWMOG2PlayerInfo *pi;
						
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0 && SDLNet_SocketReady(gameinfo->server_socket)) {
							/* Answer received: */ 
							if (SDLNet_TCP_Recv(gameinfo->server_socket,data,4)!=4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								last_message_type=result=SDLNet_Read16(data);
								if (result==1) {
									/* Disconnect client/server: */ 
									ci=gameinfo->find_by_LVK_identifier(SDLNet_Read16(data+2));
									if (ci==gameinfo->server ||
										ci==gameinfo->local) {
										disconnect();
										draw_message("Network connection finalized");
									} else {
										gameinfo->clients.DeleteElement(ci);
										delete ci;
									} /* if */ 
								} /* if */ 

								if (result==10) {
									/* A new client: */ 
									ci=gameinfo->find_by_LVK_identifier(SDLNet_Read16(data+2));
									if (ci==0) {
										ci=new NWMOG2ClientInfo();
										ci->LVK_ID=SDLNet_Read16(data+2);
										gameinfo->clients.Add(ci);
									} /* if */ 

									if (SDLNet_TCP_Recv(gameinfo->server_socket,ci->computer_name,80)!=80) {
										disconnect();
										draw_message("Network connection finalized");
									} /* if */ 
								} /* if */ 

								if (result==13) {
									char avatar_name[80],tmpdata[16],name2[80],tmpdata2[16];
									/* A new player: */ 
									ci=gameinfo->find_by_LVK_identifier(SDLNet_Read16(data+2));
									if (ci==0) {
										disconnect();
										draw_message("Network connection finalized");
									} else {
										pi=gameinfo->find_by_player_identifier(SDLNet_Read16(data+2));
										if (pi==0) {
											pi=new NWMOG2PlayerInfo();
											ci->players.Add(pi);
										} /* if */ 
										pi->player_ID=SDLNet_Read16(data+2);
										if (SDLNet_TCP_Recv(gameinfo->server_socket,data,2)!=2 ||
											SDLNet_TCP_Recv(gameinfo->server_socket,pi->name,80)!=80 ||
											SDLNet_TCP_Recv(gameinfo->server_socket,avatar_name,80)!=80 ||
											SDLNet_TCP_Recv(gameinfo->server_socket,tmpdata,16)!=16 ||
											SDLNet_TCP_Recv(gameinfo->server_socket,name2,80)!=80 ||
											SDLNet_TCP_Recv(gameinfo->server_socket,tmpdata2,4)!=4) {
											disconnect();
											draw_message("Network connection finalized");
										} else {
											List<OBJECT> l;
											OBJECT *o;

											pi->avatar=0;
											l.Instance(*characters);
											l.Rewind();
											while(l.Iterate(o)) if (o->cmp(avatar_name)) pi->avatar=o;
											pi->player_ID=SDLNet_Read16(data);
											pi->left_key=SDLNet_Read16(tmpdata);
											pi->right_key=SDLNet_Read16(tmpdata+2);
											pi->up_key=SDLNet_Read16(tmpdata+4);
											pi->down_key=SDLNet_Read16(tmpdata+6);
											pi->fire1_key=SDLNet_Read16(tmpdata+8);
											pi->fire2_key=SDLNet_Read16(tmpdata+10);
											pi->fire3_key=SDLNet_Read16(tmpdata+12);
											pi->fire4_key=SDLNet_Read16(tmpdata+14);

											/* Search map: */ 
											{
												MAP *m;
												List<MAP> l;
												bool found=false;

												pi->start_map=0;

												l.Instance(gameinfo->game->maps);
												l.Rewind();
												while(!found && l.Iterate(m)) {
													if (m->cmp(name2)) {
														pi->start_map=m;
														found=true;
													} /* if */ 
												} /* while */ 

												if (found) {
													/* Search room: */ 
													ROOM *r;
													List<ROOM> l;
													found=false;

													pi->start_room=0;
													
													l.Instance(pi->start_map->rooms);
													l.Rewind();
													while(!found && l.Iterate(r)) {
														if (r->get_x()==SDLNet_Read16(tmpdata2) &&
															r->get_y()==SDLNet_Read16(tmpdata2+2)) {
															pi->start_room=r;
															found=true;
														} /* if */ 
													} /* while */ 

													if (!found) {
														disconnect();
														draw_message("Network connection finalized: Map error");
													} /* if */ 
												} else {
													disconnect();
													draw_message("Network connection finalized: Map error");
												} /* if */ 
											}
										} /* if */ 
									} /* if */ 
								} /* if */ 

								if (result==16) connection_state=7;
							} /* if */ 
						} /* if */ 
					}
					break;

				case 7:
					/* READY TO PLAY: */ 
					{		
						int numready;
						
						numready=SDLNet_CheckSockets(gameinfo->socket_set, LISTENING_TIME);

						if (numready!=0 && SDLNet_SocketReady(gameinfo->server_socket)) {
							if (SDLNet_TCP_Recv(gameinfo->server_socket,data,4)!=4) {
								disconnect();
								draw_message("Network connection finalized");
							} else {
								result=SDLNet_Read16(data);

								if (result==26) connection_state=6;
							} /* if */ 
						} /* if */ 
					}
					break;

				} /* switch */ 
			}			
			break;
		} /* switch */ 
	} /* if */ 

	return retval;
} /* BNetworkConfigDlg::cycle */ 



void BNetworkConfigDlg::contens_size(int *dx,int *dy)
{
	BActiveWindow::contens_size(dx,dy);

	if (old_dx>*dx) *dx=old_dx;
	if (old_dy>*dy) *dy=old_dy;
} /* BScreenConfigDlg::contens_size */ 


void BNetworkConfigDlg::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;
	char tmp[256];

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	local_draw_button(sfc,bdisable,stx,sty,(network_mode==0 ? true : false));
	local_draw_button(sfc,bserver,stx,sty,(network_mode==1 ? true : false));
	local_draw_button(sfc,bclient,stx,sty,(network_mode==2 ? true : false));
	print_left("DISABLE NETWORK",font,255,255,255,sfc,30-stx,10-sty);
	print_left("SERVER",font,255,255,255,sfc,30-stx,30-sty);
	print_left("CLIENT",font,255,255,255,sfc,30-stx,50-sty);

	print_left("PORT:",font,255,255,255,sfc,160-stx,10-sty);
	sprintf(tmp,"%i",gameinfo->port);
	print_left(tmp,font,255,255,255,sfc,210-stx,10-sty);

	print_left("NAME:",font,255,255,255,sfc,100-stx,30-sty);
	print_left(gameinfo->local->computer_name,font,255,255,255,sfc,150-stx,30-sty);
	print_left("CONNECT TO:",font,255,255,255,sfc,100-stx,50-sty);
	print_left(connect_to,font,255,255,255,sfc,200-stx,60-sty);

	lw->draw(sfc,10-stx,70-sty,140,dy-100);

	/* Draw info of selected client: */ 
	if (client_showing!=0) {
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pi;
		int y;

		sprintf(tmp,"NAME: %s",client_showing->computer_name);
		print_left(tmp,font,255,255,255,sfc,160-stx,70-sty);
		if (client_showing->computer_address[0]==0 && client_showing->socket!=0) {
			IPaddress *ip;
			unsigned char *p;
			ip=SDLNet_TCP_GetPeerAddress(client_showing->socket);
			p=(unsigned char *)ip;
			sprintf(client_showing->computer_address,"%i.%i.%i.%i",int(p[0]),int(p[1]),int(p[2]),int(p[3]));
		} /* if */ 
		sprintf(tmp,"ADDRESS: %s",client_showing->computer_address);
		print_left(tmp,font,255,255,255,sfc,160-stx,80-sty);
		sprintf(tmp,"GAME ID: %u,%u",gameinfo->game->ID_high,gameinfo->game->ID_low);
		print_left(tmp,font,255,255,255,sfc,160-stx,90-sty);
		sprintf(tmp,"LVK_ID: %i",client_showing->LVK_ID);
		print_left(tmp,font,255,255,255,sfc,160-stx,100-sty);
		print_left("PLAYERS:",font,255,255,255,sfc,160-stx,110-sty);
		y=120;

		l.Instance(client_showing->players);
		l.Rewind();
		while(l.Iterate(pi)) {
			if (pi->avatar!=0) {
				sprintf(tmp,"N:	\"%s\", AVT: \"%s\", ID: %i",pi->name,pi->avatar->name,pi->player_ID);
				print_left(tmp,font,255,255,255,sfc,170-stx,y-sty);
				y+=10;
			} else {
				sprintf(tmp,"N:	\"%s\", AVT: -, ID: %i",pi->name,pi->player_ID);
				print_left(tmp,font,255,255,255,sfc,170-stx,y-sty);
				y+=10;
			} /* if */ 
			if (pi->start_map!=0) {
				sprintf(tmp,"   MAP: \"%s\"",pi->start_map->name);
				print_left(tmp,font,255,255,255,sfc,170-stx,y-sty);
				y+=10;
			} else {
				sprintf(tmp,"   MAP: -");
				print_left(tmp,font,255,255,255,sfc,170-stx,y-sty);
				y+=10;
			} /* if */ 
			if (pi->start_room!=0) {
				sprintf(tmp,"   ROOM: %i,%i",pi->start_room->get_x(),pi->start_room->get_y());
				print_left(tmp,font,255,255,255,sfc,170-stx,y-sty);
				y+=10;
			} else {
				sprintf(tmp,"   ROOM: -");
				print_left(tmp,font,255,255,255,sfc,170-stx,y-sty);
				y+=10;
			} /* if */ 
		} /* while */ 
		client_showing=0;
	} /* if */ 

	/* Draw connection status: */ 
	switch(network_mode) {
	case 1:
		/* SERVER: */ 
		sprintf(tmp,"SERVER STATE: %i",connection_state);
		print_left(tmp,font,255,255,255,sfc,270-stx,10-sty);
		sprintf(tmp,"N CLIENTS: %i",gameinfo->clients.Length());
		print_left(tmp,font,255,255,255,sfc,270-stx,20-sty);
		if (connection_state==2) {
			sprintf(tmp,"TIMEOUT: %i",network_timeout-(SDL_GetTicks()-network_timeout_start));
			print_left(tmp,font,255,255,255,sfc,270-stx,30-sty);
		} /* if */ 
		sprintf(tmp,"LAST MESSAGE: %i",last_message_type);
		print_left(tmp,font,255,255,255,sfc,270-stx,40-sty);
		break;
	case 2:
		/* CLIENT: */ 
		sprintf(tmp,"CLIENT STATE: %i",connection_state);
		print_left(tmp,font,255,255,255,sfc,270-stx,10-sty);
		if (connection_state==1 || connection_state==2 || connection_state==5) {
			sprintf(tmp,"TIMEOUT: %i",network_timeout-(SDL_GetTicks()-network_timeout_start));
			print_left(tmp,font,255,255,255,sfc,270-stx,20-sty);
		} /* if */ 
		sprintf(tmp,"LAST MESSAGE: %i",last_message_type);
		print_left(tmp,font,255,255,255,sfc,270-stx,30-sty);
		break;
	} /* switch */ 

} /* BNetworkConfigDlg::draw_contens */ 



bool BNetworkConfigDlg::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
			/* ACCEPT: */ 

			return false;
		}
		break;

	case 2:
		return false;
		break;

	case 3:
		/* TEST: */ 
		{
			int x,y;
			BNetworkTestWnd *dlg;

			if (network_mode==1) {				
				List<NWMOG2ClientInfo> cl;
				NWMOG2ClientInfo *ci;
				char data[16];
				
				cl.Instance(gameinfo->clients);
						
				/* Send start message: */ 
				SDLNet_Write16(16,data);
				SDLNet_Write16(0,data+2);			
				cl.Rewind();
				while(cl.Iterate(ci)) SDLNet_TCP_Send(ci->socket,data,4);

				new_window_coordinates(240,160,&x,&y);
				dlg=new BNetworkTestWnd(x,y,240,160,gameinfo,font);
				dlg->Execute();
				delete dlg;
				
				/* Send End message: */ 
				SDLNet_Write16(26,data);
				SDLNet_Write16(0,data+2);
				cl.Rewind();
				while(cl.Iterate(ci)) SDLNet_TCP_Send(ci->socket,data,4);

			} /* if */ 

			if (network_mode==2 && connection_state==7) {
				char data[16];

				/* Send start message: */ 
				SDLNet_Write16(17,data);
				SDLNet_Write16(0,data+2);				
				SDLNet_TCP_Send(gameinfo->server_socket,data,4);

				new_window_coordinates(240,160,&x,&y);
				dlg=new BNetworkTestWnd(x,y,240,160,gameinfo,font);
				dlg->Execute();
				delete dlg;

				/* Send End message: */ 
				SDLNet_Write16(26,data);
				SDLNet_Write16(0,data+2);		
				SDLNet_TCP_Send(gameinfo->server_socket,data,4);

				connection_state=6;
			} /* if */ 
		
		}
		break;

	case 4:
		/* PLAY: */ 
		{
			int old_extra_flags=extra_flags;
			NetworkGameWindow *dlg;

			if (gameinfo->render==0) {
				/* SOFTWARE RENDERING */ 
				extra_flags=0;
			} else {
				/* HARDWARE RENDERING */ 
				extra_flags=SDL_HWSURFACE|SDL_DOUBLEBUF;
			} /* if */ 

			if (network_mode==1) {				
				List<NWMOG2ClientInfo> cl;
				NWMOG2ClientInfo *ci;
				char data[16];
				
				cl.Instance(gameinfo->clients);
						
				/* Send start message: */ 
				SDLNet_Write16(16,data);
				SDLNet_Write16(0,data+2);			
				cl.Rewind();
				while(cl.Iterate(ci)) SDLNet_TCP_Send(ci->socket,data,4);

				dlg=new NetworkGameWindow(gameinfo->screen_x,gameinfo->screen_y,game_mode,replay_file,gameinfo,font);
				dlg->Execute();
				delete dlg;
				
				/* Send End message: */ 
				SDLNet_Write16(26,data);
				SDLNet_Write16(0,data+2);
				cl.Rewind();
				while(cl.Iterate(ci)) SDLNet_TCP_Send(ci->socket,data,4);

			} /* if */ 

			if (network_mode==2 && connection_state==7) {
				char data[16];

				/* Send start message: */ 
				SDLNet_Write16(17,data);
				SDLNet_Write16(0,data+2);				
				SDLNet_TCP_Send(gameinfo->server_socket,data,4);

				dlg=new NetworkGameWindow(gameinfo->screen_x,gameinfo->screen_y,game_mode,replay_file,gameinfo,font);
				dlg->Execute();
				delete dlg;

				/* Send End message: */ 
				SDLNet_Write16(26,data);
				SDLNet_Write16(0,data+2);		
				SDLNet_TCP_Send(gameinfo->server_socket,data,4);

				connection_state=6;
			} /* if */ 

			extra_flags=old_extra_flags;
		
		}
		break;

	} /* switch */ 

	return true;
} /* BScreenConfigDlg::button_pressed */ 


bool BNetworkConfigDlg::init_server(void)
{
	IPaddress ip;

	if (SDLNet_Init()==-1) return false;

	if (gameinfo->socket_set==0) gameinfo->socket_set=SDLNet_AllocSocketSet(MAX_CONNECTIONS+1);

	/* Initialize Server: */ 
	if (SDLNet_ResolveHost(&ip,0,gameinfo->port)==-1) return false;

	/* Create listening socket: */ 
	gameinfo->server_socket=SDLNet_TCP_Open(&ip);
	if (gameinfo->server_socket==0) return false;

	if (SDLNet_TCP_AddSocket(gameinfo->socket_set,gameinfo->server_socket)==-1) return false;

	gameinfo->server=gameinfo->local;
	gameinfo->server->LVK_ID=0;
	{
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pi;

		l.Instance(gameinfo->server->players);
		l.Rewind();
		while(l.Iterate(pi)) pi->player_ID=gameinfo->new_player_identifier();
	}

	connection_state=0;
	network_active=true;
	network_mode=1;
	return true;
} /* BNetworkConfigDlg::init_server */ 


bool BNetworkConfigDlg::init_client(void)
{
	IPaddress ip;

	if (SDLNet_Init()==-1) return false;

	if (gameinfo->socket_set==0) gameinfo->socket_set=SDLNet_AllocSocketSet(MAX_CONNECTIONS+1);

	/* Initialize Client: */ 
	if (SDLNet_ResolveHost(&ip,connect_to,gameinfo->port)==-1) return false;

	/* Create talking socket with the server: */ 
	gameinfo->server_socket=SDLNet_TCP_Open(&ip);
	if (gameinfo->server_socket==0) return false;

	if (SDLNet_TCP_AddSocket(gameinfo->socket_set,gameinfo->server_socket)==-1) return false;

	connection_state=0;
	network_active=true;
	network_mode=2;
	return true;
} /* BNetworkConfigDlg::init_client */ 


void BNetworkConfigDlg::disconnect(void)
{
	if (network_mode==1) {
		/* I was a server: */ 
		char data[16];
		List<NWMOG2ClientInfo> l;
		NWMOG2ClientInfo *ci;

		SDLNet_Write16(1,data);
		SDLNet_Write16(gameinfo->local->LVK_ID,data);

		l.Instance(gameinfo->clients);
		l.Rewind();
		while(l.Iterate(ci)) SDLNet_TCP_Send(ci->socket,data,4);
	} /* if */ 

	if (network_mode==2) {
		/* I was a client: */ 
		char data[16];

		SDLNet_Write16(1,data);
		SDLNet_Write16(gameinfo->local->LVK_ID,data);
		SDLNet_TCP_Send(gameinfo->server_socket,data,4);
	} /* if */ 

	/* Wait for the other coputers to have time to receive the disconnect message: */ 
	pause(CONNECTION_TIMEOUT);

	SDLNet_FreeSocketSet(gameinfo->socket_set);
	gameinfo->socket_set=0;

	gameinfo->clients.DeleteElement(gameinfo->local);
	gameinfo->clients.Delete();
	SDLNet_TCP_Close(gameinfo->server_socket);

	SDLNet_Quit();

	if (gameinfo->server!=gameinfo->local) delete gameinfo->server;
	gameinfo->server=0;
	{
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pi;

		gameinfo->local->LVK_ID=-1;
		l.Instance(gameinfo->local->players);
		l.Rewind();
		while(l.Iterate(pi)) pi->player_ID=-1;
	}
	current_client=0;
	client_showing=0;
	current_player=0;

	network_active=false;
	network_mode=0;
} /* BNetworkConfigDlg::disconnect */ 
