#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_image.h"

#include "List.h"
#include "collision.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "tiles.h"

#include "BWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"

#include "OBitmapWnd.h"
#include "interfaces.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int surface_flags;

OBitmapWnd::OBitmapWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font)
{
	BButton *b;

	editor=e;

	b=new BButton(4,0,100,12,"NEW BITMAP",1);
	buttons.Add(b);

	b=new BButton(158,0,20,12,"+",2);
	buttons.Add(b);
	b=new BButton(114,0,20,12,"-",3);
	buttons.Add(b);

} /* OBitmapWnd::OBitmapWnd */ 


OBitmapWnd::~OBitmapWnd()
{
} /* OBitmapWnd::~OBitmapWnd */ 


bool OBitmapWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (((mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3))) ||
		 (mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<TILE> l,todelete;
		TILE *t;
		int y;

		y=25+TTF_FontHeight(font);
		l.Instance(editor->game->tiles[editor->ov_bitmap_category]);
		l.Rewind();
		while(local_mouse_y>=y && l.Iterate(t)) {
			if (local_mouse_y>=y && local_mouse_y<y+t->r.h+4 &&
				local_mouse_x>=0 && local_mouse_x<t->r.w+4) {
				/* BITMAPS OPTIONS: */ 

				if (mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1)) &&
					editor->ov_selected!=0 &&
					editor->ov_selected_ob>=0) {
					editor->ov_selected->tiles[editor->ov_selected_ob]=t;
					editor->set_all_dirty();
				} /* if */ 

				if (mouse_button&SDL_BUTTON(3) &&
					!(old_mouse_button&SDL_BUTTON(3))) {

					BVChoiceDlg *dlg;
					int res=0;

					dlg=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Delete\nAssign Mask\nAssign Collision Mask\nRegrab\n",&res,font);
					dlg->Execute();
					delete dlg;

					if (res==1) {
						todelete.Add(t);
						editor->set_all_dirty();
					} /* if */ 

					if (res==2) {
						BVChoiceDlg *dlg2;

						dlg2=new BVChoiceDlg(-1,-1,0,0,"Choose Mask:","No Mask\nAutomatic Mask\nFrom Bitmap\n",&res,font);
						dlg2->Execute();
						delete dlg2;

						if (res==1) {
							SDL_SetAlpha(t->orig,0,SDL_ALPHA_OPAQUE);
							t->mask_type=0;

							SDL_FreeSurface(t->mask_visualization);
							t->mask_visualization=0;
						} /* if */ 

						if (res==2) {
							SDL_SetAlpha(t->orig,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
							surface_automatic_alpha(t->orig);
							t->mask_type=1;

							SDL_FreeSurface(t->mask_visualization);
							t->mask_visualization=0;
						} /* if */ 

						if (res==3) {
							List<TILE_SOURCE> l;
							TILE_SOURCE *ts;
							bool found=false;

							l.Instance(editor->game->tile_sources);
							l.Rewind();
							while(!found && l.Iterate(ts)) {
								if (ts->cmp(t->orig_name)) {
									SDL_SetAlpha(t->orig,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
									surface_mask_from_bitmap(t->orig,ts->sfc,t->r.x+t->r.w,t->r.y);
									found=true;
									t->mask_type=2;

									SDL_FreeSurface(t->mask_visualization);
									t->mask_visualization=0;
								} /* if */ 
							} /* while */ 
						} /* if */ 
						set_dirty();
					} /* if */ 

					if (res==3) {
						BVChoiceDlg *dlg2;

						dlg2=new BVChoiceDlg(-1,-1,0,0,"Choose Mask:","No collision\nFull collision\nFrom Bitmap\nSame as Transparency\n",&res,font);
						dlg2->Execute();
						delete dlg2;

						if (res==1) {
							if (t->mask_collision==0) t->mask_collision=SDL_CreateRGBSurface(surface_flags,t->r.w,t->r.h,32,0,0,0,0);
							SDL_FillRect(t->mask_collision,0,0x00000000);
							if (t->base_collision_data!=0) MOG2_destroy_cmap(t->base_collision_data);
							t->base_collision_data=MOG2_make_cmap(t->mask_collision);
							t->collision_mask_type=0;
						} /* if */ 

						if (res==2) {
							if (t->mask_collision==0) t->mask_collision=SDL_CreateRGBSurface(surface_flags,t->r.w,t->r.h,32,0,0,0,0);
							SDL_FillRect(t->mask_collision,0,0xffffffff);
							if (t->base_collision_data!=0) MOG2_destroy_cmap(t->base_collision_data);
							t->base_collision_data=MOG2_make_cmap(t->mask_collision);
							t->collision_mask_type=1;
						} /* if */ 
						
						if (res==3) {
							List<TILE_SOURCE> l;
							TILE_SOURCE *ts;
							bool found=false;

							l.Instance(editor->game->tile_sources);
							l.Rewind();
							while(!found && l.Iterate(ts)) {
								if (ts->cmp(t->orig_name)) {
									SDL_Rect o;

									o.x=t->r.x+(t->r.w*2);
									o.y=t->r.y;
									o.w=t->r.w;
									o.h=t->r.h;

									if (t->mask_collision==0) t->mask_collision=SDL_CreateRGBSurface(surface_flags,t->r.w,t->r.h,32,0,0,0,0);
									SDL_BlitSurface(ts->sfc,&o,t->mask_collision,0);

									if (t->base_collision_data!=0) MOG2_destroy_cmap(t->base_collision_data);
									t->base_collision_data=MOG2_make_cmap(t->mask_collision);
									t->collision_mask_type=2;
									found=true;
								} /* if */ 
							} /* while */ 
						} /* if */ 

						if (res==4) {
							List<TILE_SOURCE> l;
							TILE_SOURCE *ts;
							bool found=false;

							l.Instance(editor->game->tile_sources);
							l.Rewind();
							while(!found && l.Iterate(ts)) {
								if (ts->cmp(t->orig_name)) {
									SDL_Rect o;

									o.x=t->r.x+(t->r.w);
									o.y=t->r.y;
									o.w=t->r.w;
									o.h=t->r.h;

									if (t->mask_collision==0) t->mask_collision=SDL_CreateRGBSurface(surface_flags,t->r.w,t->r.h,32,0,0,0,0);
									SDL_BlitSurface(ts->sfc,&o,t->mask_collision,0);

									if (t->base_collision_data!=0) MOG2_destroy_cmap(t->base_collision_data);
									t->base_collision_data=MOG2_make_cmap(t->mask_collision);
									t->collision_mask_type=3;
									found=true;
								} /* if */ 
							} /* while */ 
						} /* if */ 
					
						set_dirty();
						res=0;
					} /* if */ 

					if (res==4) {
						/* Regrab BITMAP: */ 
						char tmp[256]="*.*";
						BFileDlg *dlg=new BFileDlg(30,30,200,200,"Choose a file from where to grab","Select",tmp,editor->font);

						dlg->Execute();
						delete dlg;

						if (tmp[0]!=0) {
							SDL_Surface *bmp_sfc,*tmp_sfc;

							tmp_sfc = IMG_Load(tmp);
							if (tmp_sfc!=0) {
								bmp_sfc = SDL_CreateRGBSurface(surface_flags,tmp_sfc->w,tmp_sfc->h,32,0,0,0,AMASK);
								SDL_SetAlpha(bmp_sfc,0,SDL_ALPHA_OPAQUE);
								SDL_BlitSurface(tmp_sfc,0,bmp_sfc,0);
								SDL_FreeSurface(tmp_sfc);

								/* Grab bitmaps: */ 
								editor->ov_regrabbing=t;
								editor->grabv_bitmap=bmp_sfc;
								strcpy(editor->grabv_bitmap_file,tmp);
								editor->view=GRABBER_VIEW;
								editor->grabv_state=0;
								editor->set_all_dirty();
							} else {
								int res;
								BChoiceDlg *dlg2=new BChoiceDlg(40,40,200,50,"Cannot open file","Accept",&res,editor->font);
								dlg2->Execute();
								delete dlg2;
							} /* if */ 
						} /* if */ 
					} /* if */ 

				} /* if */ 
			} /* if */ 
				
			y+=t->r.h+6;
		} /* while */ 

		while(!todelete.EmptyP()) {
			TILE *t;
			t=todelete.ExtractIni();
			editor->delete_tile(t);
		} /* while */ 
		
	} /* if */ 

	return true;
} /* OBitmapWnd::cycle */ 


void OBitmapWnd::contens_size(int *dx,int *dy)
{
	List<TILE> l;
	TILE *t;
	int y;
	int max_x=0;
	int max_y=0;
	
	BWindow::contens_size(dx,dy);
	
	max_x=get_text_width("AVAILABLE BITMAPS:",font);

	y=25+TTF_FontHeight(font);
	l.Instance(editor->game->tiles[editor->ov_bitmap_category]);
	l.Rewind();
	while(l.Iterate(t)) {
		if ((t->r.w+8)*3+4>max_x) max_x=(t->r.w+8)*3+4;
		y+=t->r.h+6;
	} /* while */ 
	max_y=y;

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* OBitmapWnd::contens_size */ 


void OBitmapWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	List<TILE> l;
	TILE *t;
	int y;
	char tmp[8];

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	print_left("AVAILABLE BITMAPS:",font,255,255,255,sfc,0-stx,17-sty);

	sprintf(tmp,"%i",editor->ov_bitmap_category);
	print_left(tmp,font,255,255,255,sfc,140-stx,2-sty);

	y=25+TTF_FontHeight(font);
	l.Instance(editor->game->tiles[editor->ov_bitmap_category]);
	l.Rewind();
	while(l.Iterate(t)) {
		draw_rectangle(sfc,0-stx,(y-2)-sty,t->r.w+3,t->r.h+3,0x00ffffff);
		t->draw(2-stx,y-sty,sfc);

		draw_rectangle(sfc,t->r.w+8-stx,(y-2)-sty,t->r.w+3,t->r.h+3,0x00ffffff);
		t->draw_collision_mask(t->r.w+10-stx,y-sty,sfc);

		if (t->mask_type!=0) {
			draw_rectangle(sfc,(t->r.w+8)*2-stx,(y-2)-sty,t->r.w+3,t->r.h+3,0x00ffffff);
			t->draw_mask((t->r.w+8)*2+2-stx,y-sty,sfc);
		} /* if */ 
		
		y+=t->r.h+6;
	} /* while */ 

} /* OBitmapWnd::draw_contens */ 


bool OBitmapWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* NEW BITMAP: */ 
		{
			char tmp[256]="*.*";
			BFileDlg *dlg=new BFileDlg(30,30,200,200,"Choose a file from where to grab","Select",tmp,editor->font);

			dlg->Execute();
			delete dlg;

			if (tmp[0]!=0) {
				SDL_Surface *bmp_sfc,*tmp_sfc;

			    tmp_sfc = IMG_Load(tmp);
				if (tmp_sfc!=0) {
					bmp_sfc = SDL_CreateRGBSurface(surface_flags,tmp_sfc->w,tmp_sfc->h,32,0,0,0,AMASK);
					SDL_SetAlpha(bmp_sfc,0,SDL_ALPHA_OPAQUE);
					SDL_BlitSurface(tmp_sfc,0,bmp_sfc,0);
					SDL_FreeSurface(tmp_sfc);

					/* Grab bitmaps: */ 
					editor->ov_regrabbing=0;
					editor->grabv_bitmap=bmp_sfc;
					strcpy(editor->grabv_bitmap_file,tmp);
					editor->view=GRABBER_VIEW;
					editor->grabv_state=0;

				} else {
					int res;
					BChoiceDlg *dlg2=new BChoiceDlg(40,40,200,50,"Cannot open file","Accept",&res,editor->font);
					dlg2->Execute();
					delete dlg2;
				} /* if */ 
			} /* if */ 
			set_dirty();
		}
		break;

	case 2:
		{
			editor->ov_bitmap_category++;
			if (editor->ov_bitmap_category>=TILECATEGORIES) editor->ov_bitmap_category=0;
			set_dirty();
		}
		break;

	case 3:
		{
			editor->ov_bitmap_category--;
			if (editor->ov_bitmap_category<0) editor->ov_bitmap_category=TILECATEGORIES-1;
			set_dirty();
		}
		break;
	default:
		break;
	} /* switch */ 

	return true;
} /* OBitmapWnd::button_pressed */ 
