#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"

#include "OObjectWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

OObjectWnd::OObjectWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	BButton *b;

	editor=e;

	b=new BButton(0,0,100,12,"NEW OBJECT",1);
	buttons.Add(b);

} /* OObjectWnd::OObjectWnd */ 


OObjectWnd::~OObjectWnd()
{
} /* OObjectWnd::~OObjectWnd */ 


bool OObjectWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (((mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3))) ||
		 (mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {
		List<OBJECT> l,todelete;
		OBJECT *o;
		int y=TTF_FontHeight(font)+25;

		l.Instance(editor->game->objects);
		l.Rewind();
		while(l.Iterate(o)) {
			if (local_mouse_y>=y && 
				local_mouse_y<=y+TTF_FontHeight(font) &&
				local_mouse_x>=0 &&
				local_mouse_x<=get_text_width(o->name,font)) {

				if (mouse_button&SDL_BUTTON(1) &&
					!(old_mouse_button&SDL_BUTTON(1))) {
					/* LEFT BUTTON: */ 
					editor->ov_selected=o;
					editor->ov_selected_ob=-1;
					editor->set_all_dirty();
				} /* if */ 

				if (mouse_button&SDL_BUTTON(3) &&
					!(old_mouse_button&SDL_BUTTON(3))) {
					/* RIGHT BUTTON: */ 
					BStringEdDlg *dlg;
					BVChoiceDlg *dlg2;
					char tmp[80];
					int res=0;
					int x,y;

					dlg2=new BVChoiceDlg(-1,-1,0,0,"Choose action:","Edit name\nDelete\nDuplicate\n",&res,font);
					dlg2->Execute();
					delete dlg2;

					if (res==1) {
						strcpy(tmp,o->name);
						new_window_coordinates(320,60,&x,&y);
						dlg=new BStringEdDlg(x,y,320,60,"Enter object name:",tmp,40," ",font);
						dlg->Execute();
						delete dlg;

						delete []o->name;
						o->name=new char[strlen(tmp)+1];
						strcpy(o->name,tmp);

						editor->game->objects.Sort(&object_order);
						set_dirty();
					} /* if */ 

					if (res==2) {
						if (editor->ov_selected==o) editor->ov_selected=0;
						todelete.Add(o);
						set_dirty();
					} /* if */ 

					if (res==3) {
						OBJECT *o2,*o3;
						List<OBJECT> l;
						int i=0;
						char tmp[80];
						bool added=false,found;

						do{
							if (i==0) {
								sprintf(tmp,"object copy");
							} else {
								sprintf(tmp,"object copy_%i",i);
							} /* if */ 

							l.Instance(editor->game->objects);
							l.Rewind();
							found=false;
							while(!found && l.Iterate(o3)) {
								if (o3->cmp(tmp)) found=true;
							} /* while */ 

							if (!found) {
								o2=new OBJECT(tmp,o);
								editor->game->objects.Add(o2);
								added=true;
							} /* if */ 

							i++;
						}while(!added);

						editor->game->objects.Sort(&object_order);
						set_dirty();

					} /* if */ 
				} /* if */ 
			} /* if */ 

			y+=TTF_FontHeight(font)+2;
		} /* while */ 	

		while(!todelete.EmptyP()) {
			OBJECT *o;
			o=todelete.ExtractIni();
			editor->delete_object(o);
		} /* while */ 
	} /* if */ 

	return true;
} /* OObjectWnd::cycle */ 


void OObjectWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;
	List<OBJECT> l;
	OBJECT *o;
	
	BWindow::contens_size(dx,dy);
	
	max_x=0;
	max_y=TTF_FontHeight(font)+25+editor->game->objects.Length()*(TTF_FontHeight(font)+2);

	l.Instance(editor->game->objects);
	l.Rewind();
	while(l.Iterate(o)) {
		if (get_text_width(o->name,font)>max_x) max_x=get_text_width(o->name,font);
	} /* while */ 
	

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* OObjectWnd::contens_size */ 


void OObjectWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	List<OBJECT> l;
	OBJECT *o;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	print_left("OBJECTS:",font,255,255,255,sfc,0-stx,17-sty);

	l.Instance(editor->game->objects);
	l.Rewind();
	y=TTF_FontHeight(font)+25;
	while(l.Iterate(o)) {
		if (editor->ov_selected==o) {
			SDL_Rect r;
			r.x=0-stx;
			r.y=y-sty;
			r.w=get_text_width(o->name,font);
			r.h=TTF_FontHeight(font);
			SDL_FillRect(sfc,&r,0x00ff0000);
		} /* if */ 
		print_left(o->name,font,255,255,255,sfc,0-stx,y-sty);
		y+=TTF_FontHeight(font)+2;
	} /* while */ 
} /* OObjectWnd::draw_contens */ 


bool OObjectWnd::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* New Object!!: */ 
		{
			OBJECT *o;
			List<OBJECT> l;
			int i=0;
			char tmp[80];
			bool added=false,found;

			do{
				if (i==0) {
					sprintf(tmp,"new_object");
				} else {
					sprintf(tmp,"new_object_%i",i);
				} /* if */ 

				l.Instance(editor->game->objects);
				l.Rewind();
				found=false;
				while(!found && l.Iterate(o)) {
					if (o->cmp(tmp)) found=true;
				} /* while */ 

				if (!found) {
					o=new OBJECT(tmp);
					editor->game->objects.Add(o);
					added=true;
				} /* if */ 

				i++;
			}while(!added);

			/* Sort the list alphabetically: */ 
			editor->game->objects.Sort(&object_order);
			
			set_dirty();
		}
		break;
	default:
		break;
	} /* switch */ 

	return true;
} /* OObjectWnd::button_pressed */ 
