#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "tiles.h"
#include "object.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"
#include "BMultiListWnd.h"

#include "RMoveDlg.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BRMoveDlg::BRMoveDlg(int x,int y,int dx,int dy,MOG2 *g,MAP *m,ROOM *r,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;

	b=new BButton(10,dy-20,70,12,"MOVE",1);
	buttons.Add(b);
	b=new BButton(90,dy-20,70,12,"CANCEL",2);
	buttons.Add(b);

	game=g;
	map=m;
	room=r;

	x_offs=0;
	y_offs=0;

	{
		int i;
		char tmp[80],*tmp2;

		for(i=0;i<room->used_object_planes;i++) {
			sprintf(tmp,"Plane %i",i);
			tmp2=new char[strlen(tmp)+1];
			strcpy(tmp2,tmp);
			plane_names.Add(tmp2);
		} /* for */ 

		lw=new BMultiListWnd(10,10,100,dy-40,&plane_names,font);
	}

} /* BRMoveDlg::BRMoveDlg */ 


BRMoveDlg::~BRMoveDlg()
{
	delete lw;
} /* BRMoveDlg::~BRMoveDlg */ 


bool BRMoveDlg::cycle(int stx,int sty)
{
	bool retval;

	retval=BWindow::cycle(stx,sty);

	lw->cycle(stx+10,sty+10);
	if (lw->is_dirty()) set_dirty();

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		if (local_mouse_x>=140 && local_mouse_x<140+get_text_width("999999999",font) &&
			local_mouse_y>=20 && local_mouse_y<20+TTF_FontHeight(font)) {
			/* Change X_OFFS: */ 
			BIntegerEdDlg *dlg;
			int y,x;

			x=mouse_x-110;
			y=mouse_y;
			if (y>(old_y+old_dy-60)) y=old_y+old_dy-60;
			if (x>(old_x+old_dx-140)) x=old_x+old_dx-140;
			if (x<0) x=0;

			dlg=new BIntegerEdDlg(x,y,140,60,"Enter x move:",&(x_offs),6,font);
			dlg->Execute();
			delete dlg;
			set_dirty();
		} /* if */ 	

		if (local_mouse_x>=140 && local_mouse_x<140+get_text_width("999999999",font) &&
			local_mouse_y>=40 && local_mouse_y<40+TTF_FontHeight(font)) {
			/* Change Y_OFFS: */ 
			BIntegerEdDlg *dlg;
			int y,x;

			x=mouse_x-110;
			y=mouse_y;
			if (y>(old_y+old_dy-60)) y=old_y+old_dy-60;
			if (x>(old_x+old_dx-140)) x=old_x+old_dx-140;
			if (x<0) x=0;

			dlg=new BIntegerEdDlg(x,y,140,60,"Enter y move:",&(y_offs),6,font);
			dlg->Execute();
			delete dlg;
			set_dirty();
		} /* if */ 	

	} /* if */ 

	return retval;
} /* BRMoveDlg::cycle */ 



void BRMoveDlg::contens_size(int *dx,int *dy)
{
	*dx=old_dx;
	*dy=old_dy;
} /* BRMoveDlg::contens_size */ 


void BRMoveDlg::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;
	char tmp[80];

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	lw->draw(sfc,10-stx,10-sty,100,dy-40);

	sprintf(tmp,"X: %i",x_offs);
	print_left(tmp,font,255,255,255,sfc,140,20);
	sprintf(tmp,"Y: %i",y_offs);
	print_left(tmp,font,255,255,255,sfc,140,40);

} /* BRMoveDlg::draw_contens */ 


bool BRMoveDlg::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		/* Move all the objetcs of the selected planes: */ 
		{
			int i;
			for(i=0;i<MAX_OBJECT_PLANES;i++) {
				if (lw->selected_p(i)) {
					List<PLACED_OBJECT> l;
					PLACED_OBJECT *o;
					List<PLACED_TILE> l2;
					PLACED_TILE *t;

					l.Instance(room->objects[i]);
					l.Rewind();
					while(l.Iterate(o)) {
						o->x+=x_offs;
						o->y+=y_offs;
					} /* while */ 

					l2.Instance(room->tiles[i]);
					l2.Rewind();
					while(l2.Iterate(t)) {
						t->x+=x_offs;
						t->y+=y_offs;
					} /* while */ 
				} /* if */ 
			} /* for */ 
		}
		return false;
		break;

	case 2:
		return false;
		break;

	default:
		break;
	} /* switch */ 

	return true;
} /* BRMoveDlg::button_pressed */ 
