#include "string.h"
#include "stdlib.h"
#include "math.h"

#include "SDL.h"
#include "SDL_net.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "mog2multiplayer.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BMultiListWnd.h"
#include "BFileListWnd.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "BFileDlg.h"
#include "POVariableWnd.h"

#include "ScreenConfigDlg.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif


BScreenConfigDlg::BScreenConfigDlg(int x,int y,int dx,int dy,List<NWMOG2PlayerInfo> *cfgs,int *sxp,int *syp,int *renderp,bool *show_debugmsgp,bool *show_fpsp,bool d,TTF_Font *font) : BActiveWindow(x,y,dx,dy,font)
{
	BButton *b;

	b=new BButton(20,dy-18,80,12,"ACCEPT",1);
	buttons.Add(b);
	b=new BButton(120,dy-18,80,12,"CANCEL",2);
	buttons.Add(b);

	b=new BButton(10,10,100,12,"RESOLUTION",3);
	buttons.Add(b);

	b=new BButton(10,30,60,12,"RENDER",4);
	buttons.Add(b);

	b=new BButton(170,10,60,12,"MSG",5);
	buttons.Add(b);

	b=new BButton(170,30,60,12,"FPS",6);
	buttons.Add(b);
	

	cfg=cfgs;
	sx_p=sxp;
	sy_p=syp;
	render_p=renderp;
	show_debugmsg_p=show_debugmsgp;
	show_fps_p=show_fpsp;
	debug=d;

	if (debug) {
		if (*sx_p<640) {
			*sx_p=640;
			*sy_p=480;
		} /* if */ 
	} /* if */ 

	{
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pc;
		char *tmp;
		int i;

		l.Instance(*cfg);
		l.Rewind();
		i=0;
		while(l.Iterate(pc)) {
			tmp=new char[strlen(pc->name)+7];
			sprintf(tmp,"%i - %s",i,pc->name);
			playernames.Add(tmp);
			i++;
		} /* while */ 

		lw=new BMultiListWnd(10,50,120,dy-60,&playernames,font);

		l.Instance(*cfg);
		l.Rewind();
		i=0;
		while(l.Iterate(pc)) {
			if (pc->vp_area.w!=0) lw->set_selected(i);
			i++;
		} /* while */ 
	}

	build_basic_split();

} /* BScreenConfigDlg::BScreenConfigDlg */ 


BScreenConfigDlg::~BScreenConfigDlg()
{
	delete lw;
} /* BScreenConfigDlg::~BScreenConfigDlg */ 


bool BScreenConfigDlg::cycle(int stx,int sty)
{
	bool retval;

	retval=BWindow::cycle(stx,sty);

	lw->cycle(stx+10,sty+50);
	if (lw->is_dirty()) {
		build_basic_split();
		set_dirty();
	} /* if */ 

	if (((mouse_button&SDL_BUTTON(1) &&
		 !(old_mouse_button&SDL_BUTTON(1))) ||
		 (mouse_button&SDL_BUTTON(3) &&
		 !(old_mouse_button&SDL_BUTTON(3)))) &&
		local_mouse_x>=old_stx && 
		local_mouse_y>=old_sty &&
		local_mouse_x<old_endx &&
		local_mouse_y<old_endy) {

		SDL_Rect r;
		int x0=140,y0=50;
		int ddx=old_dx-150,ddy=old_dy-70;
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pc;

		l.Instance(*cfg);
		l.Rewind();
		while(l.Iterate(pc)) {
			if (pc->vp_area.w!=0) {
				r.x=((pc->vp_area.x*ddx)/(*sx_p))+x0;
				r.y=((pc->vp_area.y*ddy)/(*sy_p))+y0;
				r.w=((pc->vp_area.w*ddx)/(*sx_p));
				r.h=((pc->vp_area.h*ddy)/(*sy_p));

				if (local_mouse_x>=r.x && local_mouse_x<r.x+r.w &&
					local_mouse_y>=r.y && local_mouse_y<r.y+r.h) {
					if (mouse_button&SDL_BUTTON(1) &&
						!(old_mouse_button&SDL_BUTTON(1))) {
						if (pc->vp_zoom<2.0) {
							pc->vp_zoom+=0.125F;
						} else {
							if (pc->vp_zoom<8.0) pc->vp_zoom+=0.5F;
						} /* if */ 
						set_dirty();
					} else {
						if (pc->vp_zoom>2.0) {
							pc->vp_zoom-=0.5F;
						} else {
							if (pc->vp_zoom>0.125) pc->vp_zoom-=0.125F;
						} /* if */ 
						set_dirty();
					} /* if */ 
				} /* if */ 
			} /* if */ 
		} /* while */ 
	} /* if */ 

	return retval;
} /* BScreenConfigDlg::cycle */ 



void BScreenConfigDlg::contens_size(int *dx,int *dy)
{
	BActiveWindow::contens_size(dx,dy);

	if (old_dx>*dx) *dx=old_dx;
	if (old_dy>*dy) *dy=old_dy;
} /* BScreenConfigDlg::contens_size */ 


void BScreenConfigDlg::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_Rect r;

	SDL_FillRect(sfc,0,0x0000007f);

	r.x=1-stx;
	r.y=1-sty;
	r.w=dx-2;
	r.h=dy-2;
	SDL_FillRect(sfc,&r,0x000000ff);

	BActiveWindow::draw_contens(sfc,stx,sty,dx,dy);

	lw->draw(sfc,10-stx,50-sty,120,dy-70);

	{
		char tmp[80];

		sprintf(tmp,"%ix%i",*sx_p,*sy_p);
		print_left(tmp,font,255,255,255,sfc,120-stx,12-sty);

		if (*render_p==0) print_left("SW",font,255,255,255,sfc,80-stx,32-sty);
					 else print_left("HW",font,255,255,255,sfc,80-stx,32-sty);

		if (*show_debugmsg_p) print_left("ON",font,255,255,255,sfc,250-stx,12-sty);
						 else print_left("OFF",font,255,255,255,sfc,250-stx,12-sty);

		if (*show_fps_p) print_left("ON",font,255,255,255,sfc,250-stx,32-sty);
					else print_left("OFF",font,255,255,255,sfc,250-stx,32-sty);

	}

	/* Draw the split: */ 
	{
		int x0=140,y0=50;
		int ddx=dx-150,ddy=dy-70;
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pc;
		char tmp[16];

		r.x=x0-stx;
		r.y=y0-sty;
		r.w=ddx;
		r.h=ddy;
		SDL_FillRect(sfc,&r,0);

		l.Instance(*cfg);
		l.Rewind();
		while(l.Iterate(pc)) {
			if (pc->vp_area.w!=0) {
				r.x=((pc->vp_area.x*ddx)/(*sx_p))+x0-stx;
				r.y=((pc->vp_area.y*ddy)/(*sy_p))+y0-sty;
				r.w=((pc->vp_area.w*ddx)/(*sx_p));
				r.h=((pc->vp_area.h*ddy)/(*sy_p));
				SDL_FillRect(sfc,&r,0x003f3fff);

				r.x=((pc->vp_area.x*ddx)/(*sx_p))+x0+1-stx;
				r.y=((pc->vp_area.y*ddy)/(*sy_p))+y0+1-sty;
				r.w=((pc->vp_area.w*ddx)/(*sx_p))-2;
				r.h=((pc->vp_area.h*ddy)/(*sy_p))-2;
				SDL_FillRect(sfc,&r,0x003f3f7f);

				sprintf(tmp,"%.2f",pc->vp_zoom);
				print_left(tmp,font,255,255,255,sfc,r.x+(r.w/2)-(get_text_width(tmp,font)/2),
													r.y+(r.h/2)-(TTF_FontHeight(font)/2));

			} /* if */ 
		} /* while */ 
	}

} /* BScreenConfigDlg::draw_contens */ 



bool BScreenConfigDlg::button_pressed(int ID)
{
	switch(ID) {
	case 1:
		{
//			pc.avatar=characters->operator [](lw->get_selected());
//			*original_pc=pc;
			return false;
		}
		break;

	case 2:
		return false;
		break;

	case 3:
		switch(*sx_p) {
		case 320:
			*sx_p=512;
			*sy_p=384;
			if (debug) {
				*sx_p=640;
				*sy_p=480;
			} /* if */ 
			break;
		case 512:
			*sx_p=640;
			*sy_p=480;
			break;
		case 640:
			*sx_p=800;
			*sy_p=600;
			break;
		case 800:
			*sx_p=1024;
			*sy_p=768;
			break;
		case 1024:
			*sx_p=320;
			*sy_p=240;
			if (debug) {
				*sx_p=640;
				*sy_p=480;
			} /* if */ 
			break;
		default:
			*sx_p=640;
			*sy_p=480;
		} /* switch */ 

		build_basic_split();
		set_dirty();
		break;

	case 4:
		if (*render_p==0) {
			*render_p=1;
		} else {
			*render_p=0;
		} /* if */ 
		set_dirty();
		break;

	case 5:
		if (*show_debugmsg_p) {
			*show_debugmsg_p=false;
		} else {
			*show_debugmsg_p=true;
		} /* if */ 
		build_basic_split();
		set_dirty();
		break;

	case 6:
		if (*show_fps_p) {
			*show_fps_p=false;
		} else {
			*show_fps_p=true;
		} /* if */ 
		set_dirty();
		break;

	} /* switch */ 

	return true;
} /* BScreenConfigDlg::button_pressed */ 


void BScreenConfigDlg::build_basic_split(void)
{
	List<int> *s;
	int *ip;
	int n;

	{
		List<NWMOG2PlayerInfo> l;
		NWMOG2PlayerInfo *pc;

		l.Instance(*cfg);
		l.Rewind();
		while(l.Iterate(pc)) {
			pc->vp_area.x=0;
			pc->vp_area.y=0;
			pc->vp_area.w=0;
			pc->vp_area.h=0;
			pc->vp_zoom=1.0;
		} /* while */ 
	}

	s=lw->get_selected();
	n=s->Length();

	if (n==0) return;

	/* the available size is: *sx_p - 0, and *sx_p - 64 */ 
	/* we need 'n' windows */ 
	{
		int i,rest;
		int rows;
		int *columns;
		int up_space=64;

		if (!(*show_debugmsg_p)) up_space=0;

		/* Compute how many rows and columns will have the split: */ 
		rows=int(ceil(sqrt(float(n)/2)));
		columns=new int[rows];

		rest=n;
		for(i=0;i<rows;i++) {
			columns[i]=n/rows;
			rest-=n/rows;
		} /* for */ 
		
		i=rows-1;
		while(rest!=0) {
			columns[i]++;
			rest--;
			i--;
			if (i<0) i=rows-1;
		} /* while */ 

		{
			int row,column;
			int width,height;

			if (debug) {
				height=320/rows;
			} else {
				height=((*sy_p)-up_space)/rows;
			} /* if */ 

			/* Compute the windows: */ 
			row=0;
			column=0;
			s->Rewind();
			while(s->Iterate(ip)) {
				if (debug) width=((*sx_p)-100)/columns[row];
					  else width=(*sx_p)/columns[row];

				if (debug) {
					cfg->operator [](*ip)->vp_area.x=100+width*column;
					cfg->operator [](*ip)->vp_area.y=height*row;
					cfg->operator [](*ip)->vp_area.w=width-1;
					cfg->operator [](*ip)->vp_area.h=height-1;
				} else {
					cfg->operator [](*ip)->vp_area.x=width*column;
					cfg->operator [](*ip)->vp_area.y=up_space+height*row;
					cfg->operator [](*ip)->vp_area.w=width-1;
					cfg->operator [](*ip)->vp_area.h=height-1;
				} /* if */ 

				column++;
				if (column>=columns[row]) {
					column=0;
					row++;
				} /* if */ 
			} /* while */ 
		}

		delete []columns;
	}

	delete s;

} /* BScreenConfigDlg::build_basic_split */ 


