#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"
#include "MOG2_auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "interfaces.h"

#include "actionscript.h"
#include "conditionscript.h"
#include "valuescript.h"

#include "BWindow.h"
#include "BActiveWindow.h"

#include "ScriptDebugWnd.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

BScriptDebugWnd::BScriptDebugWnd(int x,int y,int dx,int dy,ActionSCRIPT *s,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	script=s;
} /* BScriptDebugWnd::BScriptDebugWnd */ 


BScriptDebugWnd::~BScriptDebugWnd()
{
} /* BScriptDebugWnd::~BScriptDebugWnd */ 


bool BScriptDebugWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	return true;
} /* BScriptDebugWnd::cycle */ 


void BScriptDebugWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;
	
	BWindow::contens_size(dx,dy);

	if (script!=0) {
		char *tmp=action_script_state(script);
		char tmp2[512];
		int i=0,j,x,y;
		int prev_indents[1024];
		int current_indent=0,indent=0;

		x=0;
		y=0;
		if (get_text_width("SCRIPT STATE:",font)+x>max_x) max_x=get_text_width("SCRIPT STATE:",font)+x;
		y+=10;

		do{
			for(j=0;j<indent;j++) tmp2[j]=' ';
			for(;j<512 && tmp[i]!=0 && tmp[i]!='\n';i++,j++) {
				if (tmp[i]=='{') {
					indent=j;
					prev_indents[current_indent]=j;
					current_indent++;
				} /* if */ 

				if (tmp[i]=='}') {
					current_indent--;
					if (current_indent==0) indent=0;
									  else indent=prev_indents[current_indent-1];
				} /* if */ 

				tmp2[j]=tmp[i];
			} /* for */ 
			tmp2[j]=0;
			if (tmp[i]=='\n') i++;

			if (get_text_width(tmp2,font)+x>max_x) max_x=get_text_width(tmp2,font)+x;
			y+=10;
		}while(tmp[i]!=0);
		max_y=y;
	} /* if */ 
	
	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* BScriptDebugWnd::contens_size */ 


void BScriptDebugWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	if (script!=0) {
		char *tmp=action_script_state(script);
		char tmp2[512];
		int i=0,j,y;
		int prev_indents[1024];
		int current_indent=0,indent=0;

		y=0;
		print_left("SCRIPT STATE:",font,255,255,255,sfc,0-stx,y-sty);
		y+=10;

		do{
			for(j=0;j<indent;j++) tmp2[j]=' ';
			for(;j<512 && tmp[i]!=0 && tmp[i]!='\n';i++,j++) {
				if (tmp[i]=='{') {
					indent=j;
					prev_indents[current_indent]=j;
					current_indent++;
				} /* if */ 

				if (tmp[i]=='}') {
					current_indent--;
					if (current_indent==0) indent=0;
									  else indent=prev_indents[current_indent-1];
				} /* if */ 

				tmp2[j]=tmp[i];
			} /* for */ 
			tmp2[j]=0;
			if (tmp[i]=='\n') i++;

			print_left(tmp2,font,255,255,255,sfc,0-stx,y-sty);
			y+=10;
		}while(tmp[i]!=0);
	} /* if */ 

} /* BScriptDebugWnd::draw_contens */ 


void BScriptDebugWnd::set_script(ActionSCRIPT *s)
{
	script=s;
} /* BScriptDebugWnd::set_script */ 


char *BScriptDebugWnd::action_script_state(ActionSCRIPT *script)
{
	char *tmp=0;

	if (script==0) return tmp;

	switch(script->command) {
	case ACTION_CMD_NONE:
		tmp=new char[strlen("none")+1];
		strcpy(tmp,"none");
		break;
	case ACTION_CMD_SEQ:
		{
			char *tmp2,*tmp3,*tmp4;
			List<ActionSCRIPT> l;
			ActionSCRIPT *s;

			l.Instance(script->aparlist);
			if (script->aparlist.EndP()) {
				tmp=new char[strlen("SEQ{ ... }")+1];
				strcpy(tmp,"SEQ{ ... }");
			} else {
				s=script->aparlist.GetObj();

				l.Rewind();
				if (l.GetObj()==s) {
					/* The current command is the first one: */ 
					tmp2=new char[strlen("SEQ{\n")+1];
					strcpy(tmp2,"SEQ{\n");
				} else {
					/* The current command is not the first one: */ 
					tmp2=new char[strlen("SEQ{\n...\n")+1];
					strcpy(tmp2,"SEQ{\n...\n");
				} /* if */ 

				l.Forward();
				if (l.GetObj()==s) {
					/* The current command is the last one: */ 
					tmp4=new char[strlen("\n}")+1];
					strcpy(tmp4,"\n}");
				} else {
					/* The current command is not the last one: */ 
					tmp4=new char[strlen("\n...\n}")+1];
					strcpy(tmp4,"\n...\n}");
				} /* if */ 

				tmp3=action_script_state(s);

				tmp=new char[strlen(tmp2)+strlen(tmp3)+strlen(tmp4)+1];
				sprintf(tmp,"%s%s%s",tmp2,tmp3,tmp4);
				delete []tmp2;
				delete []tmp3;
				delete []tmp4;
			} /* if */ 
		}
		break;
            
	case ACTION_CMD_PARALLEL:
		{
			List<ActionSCRIPT> l;
			ActionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=action_script_state(as);
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length();
			} /* if */ 
			len+=strlen("PARALLEL{\n\n}")+1;

			tmp=new char[len];

			sprintf(tmp,"PARALLEL{\n");
			pos=strlen("PARALLEL{\n");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(tmp+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(tmp+pos,"\n");
					pos+=1;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(tmp+pos,"\n}");
		}		
		break;
                
	case ACTION_CMD_IF:
		{
			char *s,*s2,*s3;
			int len=0;

			s=condition_script_state(script->cond);
			len+=strlen(s);
		
			s2=action_script_state(script->aparlist[0]);
			s3=action_script_state(script->aparlist[1]);
			
			len+=strlen(s2);
			len+=strlen(s3);
			len+=strlen("IF (  )\n\nELSE ")+1;

			tmp=new char[len];

			sprintf(tmp,"IF ( %s )\n%s\nELSE %s",s,s2,s3);

			delete []s;
			delete []s2;
			delete []s3;
		}
		break;
                
	case ACTION_CMD_WHILE:
		{
			char *tmp2,*tmp3;

			tmp2=condition_script_state(script->cond);
			tmp3=action_script_state(script->aparlist[0]);

			tmp=new char[strlen(tmp2)+strlen(tmp3)+16];
			sprintf(tmp,"WHILE( %s ) {\n%s\n}",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case ACTION_CMD_WHILE_ELSE:
		{
			char *tmp2,*tmp3,*tmp4;

			tmp2=condition_script_state(script->cond);
			tmp3=action_script_state(script->aparlist[0]);
			tmp4=action_script_state(script->aparlist[1]);

			tmp=new char[strlen(tmp2)+strlen(tmp3)+strlen(tmp4)+21];
			sprintf(tmp,"WHILE( %s ) {\n%s\n} ELSE %s",tmp2,tmp3,tmp4);
			delete []tmp2;
			delete []tmp3;
			delete []tmp4;
		}
		break;

	case ACTION_CMD_DO:
		{
			char *tmp2,*tmp3;

			tmp2=condition_script_state(script->cond);
			tmp3=action_script_state(script->aparlist[0]);

			tmp=new char[strlen(tmp2)+strlen(tmp3)+19];
			sprintf(tmp,"DO {\n%s\n} WHILE( %s )",tmp3,tmp2);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case ACTION_CMD_PAUSE:
		{
			char *tmp2;
			char tmp3[8];

			tmp2=value_script_state(script->vpar[0]);
			sprintf(tmp3,"%i",script->timmer);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+11];
			sprintf(tmp,"PAUSE( %s/%s )",tmp3,tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_ATOMIC:
		{
			char *tmp2;

			tmp2=action_script_state(script->aparlist[0]);
			tmp=new char[strlen(tmp2)+11];
			sprintf(tmp,"ATOMIC( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_TERMINATE:
		tmp=new char[strlen("terminate")+1];
		strcpy(tmp,"terminate");
		break;

	case ACTION_CMD_TERMINATECHILD:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+19];
			sprintf(tmp,"TERMINATECHILD( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_STATE:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+10];
			sprintf(tmp,"STATE( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_SET:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+8];
			sprintf(tmp,"SET %s = %s",tmp2,tmp3);
			delete []tmp2;
		}
		break;

	case ACTION_CMD_MOVE:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+12];
			sprintf(tmp,"MOVE( %s , %s )",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case ACTION_CMD_CREATEOBJECT:
		{
			/* CREATEOBJECT (xoffs,yoffs,TYPE,precedence);	*/ 
			char *s,*s2,*s3;

			s=value_script_state(script->vpar[0]);
			s2=value_script_state(script->vpar[1]);
			s3=value_script_state(script->vpar[2]);

			tmp=new char[strlen("CREATEOBJECT (  ,  ,  )")+strlen(s)+strlen(s2)+strlen(s3)+1];
			sprintf(tmp,"CREATEOBJECT ( %s , %s , %s )",s,s2,s3);
			delete []s;
			delete []s2;
			delete []s3;
		}
		break;


	case ACTION_CMD_CREATECHILD:
		{
			/* CREATECHILD (xoffs,yoffs,TYPE,precedence);	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6;

			s=value_script_state(script->vpar[0]);
			s2=value_script_state(script->vpar[1]);
			s3=value_script_state(script->vpar[2]);
			s4=value_script_state(script->vpar[3]);
			s5=value_script_state(script->vpar[4]);
			s6=value_script_state(script->vpar[5]);

			tmp=new char[strlen("CREATECHILD (  ,  ,  ,  ,  ,  )")+strlen(s)+strlen(s2)+strlen(s3)+strlen(s4)+strlen(s5)+strlen(s6)+1];
			sprintf(tmp,"CREATECHILD ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;
	case ACTION_CMD_CREATEGLUEDCHILD:
		{
			/* CREATEGLUEDCHILD (xoffs,yoffs,TYPE,precedence);	*/ 
			char *s,*s2,*s3,*s4,*s5,*s6;

			s=value_script_state(script->vpar[0]);
			s2=value_script_state(script->vpar[1]);
			s3=value_script_state(script->vpar[2]);
			s4=value_script_state(script->vpar[3]);
			s5=value_script_state(script->vpar[4]);
			s6=value_script_state(script->vpar[5]);

			tmp=new char[strlen("CREATEGLUEDCHILD (  ,  ,  ,  ,  ,  )")+strlen(s)+strlen(s2)+strlen(s3)+strlen(s4)+strlen(s5)+strlen(s6)+1];
			sprintf(tmp,"CREATEGLUEDCHILD ( %s , %s , %s , %s , %s , %s )",s,s2,s3,s4,s5,s6);
			delete []s;
			delete []s2;
			delete []s3;
			delete []s4;
			delete []s5;
			delete []s6;
		}
		break;

	default:
		tmp=new char[strlen("#action#")+1];
		strcpy(tmp,"#action#");
		break;
	} /* switch */ 

	return tmp;
} /* BScriptDebugWnd::action_script_state */ 


char *BScriptDebugWnd::condition_script_state(ConditionSCRIPT *script)
{
	char *tmp=0;

	if (script==0) return tmp;

	switch(script->command) {
	case CONDITION_CMD_TRUE:
		tmp=new char[strlen("true")+1];
		strcpy(tmp,"true");
		break;
	case CONDITION_CMD_FALSE:
		tmp=new char[strlen("false")+1];
		strcpy(tmp,"false");
		break;
	case CONDITION_CMD_NOT:
		{
			char *tmp2;

			tmp2=condition_script_state(script->aparlist[0]);
			tmp=new char[strlen(tmp2)+8];
			sprintf(tmp,"NOT( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_AND:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=condition_script_state(as);
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("AND {\n\n}")+1;

			tmp=new char[len];

			sprintf(tmp,"AND {\n");
			pos=strlen("AND {\n");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(tmp+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(tmp+pos,"\n");
					pos++;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(tmp+pos,"}");
		}
		break;

	case CONDITION_CMD_OR:
		{
			List<ConditionSCRIPT> l;
			ConditionSCRIPT *as;
			List<char> ls;
			char *s;
			int len=0,pos;

			l.Instance(script->aparlist);
			l.Rewind();
			while(l.Iterate(as)) {
				s=condition_script_state(as);
				ls.Add(s);
				len+=strlen(s);
			} /* while */ 
			if (!l.EmptyP()) {
				len+=l.Length()*3;
			} /* if */ 
			len+=strlen("OR {\n\n}")+1;

			tmp=new char[len];

			sprintf(tmp,"OR {\n");
			pos=strlen("OR {\n");

			ls.Rewind();
			while(!ls.EmptyP()) {
				s=ls.ExtractIni();
				sprintf(tmp+pos,s);
				pos+=strlen(s);
				if (!ls.EmptyP()) {
					sprintf(tmp+pos,"\n");
					pos++;
				} /* if */ 
				delete []s;
			} /* while */ 
			sprintf(tmp+pos,"}");
		}
		break;

	case CONDITION_CMD_TIMMER:
		{
			char *tmp2;
			char tmp3[8];

			tmp2=value_script_state(script->vpar[0]);
			sprintf(tmp3,"%i",script->timmer);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+12];
			sprintf(tmp,"TIMMER( %s/%s )",tmp3,tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_COLLISION:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+17];
			sprintf(tmp,"COLLISION( %s , %s )",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case CONDITION_CMD_INSIDE_ROOM:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+19];
			sprintf(tmp,"INSIDE_ROOM( %s , %s )",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	case CONDITION_CMD_KEYPRESS:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+13];
			sprintf(tmp,"KEYPRESS( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_KEYPRESSED:
		{
			char *tmp2;

			tmp2=value_script_state(script->vpar[0]);
			tmp=new char[strlen(tmp2)+15];
			sprintf(tmp,"KEYPRESSED( %s )",tmp2);
			delete []tmp2;
		}
		break;

	case CONDITION_CMD_VARIABLE_EQ:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+5];
			sprintf(tmp,"%s == %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_GR:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+4];
			sprintf(tmp,"%s > %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_GREQ:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+5];
			sprintf(tmp,"%s >= %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_LW:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+4];
			sprintf(tmp,"%s < %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;
	case CONDITION_CMD_VARIABLE_LWEQ:
		{
			char *tmp2,*tmp3;

			tmp2=value_script_state(script->vpar[0]);
			tmp3=value_script_state(script->vpar[1]);
			tmp=new char[strlen(tmp2)+strlen(tmp3)+5];
			sprintf(tmp,"%s <= %s",tmp2,tmp3);
			delete []tmp2;
			delete []tmp3;
		}
		break;

	default:
		tmp=new char[strlen("#condition#")+1];
		strcpy(tmp,"#condition#");
		break;
	} /* switch */ 

	return tmp;
} /* BScriptDebugWnd::condition_script_state */ 


char *BScriptDebugWnd::value_script_state(ValueSCRIPT *script)
{
	char *tmp=0;

	if (script==0) return tmp;

	switch(script->command) {
	case VALUE_CMD_INTEGER:
		{
			char tmp2[16];

			sprintf(tmp2,"%i",script->ipar);
			tmp=new char[strlen(tmp2)+1];
			strcpy(tmp,tmp2);
		}
		break;
	case VALUE_CMD_STRING:
		tmp=new char[strlen(script->spar)+1];
		strcpy(tmp,script->spar);
		break;
	case VALUE_CMD_KEY:
		{
			char tmp2[16];
			sprintf(tmp2,input_key_name(script->ipar));

			tmp=new char[strlen(tmp2)+1];
			strcpy(tmp,tmp2);
		}
		break;
	case VALUE_CMD_VARIABLE:
		tmp=new char[strlen(script->vname)+1];
		sprintf(tmp,"%s",script->vname);
		break;
/*
#define VALUE_CMD_RANDOM		5
#define VALUE_CMD_ADD			6
#define VALUE_CMD_SUB			7
#define VALUE_CMD_MUL			8
#define VALUE_CMD_DIV			9
*/
	default:
		tmp=new char[strlen("#value#")+1];
		strcpy(tmp,"#value#");
		break;
	} /* switch */ 

	return tmp;
} /* BScriptDebugWnd::value_script_state */ 

