#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "List.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "object.h"
#include "interfaces.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"
#include "VariableListWnd.h"

#include "valuescript.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

extern int SCREEN_X,SCREEN_Y;

VariableListWnd::VariableListWnd(int x,int y,int dx,int dy,EDITOR *e,OBJECT *o,ROOM *r,MAP *m,MOG2 *g,INTERFACE *i,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	scope_to_refind=VALUE_VAR_OBJECT;
	editor=e;
	selected=0;
	object=o;
	room=r;
	map=m;
	game=g;
	interf=i;
	scope=VALUE_VAR_NONE;
} /* VariableListWnd::VariableListWnd */ 


VariableListWnd::~VariableListWnd()
{
	selected=0;
} /* VariableListWnd::~VariableListWnd */ 


bool VariableListWnd::cycle(int stx,int sty)
{
	BWindow::cycle(stx,sty);

	if (scope_to_refind!=VALUE_VAR_NONE) {
		List<MAP> lm;
		MAP *m;
		List<ROOM> lr;
		ROOM *r;
		List<OBJECT> lo;
		OBJECT *o;
		List<VARIABLE> lv;
		VARIABLE *v;
		List<INTERFACE> li;
		INTERFACE *i;
		List<char> lc;
		char *tmp,*tmp2;
		bool found;
		
		variable_names.Delete();
		selected=0;

		switch(scope_to_refind) {
		case VALUE_VAR_OBJECT:
			if (object!=0) object->get_variable_names(&variable_names);
			break;

		case VALUE_VAR_ROOM:
			/* Room variables: */ 
			if (object!=0) {
				lm.Instance(editor->game->maps);
				lm.Rewind();
				while(lm.Iterate(m)) {
					lr.Instance(m->rooms);
					lr.Rewind();
					while(lr.Iterate(r)) {
						lv.Instance(r->room_variables);
						lv.Rewind();
						while(lv.Iterate(v)) {
							tmp=new char[strlen(v->name)+1];
							strcpy(tmp,v->name);
							variable_names.Add(tmp);
						} /* while */ 
					} /* while */ 
				} /* while */ 
			} else {
				if (room!=0) {
					lv.Instance(room->room_variables);
					lv.Rewind();
					while(lv.Iterate(v)) {
						tmp=new char[strlen(v->name)+1];
						strcpy(tmp,v->name);
						variable_names.Add(tmp);
					} /* while */ 
				} /* if */ 
			} /* if */ 
			break;

		case VALUE_VAR_MAP:
			/* Map variables: */ 
			if (room!=0 || object!=0) {
				lm.Instance(editor->game->maps);
				lm.Rewind();
				while(lm.Iterate(m)) {
					lv.Instance(m->map_variables);
					lv.Rewind();
					while(lv.Iterate(v)) {
						tmp=new char[strlen(v->name)+1];
						strcpy(tmp,v->name);
						variable_names.Add(tmp);
					} /* while */ 
				} /* while */ 
			} else {
				if (map!=0) {
					lv.Instance(map->map_variables);
					lv.Rewind();
					while(lv.Iterate(v)) {
						tmp=new char[strlen(v->name)+1];
						strcpy(tmp,v->name);
						variable_names.Add(tmp);
					} /* while */ 
				} /* if */ 
			} /* if */ 
			break;

		case VALUE_VAR_GAME:

			lv.Instance(editor->game->game_variables);
			lv.Rewind();
			while(lv.Iterate(v)) {
				tmp=new char[strlen(v->name)+1];
				strcpy(tmp,v->name);
				variable_names.Add(tmp);
			} /* while */ 

			tmp=new char[18];
			strcpy(tmp,"game_cycle");
			variable_names.Add(tmp);
			break;

		case VALUE_VAR_INTERFACE:
			/* Interface variables: */ 
			if (interf==0) {
				li.Instance(editor->game->interfaces);
				li.Rewind();
				while(li.Iterate(i)) {
					lv.Instance(i->interface_variables);
					lv.Rewind();
					while(lv.Iterate(v)) {
						bool found=false;
						
						lc.Instance(variable_names);
						lc.Rewind();
						while(!found && lc.Iterate(tmp2)) {
							if (strcmp(tmp2,v->name)==0) found=true;
						} /* while */ 

						if (!found) {
							tmp=new char[strlen(v->name)+1];
							strcpy(tmp,v->name);
							variable_names.Add(tmp);
						} /* if */ 
					} /* while */ 
				} /* while */ 
			} else {
				lv.Instance(interf->interface_variables);
				lv.Rewind();
				while(lv.Iterate(v)) {
					tmp=new char[strlen(v->name)+1];
					strcpy(tmp,v->name);
					variable_names.Add(tmp);
				} /* while */ 
			} /* if */ 

			tmp=new char[18];
			strcpy(tmp,"interf_focusing_x");
			variable_names.Add(tmp);
			tmp=new char[18];
			strcpy(tmp,"interf_focusing_y");
			variable_names.Add(tmp);
			break;

		case VALUE_VAR_APPLICATION:

			tmp=new char[13];
			strcpy(tmp,"music_volume");
			variable_names.Add(tmp);

			tmp=new char[11];
			strcpy(tmp,"sfx_volume");
			variable_names.Add(tmp);

			tmp=new char[15];
			strcpy(tmp,"music_priority");
			variable_names.Add(tmp);

			tmp=new char[14];
			strcpy(tmp,"music_playing");
			variable_names.Add(tmp);

			lv.Instance(editor->game->application_variables);
			lv.Rewind();
			while(lv.Iterate(v)) {
				tmp=new char[strlen(v->name)+1];
				strcpy(tmp,v->name);
				variable_names.Add(tmp);
			} /* while */ 
			break;

		case VALUE_VAR_FOREIGN:
			tmp=new char[6];
			strcpy(tmp,"state");
			variable_names.Add(tmp);

			tmp=new char[5];
			strcpy(tmp,"this");
			variable_names.Add(tmp);

			tmp=new char[2];
			strcpy(tmp,"x");
			variable_names.Add(tmp);

			tmp=new char[2];
			strcpy(tmp,"y");
			variable_names.Add(tmp);

			tmp=new char[6];
			strcpy(tmp,"scale");
			variable_names.Add(tmp);

			tmp=new char[6];
			strcpy(tmp,"angle");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"energy");
			variable_names.Add(tmp);

			tmp=new char[11];
			strcpy(tmp,"experience");
			variable_names.Add(tmp);

			tmp=new char[20];
			strcpy(tmp,"relative_precedence");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"damage");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"damage_type");
			variable_names.Add(tmp);


			tmp=new char[16];
			strcpy(tmp,"dm_drawing_mode");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color1_r");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color1_g");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color1_b");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color1_a");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color2_r");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color2_g");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color2_b");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_color2_a");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"dm_factor");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_blink_f0");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"dm_blink_f1");
			variable_names.Add(tmp);

			tmp=new char[15];
			strcpy(tmp,"dm_blink_speed");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"object_plane");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"text_text");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"text_font");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"text_size");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"text_spacing");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"text_r");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"text_g");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"text_b");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"text_a");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"text_h_align");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"text_v_align");
			variable_names.Add(tmp);

			tmp=new char[11];
			strcpy(tmp,"text_frame");
			variable_names.Add(tmp);

			tmp=new char[19];
			strcpy(tmp,"text_frame_point_x");
			variable_names.Add(tmp);
			
			tmp=new char[19];
			strcpy(tmp,"text_frame_point_y");
			variable_names.Add(tmp);

			tmp=new char[14];
			strcpy(tmp,"text_frame_r1");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_g1");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_b1");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_a1");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_r2");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_g2");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_b2");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"text_frame_a2");
			variable_names.Add(tmp);
			
			tmp=new char[17];
			strcpy(tmp,"text_frame_width");
			variable_names.Add(tmp);

			tmp=new char[18];
			strcpy(tmp,"text_frame_radius");
			variable_names.Add(tmp);

			tmp=new char[11];
			strcpy(tmp,"text_max_w");
			variable_names.Add(tmp);
					 
			tmp=new char[12];
			strcpy(tmp,"gw_focusing");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"gw_focusing_x");
			variable_names.Add(tmp);
			
			tmp=new char[14];
			strcpy(tmp,"gw_focusing_y");
			variable_names.Add(tmp);
			
			tmp=new char[6];
			strcpy(tmp,"gw_dx");
			variable_names.Add(tmp);
			
			tmp=new char[6];
			strcpy(tmp,"gw_dy");
			variable_names.Add(tmp);

			tmp=new char[9];
			strcpy(tmp,"gw_scale");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"gw_min_hs_x");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"gw_min_hs_y");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"gw_max_hs_x");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"gw_max_hs_y");
			variable_names.Add(tmp);

			tmp=new char[7];
			strcpy(tmp,"key_up");
			variable_names.Add(tmp);

			tmp=new char[9];
			strcpy(tmp,"key_down");
			variable_names.Add(tmp);

			tmp=new char[9];
			strcpy(tmp,"key_left");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"key_right");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"key_fire1");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"key_fire2");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"key_fire3");
			variable_names.Add(tmp);

			tmp=new char[10];
			strcpy(tmp,"key_fire4");
			variable_names.Add(tmp);
			
			tmp=new char[13];
			strcpy(tmp,"jumping_ctnt");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"gravity_ctnt");
			variable_names.Add(tmp);

			tmp=new char[14];
			strcpy(tmp,"walking_speed");
			variable_names.Add(tmp);

			tmp=new char[16];
			strcpy(tmp,"ladder_up_speed");
			variable_names.Add(tmp);

			tmp=new char[18];
			strcpy(tmp,"ladder_down_speed");
			variable_names.Add(tmp);

			tmp=new char[8];
			strcpy(tmp,"menu_dx");
			variable_names.Add(tmp);

			tmp=new char[8];
			strcpy(tmp,"menu_dy");
			variable_names.Add(tmp);

			tmp=new char[11];
			strcpy(tmp,"menu_title");
			variable_names.Add(tmp);

			tmp=new char[22];
			strcpy(tmp,"menu_appearance_flags");
			variable_names.Add(tmp);

			tmp=new char[24];
			strcpy(tmp,"menu_entry_effect_flags");
			variable_names.Add(tmp);

			tmp=new char[18];
			strcpy(tmp,"menu_blink_timmer");
			variable_names.Add(tmp);

			tmp=new char[17];
			strcpy(tmp,"menu_zoom_factor");
			variable_names.Add(tmp);

			tmp=new char[19];
			strcpy(tmp,"menu_appear_factor");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"menu_text_r2");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"menu_text_g2");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"menu_text_b2");
			variable_names.Add(tmp);

			tmp=new char[13];
			strcpy(tmp,"menu_text_a2");
			variable_names.Add(tmp);

			tmp=new char[12];
			strcpy(tmp,"menu_timmer");
			variable_names.Add(tmp);

			tmp=new char[14];
			strcpy(tmp,"menu_selected");
			variable_names.Add(tmp);


			/* Temporal variables: */ 
			lo.Instance(editor->game->objects);
			lo.Rewind();
			while(lo.Iterate(o)) {
				lv.Instance(o->object_variables);
				lv.Rewind();
				while(lv.Iterate(v)) {
					found=false;
					
					lc.Instance(variable_names);
					lc.Rewind();
					while(!found && lc.Iterate(tmp2)) {
						if (strcmp(tmp2,v->name)==0) found=true;
					} /* while */ 

					if (!found) {
						tmp=new char[strlen(v->name)+1];
						strcpy(tmp,v->name);
						variable_names.Add(tmp);
					} /*  if */ 
				} /* while */ 
			} /* while */ 
			break;
		} /* switch */ 
		
		scope=scope_to_refind;
		scope_to_refind=VALUE_VAR_NONE;
		set_dirty();
	} /* if */ 

	/* Click over variables: */ 
	if (( (mouse_button&SDL_BUTTON(1) &&
		  !(old_mouse_button&SDL_BUTTON(1)) ) ||
		  (mouse_button&SDL_BUTTON(3) &&
		  !(old_mouse_button&SDL_BUTTON(3)) ) )) {

		if (local_mouse_x>old_stx &&
			local_mouse_y>old_sty &&
			local_mouse_x<old_endx &&
			local_mouse_y<old_endy) {
			int i;

			i=(local_mouse_y-13)/10;

			if (local_mouse_x>=13 &&
			    i<variable_names.Length()) {
				selected=variable_names[i];
			} else {
				selected=0;
			} /* if */ 
			set_dirty();
		} /* if */ 
		
	} /* if */ 

	return true;
} /* VariableListWnd::cycle */ 


void VariableListWnd::contens_size(int *dx,int *dy)
{
	int y;
	int max_x=0;
	int max_y=0;
	char *tmp;

//	max_x=2+int(strlen("OBJECT VARIABLES:"))*font->get_dx();
//	max_x=2+int(strlen("ROOM VARIABLES:"))*font->get_dx();
//	max_x=2+int(strlen("MAP VARIABLES:"))*font->get_dx();
//	max_x=2+int(strlen("GAME VARIABLES:"))*font->get_dx();
	max_x=2+get_text_width("APPLICATION VARIABLES:",font);

	y=13;
	variable_names.Rewind();
	while(variable_names.Iterate(tmp)) {
		if (10+get_text_width(tmp,font)>max_x) max_x=10+get_text_width(tmp,font);
		y+=10;
	} /* while */ 

	max_y=y;

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* VariableListWnd::contens_size */ 


void VariableListWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	int y;
	char *tmp;

	SDL_FillRect(sfc,0,0);

	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	y=3;
	if (scope==VALUE_VAR_OBJECT) print_left("OBJECT VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);
	if (scope==VALUE_VAR_ROOM) print_left("ROOM VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);
	if (scope==VALUE_VAR_MAP) print_left("MAP VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);
	if (scope==VALUE_VAR_GAME) print_left("GAME VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);
	if (scope==VALUE_VAR_INTERFACE) print_left("INTERFACE VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);
	if (scope==VALUE_VAR_APPLICATION) print_left("APPLICATION VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);
	if (scope==VALUE_VAR_FOREIGN) print_left("FOREIGN VARIABLES:",font,255,255,255,sfc,2-stx,y-sty);

	y+=10;
	variable_names.Rewind();
	while(variable_names.Iterate(tmp)) {
		if (selected!=0 && 
			strcmp(selected,tmp)==0) {
			SDL_Rect r;
			r.x=0-stx;
			r.y=y-sty;
			r.w=get_text_width(tmp,font)+20;
			r.h=TTF_FontHeight(font);
			SDL_FillRect(sfc,&r,0x00ff0000);
		} /* if */ 
		print_left(tmp,font,255,255,255,sfc,10-stx,y-sty);
		y+=10;
	} /* while */ 

} /* VariableListWnd::draw_contens */ 

