#ifdef _WIN32
#include <windows.h>
#include <windowsx.h>
#else
#include <sys/time.h>
#include <time.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_mixer.h"
#include "SDL_sound.h"

#include "List.h"

#include "auxiliar.h"
#include "MOG2_auxiliar.h"
#include "reflection_effect.h"
#include "refraction_effect.h"

#include "variables.h"
#include "mog2.h"
#include "interfaces.h"

#include "BWindow.h"
#include "GVariableWnd.h"
#include "GMapWnd.h"
#include "GTopWnd.h"

#include "RVariableWnd.h"

#include "OTopWnd.h"

#include "Editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

/*						GLOBAL VARIABLES INITIALIZATION:							*/ 

int SCREEN_X=800;
int SCREEN_Y=600;
int COLOUR_DEPTH=32;
char *application_name="MoG2 Editor v0.93";
bool SOUND_ENABLED=true;

#ifdef _WIN32
bool fullscreen=false;
//int extra_flags=SDL_HWSURFACE|SDL_DOUBLEBUF;
int extra_flags=0;
int surface_flags=SDL_HWSURFACE;
// int surface_flags=0;

#else
bool fullscreen=false;
int extra_flags=0;
int surface_flags=0;
#endif

/* Redrawing constant: */ 
int REDRAWING_PERIOD=20;
int buffer_fps,act_fps,fps_timer;
int buffer_fps_d,fps_d;
int n_collisions=0;
int acum_collisions[100]={0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0,
						  0,0,0,0,0,0,0,0,0,0};

/* Surfaces: */ 
SDL_Surface *screen_sfc=0;

EDITOR *editor=0;
TTF_Font *font=0;


/*						AUXILIAR FUNCTION DEFINITION:							*/ 

SDL_Surface* initializeSDL(int moreflags)
{
	SDL_Surface *screen;

	//int flags = SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_HWPALETTE;
	int flags = extra_flags|moreflags;
	if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_JOYSTICK)<0) return 0;
	atexit(SDL_Quit);
	SDL_WM_SetCaption(application_name, 0);
	SDL_ShowCursor(SDL_DISABLE);
	SDL_EnableUNICODE(1);

	screen = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, flags);

	if (SOUND_ENABLED) {
		char SoundcardName[256];
		int audio_rate = 44100;
		int audio_channels = 2;
		int audio_bufsize = 2048;
		Uint16 audio_format = AUDIO_S16;
		SDL_version compile_version;

		fprintf (stderr, "Initializing SDL_mixer.\n");
		if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_bufsize)) {
			fprintf (stderr, "Unable to open audio: %s\n", Mix_GetError());
			return 0;
		} /* if */ 

		SDL_AudioDriverName (SoundcardName, sizeof (SoundcardName));
		Mix_QuerySpec (&audio_rate, &audio_format, &audio_channels);
		fprintf (stderr, "    opened %s at %d Hz %d bit %s, %d bytes audio buffer\n",
				 SoundcardName, audio_rate, audio_format & 0xFF,
				 audio_channels > 1 ? "stereo" : "mono", audio_bufsize);
		MIX_VERSION (&compile_version);
		fprintf (stderr, "    compiled with SDL_mixer version: %d.%d.%d\n",
				 compile_version.major,
				 compile_version.minor,
				 compile_version.patch);
		fprintf (stderr, "    running with SDL_mixer version: %d.%d.%d\n",
				 Mix_Linked_Version()->major,
				 Mix_Linked_Version()->minor,
				 Mix_Linked_Version()->patch);

		Sound_Init();
	}

	TTF_Init();

	reflection_init();

	return screen;
} /* initializeSDL */ 


void finalizeSDL()
{
	reflection_init();

	if (SOUND_ENABLED) {
		Sound_Quit();
		Mix_CloseAudio();
	} /* if */ 
	TTF_Quit();
	SDL_Quit();
} /* finalizeSDL */ 



#ifdef _WIN32
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPSTR lpCmdLine, int nCmdShow)
{
#else
int main(int argc, char** argv)
{
#endif

	int time,act_time;
	SDL_Event event;
    bool quit = false;

	screen_sfc = initializeSDL((fullscreen ? SDL_FULLSCREEN : 0));
	if (screen_sfc==0) return 0;
	buffer_fps=act_fps=buffer_fps_d=fps_d=0;
	fps_timer=time=SDL_GetTicks();

	font=TTF_OpenFont("fonts/lucon.ttf",10);

	editor=new EDITOR(font);

	while (!quit) {
		while( SDL_PollEvent( &event ) ) {
			bool captured=false;

            switch( event.type ) {
                /* Keyboard event */ 
                case SDL_KEYDOWN:
					if (event.key.keysym.sym==SDLK_F12) {
						captured=true;
						quit = true;
					} /* if */ 

					if (event.key.keysym.sym==SDLK_RETURN) {
						SDLMod modifiers=SDL_GetModState();

						captured=true;

						if ((modifiers&KMOD_ALT)!=0) {
							/* Toogle FULLSCREEN mode: */ 
							if (fullscreen) fullscreen=false;
									   else fullscreen=true;
//							SDL_QuitSubSystem(SDL_INIT_VIDEO);
//							SDL_InitSubSystem(SDL_INIT_VIDEO);
//							if (SDL_WasInit(SDL_INIT_VIDEO)) {
								screen_sfc = SDL_SetVideoMode(SCREEN_X, SCREEN_Y, COLOUR_DEPTH, (fullscreen ? SDL_FULLSCREEN : 0)|extra_flags);
								SDL_WM_SetCaption(application_name, 0);
								SDL_ShowCursor(SDL_DISABLE);
								SDL_EnableUNICODE(1);
//							} else {
//								quit = true;
//							} /* if */ 
						} /* if */ 
					} /* if */ 
                    break;

                /* SDL_QUIT event (window close) */ 
                case SDL_QUIT:
                    quit = true;
                    break;
            } /* switch */ 

			if (!captured) {
				if (!editor->event(&event)) quit=true;
			} /* if */ 
        } /* while */ 

		act_time=SDL_GetTicks();
		if (act_time-time>=REDRAWING_PERIOD)
		{
			act_fps++;
			if ((act_time-fps_timer)>1000) {
				fps_timer=act_time;
				buffer_fps=act_fps;
				buffer_fps_d=fps_d;
				act_fps=0;
				fps_d=0;
			} /* if */ 

			time+=REDRAWING_PERIOD;
			if ((act_time-time)>8*REDRAWING_PERIOD) time=act_time;

			if (!editor->cycle()) {
				quit=true;
			} /* if */ 
		}

		if (!quit) {
			fps_d++;
			editor->redraw(screen_sfc);
			SDL_Flip(screen_sfc);
		} /* if */ 
	}

	delete editor;
	editor=0;
	TTF_CloseFont(font);
	font=0;
	finalizeSDL();

	return 0;
}


